package com.starbuds.server.service.activity.dao;

import com.starbuds.server.common.pojo.activity.ActivityInfo;
import com.starbuds.server.common.pojo.activity.ActivityItem;
import com.starbuds.server.common.pojo.activity.ActivityVO;
import com.starbuds.server.common.pojo.db.ActivityDO;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Set;

/**
 * 备注
 *
 * @author Lay
 * @date 2020/2/22
 */
@Mapper
@Repository
public interface ActivityDAO {
    /**
     * 根据 id 查询
     *
     * @param activityId
     * @return
     */
    ActivityDO selectById(@Param("activityId") Long activityId);

    /**
     * 查询直播间活动
     *
     * @param curTime
     * @param moduleType
     * @return
     */
    List<ActivityDO> selectLiveActivityList(@Param("curTime") long curTime, @Param("moduleType") int moduleType);

    /**
     * 根据活动名查询
     *
     * @param activityType
     * @return
     */
    ActivityVO selectByType(@Param("activityType") Integer activityType);

    /**
     * 判断活动id是否存在
     *
     * @param activityId
     * @return
     */
    int countById(Long activityId);

    /**
     * 根据id查询
     *
     * @param activityId
     * @return
     */
    ActivityInfo selectInfo(Long activityId);

    /**
     * 根据活动类型获取进行中的活动
     *
     * @param activityType
     * @param curTime
     * @return
     */
    List<ActivityDO> selectAllByType(@Param("activityType") int activityType, @Param("curTime") long curTime);


    /**
     * 根据活动类型获取进行中的活动
     *
     * @param activityTypes
     * @param curTime
     * @return
     */
    List<ActivityDO> selectAllByTypes(@Param("activityTypes") List<Integer> activityTypes, @Param("curTime") long curTime);

    /**
     * 获取活动
     *
     * @param activityIds
     * @return
     */
    List<ActivityItem> selectByIds(@Param("activityIds") Set<Long> activityIds);

    /**
     * 通过ID获取活动列表并强制按照ID排序
     *
     * @param activityIds
     * @return
     */
    List<ActivityDO> getActivityList(@Param("activityIds") List<Long> activityIds);

    /**
     * 通过ID获取
     *
     * @param activityId
     * @return
     */
    ActivityDO getActivityById(@Param("activityId") Long activityId);

    /**
     * 根据类型获取所有正在进行的活动
     *
     * @param activityType
     * @param promiseTime
     * @return
     */
    List<ActivityDO> getRuningActivityByType(@Param("activityType") int activityType, @Param("promiseTime") Long promiseTime);

    /**
     * 根据类型获取所有正在进行的活动
     *
     * @param activityTypes
     * @param promiseTime
     * @return
     */
    List<ActivityDO> getActivityListByTypes(@Param("activityTypes") List<Integer> activityTypes, @Param("promiseTime") Long promiseTime);
}
