package com.starbuds.server.service.activity.dao;

import com.starbuds.server.common.pojo.db.ActivityGiftsRankBindDO;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;

import org.apache.ibatis.annotations.MapKey;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Mapper;
import org.springframework.stereotype.Repository;

import java.util.Collection;
import java.util.List;
import java.util.Map;

/**
 * 多礼物活动关系榜单，按天分隔。赠送人+收礼人当天只有一条记录 Mapper
 */
@Mapper
@Repository
public interface ActivityGiftsRankBindDAO {

    /**
     * 插入数据
     */
    int insert(ActivityGiftsRankBindDO po);

    /**
     * 插入批量数据
     */
    int insertBatch(List<ActivityGiftsRankBindDO> list);

    /**
     * 插入或更新数据
     */
    int insertOrUpdate(ActivityGiftsRankBindDO po);

    /**
     * 更新数据
     */
    int update(UpdateWrapper uw);

    /**
     * 查询一条记录
     */
    ActivityGiftsRankBindDO selectOne(QueryWrapper qw);

    /**
     * 查询所有记录
     */
    ActivityGiftsRankBindDO selectList(QueryWrapper qw);

    /**
     * 查询总数
     */
    int selectCount(QueryWrapper qw);

    /**
     * 查询列表(分页)
     */
    List<ActivityGiftsRankBindDO> selectPageList(QueryWrapper qw);

    /**
     * 根据ids获得 DO Map
     *
     * @param ids id集合
     * @return Map<主键, DO对象>
     */
    @MapKey("activityGiftsRankBindId")
    Map<Long, ActivityGiftsRankBindDO> selectMapByIds(@Param("ids") Collection ids);

    /**
     * 排行榜(分页)
     */
    List<ActivityGiftsRankBindDO> selectRankPageList(QueryWrapper qw);

    /**
     * 总排行榜(分页)
     */
    List<ActivityGiftsRankBindDO> selectTotalRankPageList(QueryWrapper qw);


    /**
     * 查询个人我的心动总榜(分页)
     */
    List<ActivityGiftsRankBindDO> selectTotalMyLikeRankPageList(QueryWrapper qw);


    /**
     * 查询个人心动我的总榜(分页)
     */
    List<ActivityGiftsRankBindDO> selectTotalLikeMeRankPageList(QueryWrapper qw);

    /**
     * 查询个人心动我的总榜数量
     */
    Integer selectTotalLikeMeRankCount(QueryWrapper qw);

    /**
     * 查询个人我的心动总榜数量
     */
    Integer selectTotalMyLikeRankCount(QueryWrapper qw);

    /**
     * 查询当日数据是否已存在
     */
    boolean isExistCurrentDay(QueryWrapper qw);

    /**
     * 查询两人之间的亲密值
     */
    Long selectRelatedTotalValue(QueryWrapper qw);

    /**
     * 查询个人某日排行榜总条数
     */
    Integer selectRankCount(QueryWrapper qw);


    /**
     * 更新total
     */
    int updateTotal(@Param("total") Long total,
                    @Param("activityId") Long activityId,
                    @Param("senderUserId") Long senderUserId,
                    @Param("receiveUserId") Long receiveUserId,
                    @Param("createDate") Long createDate,
                    @Param("timestamp") Long timestamp);

}