package com.starbuds.server.service.activity.dao;

import com.starbuds.server.common.pojo.activity.ActivityGrabPeriod;
import com.starbuds.server.common.pojo.db.ActivityGrabPeriodDO;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;
import org.apache.ibatis.annotations.Update;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * ActivityGrabPeriodDAO
 *
 * @author Lay
 * @date 2021/3/29
 */
@Mapper
@Repository
public interface ActivityGrabPeriodDAO {
    /**
     * 创建记录
     *
     * @param activityGrabPeriodDO
     * @return
     */
    int insert(ActivityGrabPeriodDO activityGrabPeriodDO);

    /**
     * 查询最新一期
     *
     * @param activityId
     * @return
     */
    ActivityGrabPeriodDO selectLatest(long activityId);

    /**
     * 根据 ID 查询
     *
     * @param periodId
     * @return
     */
    ActivityGrabPeriodDO selectById(long periodId);

    /**
     * 更新状态
     *
     * @param periodId
     * @param periodStatus
     * @return
     */
    @Update("UPDATE tb_activity_grab_period SET periodStatus = #{periodStatus}, openingTime = #{openingTime} WHERE periodId = #{periodId}")
    int updateStatus(@Param("periodId") long periodId, @Param("periodStatus") int periodStatus, @Param("openingTime") long openingTime);

    /**
     * 根据标识统计
     *
     * @param periodId
     * @return
     */
    @Select("SELECT COUNT(0) FROM tb_activity_grab_period WHERE periodId = #{periodId}")
    int countById(long periodId);

    /**
     * 查询用户参与的期数列表
     *
     * @param activityId
     * @param userId
     * @param pageCursor
     * @param pageSize
     * @return
     */
    List<ActivityGrabPeriod> selectUserPeriods(@Param("activityId") long activityId, @Param("userId") long userId, @Param("pageCursor") int pageCursor, @Param("pageSize") int pageSize);

    /**
     * 统计用户参与的期数列表
     *
     * @param activityId
     * @param userId
     * @return
     */
    int countUserPeriods(@Param("activityId") long activityId, @Param("userId") long userId);
}
