package com.starbuds.server.service.activity.dao;

import com.starbuds.server.common.pojo.activity.ActivityLotteryHisVO;
import com.starbuds.server.common.pojo.activity.ActivityLotteryRankItem;
import com.starbuds.server.common.pojo.db.ActivityLotteryHisDO;
import com.starbuds.server.common.pojo.db.ActivityStatRankDO;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * 备注
 *
 * @author Lay
 * @date 2020/2/20
 */
@Mapper
@Repository
public interface ActivityLotteryHisDAO {
    /**
     * 创建记录
     *
     * @param activityLotteryHisDO
     * @return
     */
    int insert(ActivityLotteryHisDO activityLotteryHisDO);

    /**
     * 查询抽奖记录列表
     *
     * @param pageCursor
     * @param pageSize
     * @param activityId
     * @param lotteryId
     * @param userId
     * @return
     */
    List<ActivityLotteryHisVO> selectAll(@Param("pageCursor") int pageCursor,
                                         @Param("pageSize") int pageSize,
                                         @Param("activityId") Long activityId,
                                         @Param("lotteryId") String lotteryId,
                                         @Param("userId") Long userId);

    /**
     * 统计抽奖记录
     *
     * @param activityId
     * @param lotteryId
     * @param userId
     * @return
     */
    int count(@Param("activityId") Long activityId, @Param("lotteryId") String lotteryId, @Param("userId") Long userId);

    /**
     * 根据 id 统计
     *
     * @param hisId
     * @param userId
     * @return
     */
    int countById(@Param("hisId") Long hisId, @Param("userId") Long userId);

    /**
     * 查询排行榜
     *
     * @param lotteryId
     * @param beginTime
     * @param endTime
     * @param limit
     * @return
     */
    List<ActivityLotteryRankItem> selectRankItems(@Param("lotteryId") String lotteryId, @Param("beginTime") long beginTime, @Param("endTime") long endTime, @Param("limit") int limit);

    /**
     * 查询排行榜
     *
     * @param lotteryId
     * @param beginTime
     * @param endTime
     * @param limit
     * @return
     */
    List<ActivityLotteryRankItem> selectRewardRankItems(@Param("lotteryId") String lotteryId, @Param("beginTime") long beginTime, @Param("endTime") long endTime, @Param("limit") int limit);

    /**
     * 获取抽奖花费
     *
     * @param activityId
     * @param userId
     * @param beginTime
     * @param endTime
     * @return
     */
    ActivityStatRankDO getMySelfRank(@Param("activityId") Long activityId, @Param("userId") Long userId, @Param("beginTime") Long beginTime, @Param("endTime") Long endTime);

    /**
     * 获取抽奖花费
     *
     * @param activityId
     * @param beginTime
     * @param endTime
     * @return
     */
    List<ActivityStatRankDO> getOctoberRank(@Param("activityId") Long activityId, @Param("beginTime") Long beginTime, @Param("endTime") Long endTime);
}
