package com.starbuds.server.service.activity.dao;

import com.starbuds.server.common.pojo.activity.LotteryHisRewardVO;
import com.starbuds.server.common.pojo.db.ActivityLotteryHisRewardsDO;
import org.apache.ibatis.annotations.MapKey;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 备注
 *
 * @author Lay
 * @date 2020/2/20
 */
@Mapper
@Repository
public interface ActivityLotteryHisRewardsDAO {
    /**
     * 批量插入
     *
     * @param rewardsDOList
     * @return
     */
    int batchInsert(List<ActivityLotteryHisRewardsDO> rewardsDOList);

    /**
     * 查询中奖记录
     *
     * @param lotteryId
     * @param num
     * @return
     */
    List<LotteryHisRewardVO> selectHisRewards(@Param("lotteryId") String lotteryId, @Param("num") int num);


    /**
     * 查询中奖纪录
     *
     * @param activityId
     * @param num
     * @return
     */
    List<LotteryHisRewardVO> selectHisRewardsByActivityId(@Param("activityId") Long activityId, @Param("num") int num);

    /**
     * 查询中奖记录
     *
     * @param hisId
     * @param userId
     * @return
     */
    List<LotteryHisRewardVO> selectHisRewardsByHisId(@Param("hisId") Long hisId, @Param("userId") Long userId);

    /**
     * 批量查询中奖记录
     *
     * @param ids
     * @return
     */
    List<ActivityLotteryHisRewardsDO> selectHisRewardsMapByHisIds(@Param("ids") Collection ids);

    /**
     * 查询中奖记录
     *
     * @param pageCursor
     * @param pageSize
     * @param activityId
     * @param lotteryId
     * @param userId
     * @return
     */
    List<LotteryHisRewardVO> selectAll(@Param("pageCursor") int pageCursor,
                                       @Param("pageSize") int pageSize,
                                       @Param("activityId") Long activityId,
                                       @Param("lotteryId") String lotteryId,
                                       @Param("userId") Long userId);

    /**
     * 统计中奖记录
     *
     * @param activityId
     * @param lotteryId
     * @param userId
     * @return
     */
    int count(@Param("activityId") Long activityId,
              @Param("lotteryId") String lotteryId,
              @Param("userId") Long userId);
}
