package com.starbuds.server.service.activity.dao;

import com.starbuds.server.common.pojo.db.LotteryGameConfigDO;
import com.starbuds.server.common.pojo.db.LotteryGameSnapshotDO;
import com.starbuds.server.common.pojo.db.LotteryGameTimerDO;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;
import org.apache.ibatis.annotations.Update;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * 抽奖活动
 *
 * @author aoe
 * @date 2021/11/23
 */
@Mapper
@Repository
public interface LotteryGameDAO {

    /**
     * 查询抽奖游戏配置
     *
     * @param configId
     * @return
     */
    @Select("SELECT configId,configName,playType,notice " +
            "FROM tb_lottery_game_config " +
            "WHERE configId = #{configId} " +
            "AND isDeleted = 0")
    LotteryGameConfigDO selectLotteryGameConfig(long configId);

    /**
     * 查询抽奖定时器配置
     *
     * @param timerId
     * @param status  LotteryGameTimerStatusEnum
     * @return
     */
    @Select("SELECT timerId,lotteryNo,playType,lotteryGameConfigItemId,totalValue,stockNum,joinNum,rewardNum," +
            "giveRewardTotal,leftTotal,STATUS,startTime,endTime " +
            "FROM tb_lottery_game_timer " +
            "WHERE timerId = #{timerId} " +
            "AND status = #{status} " +
            "AND isDeleted = 0")
    LotteryGameTimerDO selectLotteryGameTimer(@Param("timerId") long timerId, @Param("status") int status);

    /**
     * 获得可以发送倒计时通知的id
     *
     * @return
     */
    @Select("SELECT timerId " +
            "FROM tb_lottery_game_timer " +
            "WHERE status = #{status} " +
            "AND isDeleted = 0 " +
            "AND FROM_UNIXTIME(startTime / 1000, '%Y-%m-%d %H:%i:%s') = #{current} " +
            "LIMIT 1")
    Long selectSendCountdownMsgTimerId(@Param("current") String current,
                                       @Param("status") int status);

    /**
     * 更新抽奖定时器配置状态为进行中
     *
     * @param timerId
     * @param inProgress
     * @param standby
     * @return
     */
    @Update("UPDATE tb_lottery_game_timer " +
            "SET status = #{inProgress} " +
            "WHERE timerId = #{timerId} " +
            "AND status = #{standby}")
    int updateTimer2InProgress(@Param("timerId") long timerId, @Param("inProgress") int inProgress, @Param("standby") int standby);

    /**
     * 更新抽奖定时器配置状态为已结束
     *
     * @param timerId
     * @param over
     * @param inProgress
     * @return
     */
    @Update("UPDATE tb_lottery_game_timer " +
            "SET status = #{over}, endTime = #{endTime} " +
            "WHERE timerId = #{timerId} " +
            "AND status = #{inProgress}")
    int updateTimer2Over(@Param("timerId") long timerId,
                         @Param("over") int over,
                         @Param("inProgress") int inProgress,
                         @Param("endTime") long endTime);

    /**
     * 查询活动是否可以结束
     *
     * @param timerId
     * @param inProgress
     * @return
     */
    @Select("SELECT 1 " +
            "FROM tb_lottery_game_timer " +
            "WHERE timerId = #{timerId} " +
            "AND status = #{inProgress} " +
            "AND isDeleted = 0 " +
            "LIMIT 1")
    Long selectCanTimer2Over(@Param("timerId") long timerId,
                             @Param("inProgress") int inProgress);
}
