package com.starbuds.server.service.activity.dao;

import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;
import com.starbuds.server.common.pojo.db.LotteryGameSnapshotDO;
import org.apache.ibatis.annotations.MapKey;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;
import org.springframework.stereotype.Repository;

import java.util.Collection;
import java.util.List;
import java.util.Map;

/**
 * 抽奖游戏活动执行快照 Mapper
 */
@Mapper
@Repository
public interface LotteryGameSnapshotDAO {

    /**
     * 插入数据
     */
    int insert(LotteryGameSnapshotDO po);

    /**
     * 插入批量数据
     */
    int insertBatch(List<LotteryGameSnapshotDO> list);

    /**
     * 插入或更新数据
     */
    int insertOrUpdate(LotteryGameSnapshotDO po);

    /**
     * 更新数据
     */
    int update(UpdateWrapper uw);

    /**
     * 查询一条记录
     */
    LotteryGameSnapshotDO selectOne(QueryWrapper qw);

    /**
     * 查询所有记录
     */
    List<LotteryGameSnapshotDO> selectList(QueryWrapper qw);

    /**
     * 查询总数
     */
    int selectCount(QueryWrapper qw);

    /**
     * 查询列表(分页)
     */
    List<LotteryGameSnapshotDO> selectPageList(QueryWrapper qw);

    /**
     * 根据ids获得 DO Map
     *
     * @param ids id集合
     * @return Map<主键, DO对象>
     */
    @MapKey("snapId")
    Map<Long, LotteryGameSnapshotDO> selectMapByIds(@Param("ids") Collection ids);

    /**
     * 根据ids查询列表
     *
     * @param ids
     * @return
     */
    List<LotteryGameSnapshotDO> selectListByIds(@Param("ids") Collection ids);

    /**
     * 查找timerId是否已经存在
     *
     * @param timerIds
     * @return
     */
    List<Long> selectTimerIds(@Param("ids") Collection timerIds);

    /**
     * 正在进行中的游戏快照列表
     *
     * @param timerIds
     * @return
     */
    List<LotteryGameSnapshotDO> selectInProgressingList(@Param("ids") Collection timerIds);

    /**
     * 查询抽奖游戏活动摘要信息
     *
     * @param snapId
     * @return
     */
    @Select("SELECT itemName,costType,costItem,costValue,iconUrl,clickRate,rewardPool,startTime, playType " +
            "FROM tb_lottery_game_snapshot " +
            "WHERE snapId = #{snapId} ")
    LotteryGameSnapshotDO selectLotteryGameSummary(@Param("snapId") long snapId);

    /**
     * 查询奖池配置
     *
     * @param snapId
     * @return
     */
    @Select("SELECT rewardPool " +
            "FROM tb_lottery_game_snapshot " +
            "WHERE snapId = #{snapId} ")
    LotteryGameSnapshotDO selectRewardPool(@Param("snapId") long snapId);

    /**
     * 根据发送倒计时通知时间获得最近的快照列表
     *
     * @param current 近似当前时间
     * @return
     */
    @Select("SELECT snapId,sendNoticeTime " +
            "FROM tb_lottery_game_snapshot " +
            "WHERE FROM_UNIXTIME(sendNoticeTime / 1000, '%Y-%m-%d %H:%i:%s') = #{current} ")
    List<LotteryGameSnapshotDO> selectListBySendNoticeTime(@Param("current") String current);

    /**
     * 根据活动开始时间获得最近的快照列表
     *
     * @param current 近似当前时间
     * @return
     */
    @Select("SELECT snapId,startTime,minEndTime,maxEndTime " +
            "FROM tb_lottery_game_snapshot " +
            "WHERE FROM_UNIXTIME(startTime / 1000, '%Y-%m-%d %H:%i:%s') = #{current} ")
    List<LotteryGameSnapshotDO> selectListByStartTime(@Param("current") String current);

    /**
     * 查询玩法类型
     *
     * @param snapId
     * @return
     */
    @Select("SELECT playType " +
            "FROM tb_lottery_game_snapshot " +
            "WHERE snapId = #{snapId} ")
    int selectPlayType(@Param("snapId") long snapId);

    /**
     * 查询timerId
     *
     * @param snapId
     * @return
     */
    @Select("SELECT timerId " +
            "FROM tb_lottery_game_snapshot " +
            "WHERE snapId = #{snapId} ")
    Long selectTimerIdBySnapId(@Param("snapId") long snapId);

    @Select("SELECT snapId,startTime,maxEndTime " +
            "FROM tb_lottery_game_snapshot " +
            "WHERE startTime > #{now} and sendNoticeTime < #{now} ")
    List<LotteryGameSnapshotDO> getWaitLotteryGameList(@Param("now") Long now);
}