package com.starbuds.server.service.activity.dao;

import com.starbuds.server.common.pojo.db.LotteryGameWinHisDO;
import org.apache.ibatis.annotations.Insert;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * 抽奖游戏的中奖记录
 *
 * @author aoe
 * @date 2021/11/23
 */
@Mapper
@Repository
public interface LotteryGameWinHisDAO {

    /**
     * 查询抽奖游戏的中奖记录列表
     *
     * @param snapId
     * @param limit
     * @return
     */
    @Select("SELECT userId,rewardId,rewardType,rewardNum,rewardTotalPrice " +
            "FROM tb_lottery_game_win_his " +
            "WHERE snapId = #{snapId} " +
            "ORDER BY createTime DESC LIMIT #{limit}")
    List<LotteryGameWinHisDO> selectListBySnapId(@Param("snapId") long snapId, @Param("limit") int limit);

    @Select("SELECT COUNT(DISTINCT userId) " +
            "FROM tb_lottery_game_win_his " +
            "WHERE snapId = #{snapId} ")
    Integer selectWinUserCountBySnapId(@Param("snapId") long snapId);

    /**
     * 查询抽奖游戏的中奖记录列表
     *
     * @param snapId
     * @param userId
     * @param limit
     * @return
     */
    @Select("SELECT userId,rewardId,rewardType,rewardNum,rewardTotalPrice " +
            "FROM tb_lottery_game_win_his " +
            "WHERE snapId = #{snapId} " +
            "AND userId = #{userId} " +
            "ORDER BY createTime DESC " +
            "LIMIT #{limit}")
    List<LotteryGameWinHisDO> selectListBySnapIdUserId(@Param("snapId") long snapId,
                                                       @Param("userId") long userId,
                                                       @Param("limit") int limit);

    @Select("SELECT DISTINCT userId " +
            "FROM tb_lottery_game_win_his " +
            "WHERE snapId = #{snapId} ")
    List<Long> selectUserIds(@Param("snapId") long snapId);

    @Select("SELECT SUM(rewardTotalPrice) " +
            "FROM tb_lottery_game_win_his " +
            "WHERE snapId = #{snapId} ")
    Long selectRewardTotalPrice(@Param("snapId") long snapId);

    /**
     * 插入数据
     */
    @Insert("INSERT INTO tb_lottery_game_win_his " +
            "(hisId,snapId,playType,userId,rewardId,rewardName,rewardType,rewardNum,rewardPrice,rewardTotalPrice,createTime) " +
            "VALUES " +
            "(#{hisId},#{snapId},#{playType},#{userId},#{rewardId},#{rewardName},#{rewardType},#{rewardNum},#{rewardPrice},#{rewardTotalPrice},#{createTime})")
    int insert(LotteryGameWinHisDO po);
}
