package com.starbuds.server.service.activity.dao;

import com.starbuds.server.common.pojo.db.UserActivityScoreDO;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;

import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * 用户积分表 Mapper
 */
@Mapper
@Repository
public interface UserActivityScoreDAO {

    /**
     * 插入数据
     */
    int insert(UserActivityScoreDO po);

    /**
     * 插入批量数据
     */
    int insertBatch(List<UserActivityScoreDO> list);

    /**
     * 插入或更新数据
     */
    int insertOrUpdate(UserActivityScoreDO po);

    /**
     * 更新数据
     */
    int update(UpdateWrapper uw);

    /**
     * 查询一条记录
     */
    UserActivityScoreDO selectOne(QueryWrapper qw);

    /**
     * 查询所有记录
     */
    UserActivityScoreDO selectList(QueryWrapper qw);

    /**
     * 查询总数
     */
    int selectCount(QueryWrapper qw);

    /**
     * 查询列表(分页)
     */
    List<UserActivityScoreDO> selectPageList(QueryWrapper qw);


    /**
     * 巅峰家族排行榜
     */
    List<UserActivityScoreDO> selectPeakFamilyRankList(@Param("activityId") Long activityId);

    /**
     * 巅峰家族总排行榜
     */
    List<UserActivityScoreDO> selectPeakFamilyAllRankList(@Param("activityId") Long activityId);


    /**
     * 巅峰家族积分
     */
    Long selectPeakFamilyScore(@Param("activityId") Long activityId, @Param("familyId") Long familyId);


    /**
     * 巅峰家族个人积分
     */
    Long selectPeakFamilyPersonScore(@Param("activityId") Long activityId, @Param("familyId") Long familyId, @Param("userId") Long userId);

    /**
     * 巅峰家族获奖的用户
     */
    List<UserActivityScoreDO> selectPeakFamilyRewardUserList(@Param("activityId") Long activityId, @Param("familyId") Long familyId);

    /**
     * 清除用户在家族的积分
     */
    void updateScoreDel(@Param("activityId") Long activityId, @Param("familyId") Long familyId, @Param("userId") Long userId);

    /**
     * 个人积分排行
     *
     * @param activityId
     * @param userId
     * @return
     */
    Long getUserScore(@Param("activityId") Long activityId, @Param("userId") Long userId);

    /**
     * 获取临近分数
     *
     * @param activityId
     * @param score
     * @return
     */
    UserActivityScoreDO getUserNextScore(@Param("activityId") Long activityId, @Param("score") Long score);

    /**
     * 获取人物总积分的排行
     *
     * @param activityId
     * @param limit
     * @return
     */
    List<UserActivityScoreDO> getInfluentialRankList(@Param("activityId") Long activityId, @Param("limit") Integer limit);


    List<UserActivityScoreDO> getRoomRankList(@Param("activityId") Long activityId, @Param("limit") Integer limit);

    /**
     * 获取房间分数
     *
     * @param activityId
     * @param roomId
     * @return
     */
    Long getRoomScore(@Param("activityId") Long activityId, @Param("roomId") Long roomId);

    /**
     * @param activityId
     * @param score
     * @return
     */
    UserActivityScoreDO getRoomNextScore(@Param("activityId") Long activityId, @Param("score") Long score);

    int getUserScoreCount(@Param("activityId") Long activityId, @Param("userId") Long userId,
                          @Param("familyId") Long familyId, @Param("roomId") Long roomId);

    void updateScore(@Param("activityId") Long activityId, @Param("userId") Long userId,
                     @Param("familyId") Long familyId, @Param("roomId") Long roomId, @Param("score") Long score);
}
