package com.starbuds.server.service.activity.events.listener;

import com.alibaba.fastjson.JSON;
import com.google.common.collect.ImmutableList;
import com.google.common.collect.Lists;
import com.starbuds.server.common.constant.cache.RedisActivityKeys;
import com.starbuds.server.common.pojo.activity.bo.LotteryGameRewardSummary;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.service.activity.events.event.LotteryGameCleanCacheEvent;
import com.starbuds.server.service.activity.events.message.LotteryGameCleanCacheMsg;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.context.ApplicationListener;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.IntStream;

/**
 * @author aoe
 * @date 2021/12/30
 */
@Component
@Slf4j
public class LotteryGameCleanCacheListener implements ApplicationListener<LotteryGameCleanCacheEvent> {

    @Reference(version = "1.0.0")
    private RedisService redisService;

    @Override
    public void onApplicationEvent(LotteryGameCleanCacheEvent event) {
        LotteryGameCleanCacheMsg msg = event.getMsg();
        if (msg == null) {
            return;
        }
        final long snapId = msg.getSnapId();
        final List<Long> winUserIds = msg.getWinUserIds();
        final List<LotteryGameRewardSummary> rewardSummaries = msg.getRewardSummaries();
        cleanCache(snapId, winUserIds, rewardSummaries);
    }

    private void cleanCache(final long snapId, final List<Long> winUserIds, final List<LotteryGameRewardSummary> rewardSummaries) {
        log.info("清除抽奖游戏缓存开始 snapId={} winUserIds size={} rewardSummaries size={}", snapId, winUserIds.size(), rewardSummaries.size());
        List<String> keys = new ArrayList<>();
        keys.add(RedisActivityKeys.getLotteryGameWinItemsMap(snapId));
        keys.add(RedisActivityKeys.getLotteryGameRewardCount(snapId));
        keys.add(RedisActivityKeys.getLotteryGameRewardSummaries(snapId));

        rewardSummaries.stream()
                .forEach(summary -> {
                    String key = RedisActivityKeys.getLotteryGameRewardStock(snapId, summary.getRewardType(), summary.getRewardId());
                    keys.add(key);
                });

        if (!CollectionUtils.isEmpty(winUserIds)) {
            winUserIds.stream()
                    .forEach(userId -> {
                        String key = RedisActivityKeys.getLotteryGameWinHisItems(snapId, userId);
                        keys.add(key);
                    });
        }

        // 数量很大，最后删除
//        keys.add(RedisActivityKeys.getLotteryGameRewards(snapId));
        Lists.partition(keys, 100)
                .stream()
                .forEach(partitionKeys -> redisService.delKeys(partitionKeys));
        log.info("清除抽奖游戏缓存结束 snapId={} keys size={}", snapId, keys.size());
    }

}
