package com.starbuds.server.service.activity.grab;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.activity.ActivityGrabPeriodStatusEnum;
import com.starbuds.server.common.constant.activity.LotteryRewardTypeEnum;
import com.starbuds.server.common.pojo.activity.ActivityGrabItem;
import com.starbuds.server.common.pojo.data.PropsVO;
import com.starbuds.server.common.pojo.db.ActivityGrabConfigDO;
import com.starbuds.server.common.pojo.db.ActivityGrabPeriodDO;
import com.starbuds.server.common.pojo.media.GiftItemVO;
import com.starbuds.server.common.service.data.PropsService;
import com.starbuds.server.common.service.media.GiftService;
import com.starbuds.server.service.activity.dao.ActivityGrabConfigDAO;
import com.starbuds.server.service.activity.dao.ActivityGrabPeriodDAO;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

/**
 * 抢龙期数创建器
 *
 * @author Lay
 * @date 2021/3/29
 */
@Component
@Slf4j
public class GrabPeriodCreator {

    @Resource
    ActivityGrabConfigDAO activityGrabConfigDAO;

    @Resource
    ActivityGrabPeriodDAO activityGrabPeriodDAO;

    @Reference(version = "1.0.0")
    GiftService giftService;

    @Reference(version = "1.0.0")
    PropsService propsService;

    /**
     * 异步创建
     *
     * @param activityId
     */
    public void create(long activityId) {
        // 读取配置
        ActivityGrabConfigDO configDO = activityGrabConfigDAO.selectLatestEnabled(activityId);
        if (configDO == null) {
            log.warn("抢龙创建失败：没有可用的配置");
            return;
        }

        ActivityGrabPeriodDO periodDO = new ActivityGrabPeriodDO();
        periodDO.setActivityId(activityId);
        periodDO.setConfigId(configDO.getConfigId());
        periodDO.setConfigSnapshot(JSON.toJSONString(configDO));
        periodDO.setPeriodStatus(ActivityGrabPeriodStatusEnum.Going.getValue());
        periodDO.setCreateTime(System.currentTimeMillis());

        // 读取奖项
        int itemType = configDO.getItemType();
        LotteryRewardTypeEnum rewardType = LotteryRewardTypeEnum.toEnum(itemType);
        if (rewardType == null) {
            log.warn("抢龙创建失败：奖项配置无效");
            return;
        }

        ActivityGrabItem grabItem = new ActivityGrabItem();
        grabItem.setItemId(configDO.getItemId());
        grabItem.setItemType(configDO.getItemType());
        if (LotteryRewardTypeEnum.GIFT == rewardType) {
            GiftItemVO giftItemVO = giftService.getGift(configDO.getItemId());
            grabItem.setItemName(giftItemVO.getGiftName());
            grabItem.setItemIcon(giftItemVO.getGiftIcon());
        } else if (LotteryRewardTypeEnum.PROPS == rewardType) {
            PropsVO propsVO = propsService.getProps(configDO.getItemId());
            grabItem.setItemName(propsVO.getPropsName());
            grabItem.setItemIcon(propsVO.getPropsIcon());
        }

        periodDO.setPeriodItem(JSON.toJSONString(grabItem));

        int insert = activityGrabPeriodDAO.insert(periodDO);
        log.info("创建抢龙活动 {}", insert);
    }
}
