package com.starbuds.server.service.activity.grab;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.activity.ActivityGrabPeriodStatusEnum;
import com.starbuds.server.common.constant.activity.LotteryRewardTypeEnum;
import com.starbuds.server.common.constant.cache.RedisActivityKeys;
import com.starbuds.server.common.constant.finance.BagItemTypeEnum;
import com.starbuds.server.common.constant.message.im.ImTargetType;
import com.starbuds.server.common.pojo.activity.ActivityGrabItem;
import com.starbuds.server.common.pojo.activity.GrabPlayerWinQuantity;
import com.starbuds.server.common.pojo.db.ActivityGrabConfigDO;
import com.starbuds.server.common.pojo.db.ActivityGrabPeriodDO;
import com.starbuds.server.common.pojo.db.ActivityGrabPeriodItemDO;
import com.starbuds.server.common.pojo.message.im.activity.GrabSettleMsg;
import com.starbuds.server.common.pojo.user.UserCardVO;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.finance.BagService;
import com.starbuds.server.common.service.finance.WalletService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.service.activity.dao.ActivityGrabPeriodDAO;
import com.starbuds.server.service.activity.dao.ActivityGrabPeriodItemDAO;
import com.starbuds.server.service.activity.redpacket.RandomStrategy;
import com.starbuds.server.service.activity.redpacket.RedPacketsConfig;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 抢龙结算服务
 *
 * @author Lay
 * @date 2021/3/29
 */
@Component
@Slf4j
public class GrabSettleProvider {
    @Resource
    ActivityGrabPeriodDAO activityGrabPeriodDAO;

    @Resource
    ActivityGrabPeriodItemDAO activityGrabPeriodItemDAO;

    @Resource
    GrabPeriodCreator grabPeriodCreator;

    @Reference(version = "1.0.0")
    UserService userService;

    @Reference(version = "1.0.0")
    ImService imService;

    @Reference(version = "1.0.0")
    WalletService walletService;

    @Reference(version = "1.0.0")
    BagService bagService;

    @Reference(version = "1.0.0")
    RedisService redisService;

    @Async
    @Transactional(rollbackFor = Exception.class)
    public void settleAsync(long periodId, List<Long> userIds) {
        // 获取配置
        ActivityGrabPeriodDO periodDO = activityGrabPeriodDAO.selectById(periodId);
        log.info("抢龙结算 开始 periodId = {} userIds ={} periodDO = {}", periodId, userIds, periodDO);

        // 判断存在
        if (periodDO == null) {
            return;
        }

        // 判断状态
        if (ActivityGrabPeriodStatusEnum.Going.getValue() != periodDO.getPeriodStatus()) {
            log.info("抢龙结算 结束 periodId = {} 状态不是进行中");
            return;
        }

        // 读取配置
        ActivityGrabConfigDO configDO = JSON.parseObject(periodDO.getConfigSnapshot(), ActivityGrabConfigDO.class);
        // 读取数量
        int itemsCount = configDO.getItemsCount();
        // 读取人数
        int playerNum = configDO.getPlayerNum();

        // 份数
        if (itemsCount < playerNum) {
            log.info("抢龙结算 结束 periodId = {} 物品数量 小于 玩家数量 itemsCount = {} playerNum = {}", itemsCount, playerNum);
            return;
        }

        // 读取玩家信息
        Map<Long, UserCardVO> userMap = userService.getNameAndAvatarByIds(userIds).stream().collect(Collectors.toMap(UserCardVO::getUserId, Function.identity()));

        // 随机拆份数
        RedPacketsConfig config = new RedPacketsConfig();
        config.setMinAvgMoney(BigDecimal.ONE);
        config.setMoneyScale(0);
        config.setDefaultScale(0);
        List<Integer> winData = getResults(config, itemsCount, playerNum);
        int min = winData.get(0);
        Collections.shuffle(winData);

        List<ActivityGrabPeriodItemDO> list = new ArrayList<>();
        List<GrabPlayerWinQuantity> winQuantityList = new ArrayList<>();
        Map<Long, Integer> userQuantity = new HashMap<>();
        long minUserId = 0;
        for (int i = 0; i < userIds.size(); i++) {
            int winQuantity = winData.get(i);
            Long userId = userIds.get(i);
            if (winQuantity == min) {
                minUserId = userId;
            }
            userQuantity.put(userId, winQuantity);

            ActivityGrabPeriodItemDO itemDO = new ActivityGrabPeriodItemDO();
            itemDO.setUserId(userId);
            itemDO.setWinQuantity(winQuantity);
            list.add(itemDO);

            UserCardVO userCard = userMap.get(userId);
            GrabPlayerWinQuantity playerWinQuantity = new GrabPlayerWinQuantity();
            playerWinQuantity.setUserId(userId);
            playerWinQuantity.setUserName(userCard.getUserName());
            playerWinQuantity.setUserAvatar(userCard.getUserAvatar());
            playerWinQuantity.setWinQuantity(winQuantity);
            winQuantityList.add(playerWinQuantity);
        }

        if (minUserId == 0) {
            log.info("抢龙结算 结束 没人中奖");
            return;
        }

        // 保存数据
        activityGrabPeriodItemDAO.updateWinQuantity(periodId, list);

        // 更新状态
        activityGrabPeriodDAO.updateStatus(periodId, ActivityGrabPeriodStatusEnum.Over.getValue(), System.currentTimeMillis());

        // 返还金额 发放物品
        ActivityGrabItem item = JSON.parseObject(periodDO.getPeriodItem(), ActivityGrabItem.class);
        for (Long userId : userIds) {
            BagItemTypeEnum bagItemType = null;
            // 发放物品
            if (LotteryRewardTypeEnum.GIFT.getValue() == item.getItemType()) {
                bagItemType = BagItemTypeEnum.GIFT;
            } else if (LotteryRewardTypeEnum.PROPS.getValue() == item.getItemType()) {
                bagItemType = BagItemTypeEnum.PROPS;
            }

            if (bagItemType != null) {
                bagService.addItem(userId, item.getItemId(), bagItemType.getValue(), userQuantity.get(userId), -1L, "抢龙活动获得", periodDO.getPeriodId());
                log.info("抢龙结算 发放奖励 userId = {} itemId = {} bagItemType = {} quantity = {}", userId, item.getItemId(), bagItemType.getValue(), userQuantity.get(userId));
            }

            if (minUserId == userId) {
                continue;
            }

            // 返还金额
            Long addCoin = walletService.addCoin(userId, configDO.getPriceType(), configDO.getPrice(), "预购金返还", periodId);
            log.info("抢龙结算 返还金额 userId = {} addCoin = {}", userId, addCoin);
        }

        // 移除缓存
//        redisService.hashDel(RedisActivityKeys.ACTIVITY_GRAB_PERIOD_NUMS, String.valueOf(periodDO.getPeriodId() - 100));

        try {
            redisService.delKey(RedisActivityKeys.getActivityPeriodidSeat(periodId - 100));
        } catch (Exception ex) {
            log.info("抢龙结算 删除缓存失败 key {}", RedisActivityKeys.getActivityPeriodidSeat(periodId - 100) + "");
        }


        // 推送结果
        GrabSettleMsg settleMsg = new GrabSettleMsg();
        settleMsg.setPeriodId(periodId);
        settleMsg.setWinQuantityList(winQuantityList);
        boolean sendRtcRoomMsg = imService.sendRtcRoomMsg(ImTargetType.GROUP, null, null, -1001L, settleMsg);
        log.info("抢龙结算 推送房间消息结果 {}", sendRtcRoomMsg);

        // 创建下一期活动
        grabPeriodCreator.create(periodDO.getActivityId());
        log.info("抢龙结算 结束");
    }

    private static List<Integer> getResults(RedPacketsConfig config, int itemsCount, int playerNum) {
        RandomStrategy randomStrategy = new RandomStrategy(config);
        while (true) {
            List<BigDecimal> results = randomStrategy.splitRedPackets(BigDecimal.valueOf(itemsCount), playerNum);
            List<Integer> arr = results.stream().map(BigDecimal::intValue).collect(Collectors.toList());
            Collections.sort(arr);
            if (arr.get(0).equals(arr.get(1))) {
                continue;
            }
            return arr;
        }
    }
}
