package com.starbuds.server.service.activity.headline;

import cn.hutool.core.convert.Convert;
import com.starbuds.server.common.constant.cache.RedisActivityKeys;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.message.ImService;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;

import java.util.Optional;

/**
 * 头条抽奖活动热度
 *
 * @author Lay
 * @date 2020/12/28
 */
@Component
public class HeadlineLotteryHeatProvider {

    @Reference(version = "1.0.0")
    RedisService redisService;

    @Reference(version = "1.0.0")
    ImService imService;

    private static final String ROOM_KEY = "headlineLotteryHeat";

    public void resetRoomHeat() {
        imService.setRoomProperty(0L, ROOM_KEY, "0");
    }

    /**
     * 增加热度
     *
     * @param configId
     * @param periodId
     * @param value
     * @return
     */
    public long add(long configId, int periodId, long value) {
        try {
            long heat = redisService.hashIncreaseLong(RedisActivityKeys.ACTIVITY_HEADLINE_LOTTERY_HEAT, getHeatKey(configId, periodId), value);

            // 同步热度
            imService.setRoomProperty(0L, ROOM_KEY, String.valueOf(heat));

            return heat;
        } catch (Exception e) {
            return 0;
        }
    }

    /**
     * 获取热度
     *
     * @param configId
     * @param periodId
     * @return
     */
    public long get(long configId, int periodId) {
        return Optional.ofNullable(redisService.hashGet(RedisActivityKeys.ACTIVITY_HEADLINE_LOTTERY_HEAT, getHeatKey(configId, periodId))).map(Convert::toLong).orElse(0L);
    }

    /**
     * 移除热度
     *
     * @param configId
     * @param period
     */
    public void remove(long configId, int period) {
        redisService.hashDel(RedisActivityKeys.ACTIVITY_HEADLINE_LOTTERY_HEAT, getHeatKey(configId, period));
    }

    private static String getHeatKey(long configId, int periodId) {
        return String.format("%d:%d", configId, periodId);
    }
}
