package com.starbuds.server.service.activity.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.activity.ActivityAwardTypeEnum;
import com.starbuds.server.common.constant.activity.ActivityRankStatusEnum;
import com.starbuds.server.common.constant.activity.ActivityRankTypeEnum;
import com.starbuds.server.common.constant.activity.ActivityTypeEnum;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.finance.AddItemToBagSourceEnum;
import com.starbuds.server.common.constant.finance.BagItemTypeEnum;
import com.starbuds.server.common.constant.message.im.ImTargetType;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rocketmq.RocketTopic;
import com.starbuds.server.common.pojo.activity.AssociatedRoomActivityItem;
import com.starbuds.server.common.pojo.activity.bo.AnnualGalaPeakFamilyRankBO;
import com.starbuds.server.common.pojo.activity.vo.*;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.pojo.db.*;
import com.starbuds.server.common.pojo.message.im.activity.AnnualGalaFinishMsg;
import com.starbuds.server.common.pojo.mq.QueueAddItemToBagMsg;
import com.starbuds.server.common.pojo.mq.QueueUserExitFamilyMsg;
import com.starbuds.server.common.pojo.rtc.RtcRoom;
import com.starbuds.server.common.service.activity.ActivityAnniversaryService;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.rtc.RtcRoomService;
import com.starbuds.server.common.service.user.FamilyService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.common.utils.DateUtil;
import com.starbuds.server.service.activity.dao.*;
import com.starbuds.server.service.activity.rocket.RocketProducer;
import com.starbuds.server.service.activity.util.SpringContextUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;
import org.junit.platform.commons.util.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.scheduling.annotation.Async;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.swing.text.StringContent;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author : lujiao
 * @date : 2021/12/7 15:01
 */
@Slf4j
@Service(version = "1.0.0")
public class ActivityAnniversaryServiceImpl implements ActivityAnniversaryService {

    @Resource
    private RtcRoomBorderDAO rtcRoomBorderDAO;

    @Reference(version = "1.0.0")
    IdentityService identityService;

    @Reference(version = "1.0.0")
    RtcRoomService rtcRoomService;

    @Reference(version = "1.0.0")
    UserService userService;

    @Resource
    private UserActivityScoreDAO userActivityScoreDAO;

    @Resource
    private UserActivityScoreDetailDAO userActivityScoreDetailDAO;

    @Resource
    private ActivityDAO activityDAO;

    @Resource
    private RtcRoomAdvanceDAO rtcRoomAdvanceDAO;

    @Reference(version = "1.0.0")
    private FamilyService familyService;

    @Resource
    private ActivityAnnualAwardConfigDAO awardConfigDAO;

    @Resource
    private ActivityAnnualAwardItemDAO awardItemDAO;

    @Resource
    private ActivityAnnualAwardGiveHisDAO giveHisDAO;

    @Resource
    private RocketProducer rocketProducer;

    @Reference(version = "1.0.0")
    private ImService imService;

    /**
     * 用户参加活动排名信息
     *
     * @param userId
     * @param activityId
     * @return
     */
    @Override
    public ActivityJoinRankVO getInfluentialRankInfo(Long userId, Long activityId, Integer rankType) {
        ActivityJoinRankVO res = new ActivityJoinRankVO();
        Long score = getUserScore(activityId, userId, rankType);
        ActivityDO activityDO = activityDAO.getActivityById(activityId);
        res.setConfig(JSON.parseObject(activityDO.getRankStyle(), ActivityEntranceConfig.class));
        UserDO userDO = userService.getUserById(userId);
        res.setStatus(1);
        res.setName(userDO.getUserName());
        if (score == null) {
            score = 0L;
            res.setRank(0);
            res.setMyselfScore(score);
        } else {
            res.setMyselfScore(score);
            res.setRank(getMyInfluentialRank(activityId, userId, rankType));
            if (res.getRank() > 0) {
                res.setStatus(ActivityRankStatusEnum.SUCCESS.getValue());
            }
        }
        UserActivityScoreDO nextScore = userActivityScoreDAO.getUserNextScore(activityId, score);
        if (nextScore != null) {
            res.setTopScore(nextScore.getScore());
        } else {
            res.setTopScore(score);
        }
        return res;
    }

    @Override
    public Integer getMyInfluentialRank(Long activityId, Long userId, Integer rankType) {
        List<ActivityAnniversaryRankVO> scoreList = getInfluentialRankList(activityId, rankType);
        int defaultRank = 1;
        if (CollectionUtil.isNotEmpty(scoreList)) {
            for (ActivityAnniversaryRankVO scoreDO : scoreList) {
                if (scoreDO.getId().equals(userId.toString())) {
                    return defaultRank;
                }
                defaultRank++;
            }
        }
        return 0;
    }


    /**
     * 获取用户分数
     *
     * @param activityId
     * @param userId
     * @param rankType
     * @return
     */
    @Override
    public Long getUserScore(Long activityId, Long userId, Integer rankType) {
        if (rankType == ActivityRankTypeEnum.TOTAL.getValue()) {
            return userActivityScoreDAO.getUserScore(activityId, userId);
        } else if (rankType == ActivityRankTypeEnum.DAY.getValue()) {
            Long dayBegin = DateUtil.getDayBegin();
            Long now = System.currentTimeMillis();
            return userActivityScoreDetailDAO.getUserScore(activityId, userId, dayBegin, now);
        } else if (rankType == ActivityRankTypeEnum.YESTERDAY.getValue()) {
            Long dayBegin = DateUtil.getDayBegin();
            Long dayEnd = dayBegin - 1;
            dayBegin = dayBegin - DateUtil.DAY_OF_MILLION;
            return userActivityScoreDetailDAO.getUserScore(activityId, userId, dayBegin, dayEnd);
        } else {
            // 其他类型暂时不支持
            return 0L;
        }
    }


    /**
     * 获取排行信息
     *
     * @param activityId
     * @return
     */
    @Override
    public List<ActivityAnniversaryRankVO> getInfluentialRankList(Long activityId, Integer rankType) {
        List<ActivityAnniversaryRankVO> res = new ArrayList<>();
        // 获取风云人物排行
        Integer limit = 30;
        if (rankType == ActivityRankTypeEnum.TOTAL.getValue()) {
            limit = 50;
        }
        List<UserActivityScoreDO> scoreList = null;
        if (rankType == ActivityRankTypeEnum.TOTAL.getValue()) {
            scoreList = userActivityScoreDAO.getInfluentialRankList(activityId, limit);
        } else if (rankType == ActivityRankTypeEnum.DAY.getValue()) {
            Long now = System.currentTimeMillis();
            Long dayBegin = DateUtil.getDayBegin();
            scoreList = userActivityScoreDetailDAO.getUserDayRankList(activityId, dayBegin, now, limit);
        } else if (rankType == ActivityRankTypeEnum.YESTERDAY.getValue()) {
            Long dayBegin = DateUtil.getDayBegin();
            Long dayEnd = dayBegin - 1;
            dayBegin = dayBegin - DateUtil.DAY_OF_MILLION;
            scoreList = userActivityScoreDetailDAO.getUserDayRankList(activityId, dayBegin, dayEnd, limit);
        }

        if (CollectionUtil.isNotEmpty(scoreList)) {
            int rank = 1;
            Set<Long> userIdsSets = new HashSet<>(scoreList.size());
            scoreList.forEach(i -> {
                userIdsSets.add(i.getUserId());
            });
            Map<Long, UserDO> userMap = userService.getUserMapByIds(userIdsSets);
            for (UserActivityScoreDO scoreDO : scoreList) {
                ActivityAnniversaryRankVO tmp = new ActivityAnniversaryRankVO();
                tmp.setId(String.valueOf(scoreDO.getUserId()));
                tmp.setRank(rank);
                rank++;
                tmp.setScore(scoreDO.getScore());
                UserDO tmpUser = userMap.get(scoreDO.getUserId());
                if (tmpUser != null) {
                    tmp.setName(tmpUser.getUserName());
                    tmp.setImgUrl(tmpUser.getUserAvatar());
                    tmp.setUserNo(String.valueOf(tmpUser.getUserNo()));
                }
                res.add(tmp);
            }
        }
        return res;
    }

    /**
     * 魅力花房参赛数据
     *
     * @param userId
     * @param activityId
     * @return
     */
    @Override
    public ActivityJoinRankVO getFlowerRoomRank(Long userId, Long activityId, Long roomId) {
        ActivityJoinRankVO res = new ActivityJoinRankVO();
        ActivityDO activityDO = activityDAO.getActivityById(activityId);
        RtcRoom room = rtcRoomService.getRoom(roomId);
        res.setName(room.getRoomName());
        AssociatedRoomActivityItem currentActivity = getCurrentActivity(activityDO);
        if (currentActivity != null) {
            res.setIconImg(currentActivity.getItemIcon());
            res.setConfig(JSON.parseObject(activityDO.getRankStyle(), ActivityEntranceConfig.class));
            Long score = userActivityScoreDAO.getRoomScore(currentActivity.getItemId(), roomId);
            List<UserActivityScoreDO> scoreList = userActivityScoreDAO.getRoomRankList(currentActivity.getItemId(), currentActivity.getTag() + currentActivity.getDanNum());
            res.setStatus(getFlowerRoomRankStatus(activityDO, roomId, scoreList));
            if (score == null) {
                score = 0L;
                res.setRank(0);
            } else {
                res.setRank(getMySelfFlowerRoomRank(score, scoreList, roomId));
            }
            res.setMyselfScore(score);
            UserActivityScoreDO nextScore = userActivityScoreDAO.getRoomNextScore(currentActivity.getItemId(), score);
            if (nextScore != null) {
                res.setTopScore(nextScore.getScore());
            } else {
                res.setTopScore(score);
            }
        }
        return res;
    }

    /**
     * 返回我房间参赛状态
     *
     * @param activityDO
     * @param roomId
     * @param scoreList
     * @return
     */
    public Integer getFlowerRoomRankStatus(ActivityDO activityDO, Long roomId, List<UserActivityScoreDO> scoreList) {
        // 如果是第一阶段的海选, 直接返回可参加
        if (StringUtils.isNotBlank(activityDO.getAssociatedActivities())) {
            AssociatedRoomActivityItem currentActivity = getCurrentActivity(activityDO);
            if (currentActivity != null) {
                Set<Long> roomSetIds = new HashSet<>();
                scoreList.forEach(i -> roomSetIds.add(i.getRoomId()));
                if (currentActivity.getStep() == 1) {
                    // 如果是第一阶段, 只要检查是否在榜上
                    if (roomSetIds.contains(roomId)) {
                        return ActivityRankStatusEnum.SUCCESS.getValue();
                    } else {
                        return ActivityRankStatusEnum.NONE.getValue();
                    }
                } else {
                    Set<Long> promiseRoomIds = rtcRoomAdvanceDAO.getPromiseRoomIds(currentActivity.getItemId());
                    if (promiseRoomIds.contains(roomId)) {
                        if (roomSetIds.contains(roomId)) {
                            return ActivityRankStatusEnum.SUCCESS.getValue();
                        } else {
                            return ActivityRankStatusEnum.NONE.getValue();
                        }
                    } else {
                        return ActivityRankStatusEnum.FAIL.getValue();
                    }
                }
            }
        }
        return ActivityRankStatusEnum.FAIL.getValue();
    }

    /**
     * 获取当前海选节点的活动
     *
     * @return
     */
    public AssociatedRoomActivityItem getCurrentActivity(ActivityDO activityDO) {
        try {
            List<AssociatedRoomActivityItem> itemList = JSON.parseArray(activityDO.getAssociatedActivities(), AssociatedRoomActivityItem.class);
            if (CollectionUtil.isNotEmpty(itemList)) {
                Long now = System.currentTimeMillis();
                itemList = CollectionUtil.sort(itemList, Comparator.comparingInt(AssociatedRoomActivityItem::getStep));
                // 这里进行处理, 如果开始时间小于当前时间的, 默认为第一阶段, 如果当前时间大于结束时间的, 默认第三阶段
                AssociatedRoomActivityItem first = itemList.get(0);
                if (now < first.getActivityBeginTime()) {
                    if (itemList.size() == 1) {
                        first.setIsFinal(true);
                    }
                    return first;
                }
                AssociatedRoomActivityItem last = itemList.get(itemList.size() - 1);
                if (now > last.getActivityEndTime()) {
                    last.setIsFinal(true);
                    return last;
                }
                int index = 1;
                for (AssociatedRoomActivityItem item : itemList) {
                    if (item.getActivityBeginTime() < now && item.getActivityEndTime() > now) {
                        if (index == itemList.size()) {
                            item.setIsFinal(true);
                        } else {
                            item.setIsFinal(false);
                        }
                        return item;
                    }
                    index++;
                }
            }
        } catch (Exception e) {
            log.error("获取魅力花房活动子活动异常, 请检查配置", e);
            return null;
        }
        return null;
    }

    /**
     * 获取子节点的活动
     *
     * @param activityDO
     * @param subActivityId
     * @return
     */
    public AssociatedRoomActivityItem getSubActivityItem(ActivityDO activityDO, Long subActivityId) {
        List<AssociatedRoomActivityItem> itemList = JSON.parseArray(activityDO.getAssociatedActivities(), AssociatedRoomActivityItem.class);
        if (CollectionUtil.isNotEmpty(itemList)) {
            for (AssociatedRoomActivityItem item : itemList) {
                if (item.getItemId().equals(subActivityId)) {
                    return item;
                }
            }
        }
        return null;
    }

    /**
     * 返回我房间排名
     *
     * @return
     */
    public Integer getMySelfFlowerRoomRank(Long myRoomScore, List<UserActivityScoreDO> scoreList, Long roomId) {
        if (myRoomScore > 0) {
            int rank = 0;
            int defaultRank = 1;
            for (UserActivityScoreDO scoreDo : scoreList) {
                if (scoreDo.getRoomId().equals(roomId)) {
                    rank = defaultRank;
                }
                defaultRank++;
            }
            return rank;
        } else {
            return 0;
        }
    }

    /**
     * 魅力花房列表
     *
     * @param activityId
     * @return
     */
    @Override
    public List<ActivityAnniversaryRankVO> getFlowerRoomRankList(Long activityId, Long subActivityId, Integer needDanger) {
        ActivityDO activityDO = activityDAO.getActivityById(activityId);
        // 获取魅力花房排行
        AssociatedRoomActivityItem item = getCurrentActivity(activityDO);
        if (subActivityId != null) {
            item = getSubActivityItem(activityDO, subActivityId);
        }
        Integer limit = item.getTag();
        if (needDanger != null && needDanger == 1) {
            limit = item.getTag() + item.getDanNum();
        }
        List<UserActivityScoreDO> scoreList = userActivityScoreDAO.getRoomRankList(item.getItemId(), limit);
        List<ActivityAnniversaryRankVO> res = new ArrayList<>();
        if (CollectionUtil.isNotEmpty(scoreList)) {
            int rank = 1;
            Set<Long> roomIds = new HashSet<>(scoreList.size());
            scoreList.forEach(i -> {
                roomIds.add(i.getRoomId());
            });
            Map<Long, RtcRoom> roomMap = rtcRoomService.getRoomMap(roomIds);
            for (UserActivityScoreDO scoreDO : scoreList) {
                ActivityAnniversaryRankVO tmp = new ActivityAnniversaryRankVO();
                tmp.setId(String.valueOf(scoreDO.getUserId()));
                tmp.setRank(rank);
                rank++;
                tmp.setScore(scoreDO.getScore());

                RtcRoom tmpRoom = roomMap.get(scoreDO.getRoomId());
                if (tmpRoom != null) {
                    tmp.setImgUrl(tmpRoom.getRoomCover());
                    tmp.setName(tmpRoom.getRoomName());
                    tmp.setId(tmpRoom.getRoomId().toString());
                    tmp.setRoomNo(tmpRoom.getRoomNo().toString());
                }
                res.add(tmp);
            }
        }
        return res;
    }

    /**
     * 获取我的魅力花房信息
     *
     * @param activityId
     * @param roomId
     * @return
     */
    @Override
    public ActivityFlowRoomRankInfoVO getMyselfFlowerRoomRank(Long activityId, Long roomId) {
        ActivityFlowRoomRankInfoVO res = new ActivityFlowRoomRankInfoVO();

        ActivityDO activityDO = activityDAO.getActivityById(activityId);
        AssociatedRoomActivityItem currentActivity = getCurrentActivity(activityDO);
        if (currentActivity != null) {
            res.setCurrentActivityId(currentActivity.getItemId());
            if (roomId != null) {
                Long score = userActivityScoreDAO.getRoomScore(currentActivity.getItemId(), roomId);
                RtcRoom room = rtcRoomService.getRoom(roomId);
                res.setRoomName(room.getRoomName());
                if (score == null) {
                    score = 0L;
                    res.setRank(0);
                } else {
                    if (currentActivity != null) {
                        List<UserActivityScoreDO> scoreList = userActivityScoreDAO.getRoomRankList(currentActivity.getItemId(), currentActivity.getTag() + currentActivity.getDanNum());
                        res.setRank(getMySelfFlowerRoomRank(score, scoreList, roomId));
                    } else {
                        res.setRank(0);
                    }
                }
                res.setScore(score);
            }
        }

        if (StringUtils.isNotBlank(activityDO.getAssociatedActivities())) {
            List<AssociatedRoomActivityItem> itemList = JSON.parseArray(activityDO.getAssociatedActivities(), AssociatedRoomActivityItem.class);
            List<SubActivityVO> subActivityList = new ArrayList<>();
            for (AssociatedRoomActivityItem item : itemList) {
                SubActivityVO tmp = new SubActivityVO();
                tmp.setActivityId(item.getItemId());
                tmp.setActivityName(item.getItemName());
                tmp.setBeginTime(item.getActivityBeginTime());
                tmp.setEndTime(item.getActivityEndTime());
                subActivityList.add(tmp);
            }
            res.setSubActivityList(subActivityList);
        }
        return res;
    }

    /**
     * 发放房间边框奖励
     *
     * @return
     */
    @Override
    public boolean giveRtcRoomBorderByActivity(Long roomId, String borderUrl, String remark, Long startTime, Long duration) {
        RtcRoom roomDO = rtcRoomService.getRoom(roomId);
        if (roomDO == null || roomDO.getRoomId() < 1) {
            return false;
        }
        RtcRoomBorderDO border = new RtcRoomBorderDO();
        border.setHisId(identityService.getId());
        border.setRoomId(roomId);
        border.setUserId(roomDO.getUserId());
        border.setStatus(BoolIntVal.True);
        border.setBorderUrl(borderUrl);
        border.setStartTime(startTime);
        if (duration < 0) {
            border.setEndTime(startTime + DateUtil.DAY_OF_MILLION * 9999);
        } else {
            border.setEndTime(startTime + duration);
        }
        border.setDuration(duration);
        border.setRemark(remark);
        border.setIsDel(BoolIntVal.False);
        border.setCreateTime(System.currentTimeMillis());
        rtcRoomBorderDAO.insert(border);
        return true;
    }

    /**
     * 获取房间边框
     *
     * @param roomIds
     * @return
     */
    @Override
    public Map<Long, String> getRtcRoomBorder(Set<Long> roomIds) {
        if (CollectionUtil.isNotEmpty(roomIds)) {
            Map<Long, String> res = new HashMap<>(roomIds.size());
            Long now = System.currentTimeMillis();
            List<RtcRoomBorderDO> borderList = rtcRoomBorderDAO.getRtcRoomBorderList(roomIds, now);
            borderList.forEach(i -> {
                res.put(i.getRoomId(), i.getBorderUrl());
            });
            return res;
        } else {
            return new HashMap<>();
        }
    }

    /**
     * 获取房间边框
     *
     * @param roomId
     * @return
     */
    @Override
    public String getRtcRoomBorder(Long roomId) {
        Long now = System.currentTimeMillis();
        return rtcRoomBorderDAO.getRtcRoomBorder(roomId, now);
    }


    /**
     * 添加积分
     *
     * @param orderId
     * @param userId
     * @param activityId
     * @param roomId
     * @param familyId
     * @param score
     * @param remark
     * @return
     */
    @Override
    public boolean doAddScore(Long orderId, Long userId, Long exUserId, Long activityId, Long roomId, Long familyId, Long score, String remark, ActivityDO activityDO) {
        Integer count = userActivityScoreDetailDAO.getCount(activityId, orderId, userId, exUserId);
        log.info("订单{} 记录结果 {}, 用户ID {}, || exUserId {} || 活动ID {} || 分数 {}", orderId, count, userId, exUserId, activityId, score);
        if (count > 0) {
            log.info("订单{}已经记录, 无法重复记录, 用户ID {}, || exUserId {} || 活动ID {}", orderId, userId, exUserId, activityId);
            return false;
        }
        Long now = System.currentTimeMillis();
        UserActivityScoreDetailDO detailDO = new UserActivityScoreDetailDO();
        detailDO.setHisId(identityService.getId());
        detailDO.setOrderId(orderId);
        detailDO.setUserId(userId);
        detailDO.setExUserId(exUserId);
        detailDO.setActivityId(activityId);
        detailDO.setRoomId(roomId);
        detailDO.setFamilyId(familyId);
        detailDO.setScore(score);
        detailDO.setRemark(remark);
        detailDO.setCreateDate(DateUtil.getIntDate());
        detailDO.setCreateTime(now);
        detailDO.setIsDel(BoolIntVal.False);
        userActivityScoreDetailDAO.insert(detailDO);


        int scoreCount;
        ActivityTypeEnum typeEnum = ActivityTypeEnum.toEnum(activityDO.getActivityType());
        switch (typeEnum) {
            case INFLUENCER:
            case GORGEOUS:
                scoreCount = userActivityScoreDAO.getUserScoreCount(activityId, userId, null, null);
                break;
            default:
                scoreCount = userActivityScoreDAO.getUserScoreCount(activityId, userId, familyId, roomId);
                break;
        }
        if (scoreCount > 0) {
            switch (typeEnum) {
                case INFLUENCER:
                case GORGEOUS:
                    userActivityScoreDAO.updateScore(activityId, userId, null, null, score);
                    break;
                default:
                    userActivityScoreDAO.updateScore(activityId, userId, familyId, roomId, score);
                    break;
            }
        } else {
            UserActivityScoreDO scoreDO = new UserActivityScoreDO();
            scoreDO.setHisId(identityService.getId());
            scoreDO.setUserId(userId);
            scoreDO.setActivityId(activityId);
            scoreDO.setRoomId(roomId);
            scoreDO.setFamilyId(familyId);
            scoreDO.setScore(score);
            scoreDO.setCreateTime(now);
            scoreDO.setIsDel(BoolIntVal.False);
            userActivityScoreDAO.insert(scoreDO);
        }


        return true;
    }

    /**
     * 添加积分
     *
     * @param orderId
     * @param giftId
     * @param sendUserId
     * @param receiveUserId
     * @param roomId
     * @param remark
     * @return
     */
    @Override
    @Async
    public void addScore(Long orderId, Long giftId, Long sendUserId, Long receiveUserId, Long roomId, Long giftValue, Long giftTrueValue, String remark) {
        log.info("开始添加积分: Long orderId:{}, Long giftId:{}, Long sendUserId:{}, Long receiveUserId:{}, Long roomId:{}, Long giftValue:{}, Long giftTrueValue:{}, String remark:{}", orderId, giftId, sendUserId, receiveUserId, roomId, giftValue, giftTrueValue, remark);
        // 获取所有需要添加积分的活动
        Long now = System.currentTimeMillis();
        List<Integer> activityTypes = buildNeedHandlerTypes();
        List<ActivityDO> activityList = activityDAO.getActivityListByTypes(activityTypes, now);
        for (ActivityDO activityDO : activityList) {
            Long score = getActivityScore(giftValue, giftTrueValue, activityDO, giftId);
            Long needHandlerUserId = getActivityUserIds(sendUserId, receiveUserId, activityDO);
            Long exUserId = getActivityExUserIds(sendUserId, receiveUserId, activityDO);
            Long userFamilyId = familyService.getFamilyId(needHandlerUserId);
            if (score > 0) {
                Long activityId = activityDO.getActivityId();
                if (ActivityTypeEnum.CHARMING_ROOM.getValue() == activityDO.getActivityType()) {

                    if (roomId == null) {
                        continue;
                    }
                    AssociatedRoomActivityItem item = getCurrentActivity(activityDO);
                    if (item == null) {
                        continue;
                    }
                    ActivityDO subActivity = activityDAO.getActivityById(item.getItemId());
                    // 魅力花房是由子活动控制
                    score = getActivityScore(giftValue, giftTrueValue, subActivity, giftId);
                    if (score < 1) {
                        continue;
                    }
                    activityId = item.getItemId();
                    // 还需要进行一次校验, 检查房间是否有权限进行魅力花房活动
                    Integer status = getFlowerRoomRankStatus(activityDO, roomId, new ArrayList<>());
                    if (status == ActivityRankStatusEnum.FAIL.getValue()) {
                        continue;
                    }
                }

                // 巅峰家族-收礼人如果没有家族, 不积分
                if (ActivityTypeEnum.PEAKFAMILY.getValue() == activityDO.getActivityType()) {
                    log.info("巅峰家族开始添加积分: Long orderId:{}, Long giftId:{}, Long sendUserId:{}, Long receiveUserId:{}, Long roomId:{}, Long giftValue:{}, Long giftTrueValue:{}, String remark:{}", orderId, giftId, sendUserId, receiveUserId, roomId, giftValue, giftTrueValue, remark);
                    if (userFamilyId == null) {
                        log.info("收礼人没有家族: Long orderId:{}, Long giftId:{}, Long sendUserId:{}, Long receiveUserId:{}, Long roomId:{}, Long giftValue:{}, Long giftTrueValue:{}, String remark:{}", orderId, giftId, sendUserId, receiveUserId, roomId, giftValue, giftTrueValue, remark);
                        continue;
                    }
                }
                try {
                    doAddScore(orderId, needHandlerUserId, exUserId, activityId, roomId, userFamilyId, score, remark, activityDO);
                } catch (Exception e) {
                    log.error("添加积分的参数是: 订单{}, 用户ID {}, || exUserId {} || 活动ID {} || 分数 {}", orderId, needHandlerUserId, exUserId, activityId, score);
                    log.error("添加积分失败", e);
                    continue;
                }

            }
        }
    }


    /**
     * 添加需要添加积分的列表
     *
     * @return
     */
    private List<Integer> buildNeedHandlerTypes() {
        List<Integer> activityTypes = new ArrayList<>();
        activityTypes.add(ActivityTypeEnum.CHARMING_ROOM.getValue());
        activityTypes.add(ActivityTypeEnum.INFLUENCER.getValue());
        activityTypes.add(ActivityTypeEnum.GORGEOUS.getValue());
        activityTypes.add(ActivityTypeEnum.PEAKFAMILY.getValue());
        return activityTypes;
    }

    /**
     * 获取需要添加积分的规则
     *
     * @param giftValue
     * @param giftTrueValue
     * @param activityDO
     * @param giftId
     * @return
     */
    private Long getActivityScore(Long giftValue, Long giftTrueValue, ActivityDO activityDO, Long giftId) {
        ActivityTypeEnum typeEnum = ActivityTypeEnum.toEnum(activityDO.getActivityType());
        // 检查是否包含指定的礼物ID
        if (giftId != null && StringUtils.isNotBlank(activityDO.getGiftIds()) && activityDO.getGiftIds().indexOf(giftId.toString()) < 0) {
            return 0L;
        }
        switch (typeEnum) {
            case GORGEOUS:
                return giftTrueValue;
            default:
                return giftValue;
        }
    }

    /**
     * 获取需要添加积分规则的用户
     *
     * @param sendUserId
     * @param receiveUserId
     * @param activityDO
     * @return
     */
    private Long getActivityUserIds(Long sendUserId, Long receiveUserId, ActivityDO activityDO) {
        ActivityTypeEnum typeEnum = ActivityTypeEnum.toEnum(activityDO.getActivityType());
        // 除了风云人物, 其他都是接受者增加积分
        switch (typeEnum) {
            case INFLUENCER:
                return sendUserId;
            default:
                return receiveUserId;
        }
    }

    private Long getActivityExUserIds(Long sendUserId, Long receiveUserId, ActivityDO activityDO) {
        ActivityTypeEnum typeEnum = ActivityTypeEnum.toEnum(activityDO.getActivityType());
        // 除了风云人物, 其他都是接受者增加积分
        switch (typeEnum) {
            case INFLUENCER:
                return receiveUserId;
            default:
                return sendUserId;
        }
    }

    /**
     * 海选晋级
     */
    @Async
    @Override
    public void rankLevelUp() {
        // 时间节点是选在半夜零点执行, 所以先获取昨天的时间
        Long promiseTime = System.currentTimeMillis() - 1 * DateUtil.HOUR_OF_MILLION;
        if (SpringContextUtil.isDev()) {
            promiseTime = System.currentTimeMillis() - 10 * DateUtil.MINUTE_OF_MILLION;
        }
        List<ActivityDO> activityDOList = activityDAO.getRuningActivityByType(ActivityTypeEnum.CHARMING_ROOM.getValue(), promiseTime);
        for (ActivityDO activityDO : activityDOList) {
            doRankLevelUp(activityDO);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    public void doRankLevelUp(ActivityDO activityDO) {
        AssociatedRoomActivityItem needSettleActivity = getSettleActivity(activityDO);
        if (needSettleActivity != null) {
            AssociatedRoomActivityItem currentActivity = getCurrentActivity(activityDO);
            Long now = System.currentTimeMillis();
            if (needSettleActivity.getStep().equals(currentActivity.getStep())) {
                // 说明是决赛, 需要插入晋级消息, 发送消息标明决赛奖励, 发放奖品
                // 发放奖品以后, 也还需要发放通知
                int count = rtcRoomAdvanceDAO.getFinalFlagCount(activityDO.getActivityId());
                if (count != 0) {
                    return;
                }
                Map<Integer, ActivityAnnualAwardConfigDO> totalRewardConfigMap = getRewardConfig(activityDO.getActivityId(), ActivityRankTypeEnum.TOTAL.getValue());
                List<ActivityAnniversaryRankVO> totalScoreList = getFlowerRoomRankList(activityDO.getActivityId(), needSettleActivity.getItemId(), BoolIntVal.False);
                if (CollectionUtil.isEmpty(totalScoreList)) {
                    return;
                }
                doGiveReward(totalScoreList, totalRewardConfigMap, activityDO);
                sendRankFinishMsg(activityDO, "恭喜晋升魅力花房", null);

                // 插入记录
                RtcRoomAdvanceDO advanceDO = new RtcRoomAdvanceDO();
                advanceDO.setHisId(identityService.getId());
                advanceDO.setMainActivityId(activityDO.getActivityId());
                advanceDO.setSubActivityId(activityDO.getActivityId());
                advanceDO.setStep(-1);
                advanceDO.setRoomId(0L);
                advanceDO.setScore(0L);
                advanceDO.setSortRank(1);
                advanceDO.setCreateTime(now);
                rtcRoomAdvanceDAO.insert(advanceDO);

                return;
            } else {
                // 检查数据是否存在, 是否重复插入
                int count = rtcRoomAdvanceDAO.getCountBySubActId(currentActivity.getItemId());
                if (count != 0) {
                    return;
                }
                List<UserActivityScoreDO> scoreList = userActivityScoreDAO.getRoomRankList(needSettleActivity.getItemId(), needSettleActivity.getTag());
                if (CollectionUtil.isNotEmpty(scoreList)) {
                    List<RtcRoomAdvanceDO> advanceDOS = new ArrayList<>(scoreList.size());
                    int rank = 1;
                    for (UserActivityScoreDO scoreDO : scoreList) {
                        RtcRoomAdvanceDO advanceDO = new RtcRoomAdvanceDO();
                        advanceDO.setHisId(identityService.getId());
                        advanceDO.setMainActivityId(activityDO.getActivityId());
                        advanceDO.setSubActivityId(currentActivity.getItemId());
                        advanceDO.setStep(currentActivity.getStep());
                        advanceDO.setRoomId(scoreDO.getRoomId());
                        advanceDO.setScore(scoreDO.getScore());
                        advanceDO.setSortRank(rank);
                        rank++;
                        advanceDO.setCreateTime(now);
                        advanceDOS.add(advanceDO);
                    }
                    rtcRoomAdvanceDAO.insertBatch(advanceDOS);
                    sendRankFinishMsg(activityDO, "恭喜晋级" + needSettleActivity.getTag() + "强", needSettleActivity.getItemId());
                }
            }
        }
    }


    /**
     * 分发魅力人物奖品
     */
    @Override
    @Async
    public void giveInfluenceReward() {
        List<Integer> activityTypes = new ArrayList<>();
        activityTypes.add(ActivityTypeEnum.INFLUENCER.getValue());
        activityTypes.add(ActivityTypeEnum.GORGEOUS.getValue());
        Long promiseTime = DateUtil.getYesterday(new Date()).getTime();
        if (SpringContextUtil.isDev()) {
            promiseTime = System.currentTimeMillis() - DateUtil.HOUR_OF_MILLION;
        }
        List<ActivityDO> activityDOList = activityDAO.getActivityListByTypes(activityTypes, promiseTime);
        if (CollectionUtil.isNotEmpty(activityDOList)) {
            Integer yesterdayInt = DateUtil.getIntDate(new Date(promiseTime));
            for (ActivityDO activityDO : activityDOList) {
                // 首先发放普通的
                Map<Integer, ActivityAnnualAwardConfigDO> dailyRewardConfigMap = getRewardConfig(activityDO.getActivityId(), ActivityRankTypeEnum.DAY.getValue());
                List<ActivityAnniversaryRankVO> dailyScoreList = getInfluentialRankList(activityDO.getActivityId(), ActivityRankTypeEnum.YESTERDAY.getValue());
                doGiveReward(dailyScoreList, dailyRewardConfigMap, activityDO);
                // 如果是最后一天, 需要发放最终的
                if (yesterdayInt.equals(DateUtil.getIntDate(new Date(activityDO.getActivityEndTime())))) {
                    Map<Integer, ActivityAnnualAwardConfigDO> totalRewardConfigMap = getRewardConfig(activityDO.getActivityId(), ActivityRankTypeEnum.TOTAL.getValue());
                    List<ActivityAnniversaryRankVO> totalScoreList = getInfluentialRankList(activityDO.getActivityId(), ActivityRankTypeEnum.TOTAL.getValue());
                    doGiveReward(totalScoreList, totalRewardConfigMap, activityDO);
                }
            }
        }
    }

    /**
     * 真正发放奖励
     *
     * @param scoreList
     * @param rewardConfigMap
     */
    private void doGiveReward(List<ActivityAnniversaryRankVO> scoreList, Map<Integer, ActivityAnnualAwardConfigDO> rewardConfigMap, ActivityDO activityDO) {
        if (CollectionUtil.isNotEmpty(scoreList) && CollectionUtil.isNotEmpty(rewardConfigMap)) {
            int i = 1;
            Integer dateInt = DateUtil.getIntDate();
            Long now = System.currentTimeMillis();
            for (ActivityAnniversaryRankVO rankVo : scoreList) {
                // 获取当前排名需要发放的奖品
                ActivityAnnualAwardConfigDO configDO = rewardConfigMap.get(i);
                if (SpringContextUtil.isDev()) {
                    log.info("需要发放的用户:" + JSON.toJSONString(rankVo, true));
                    log.info("需要发放的奖品:" + JSON.toJSONString(configDO, true));
                }
                if (configDO != null && CollectionUtil.isNotEmpty(configDO.getItemDoList())) {
                    AddItemToBagSourceEnum bagSourceEnum;
                    if (activityDO.getActivityType().equals(ActivityTypeEnum.GORGEOUS.getValue())) {
                        bagSourceEnum = AddItemToBagSourceEnum.GORGEOUS;
                    } else {
                        bagSourceEnum = AddItemToBagSourceEnum.INFLUENCER;
                    }
                    Long targetId = Long.valueOf(rankVo.getId());

                    for (ActivityAnnualAwardItemDO item : configDO.getItemDoList()) {
                        // 检查奖励类型, 只有 礼物道具才能发放
                        if (ActivityAwardTypeEnum.GIFT.getValue() == item.getAwardType() || ActivityAwardTypeEnum.PROP.getValue() == item.getAwardType()) {
                            sendRewardMsg(item, targetId, activityDO, dateInt, now, bagSourceEnum);
                        }
                        if (ActivityAwardTypeEnum.BORDER.getValue() == item.getAwardType()) {
                            giveRtcRoomBorderByActivity(targetId, item.getAwardValue(), "活动奖励边框", now, item.getExpire());
                        }
                    }
                    String title = "";
                    if (StringUtils.isNotBlank(configDO.getAlertStyle())) {
                        if (activityDO.getActivityType().equals(ActivityTypeEnum.GORGEOUS.getValue())) {
                            title = "恭喜你，获得「倾城人物」" + buildTitleTail(i);
                            sendRewardFinishMsg(activityDO, configDO.getAlertStyle(), targetId, title, null);
                        }
                        if (activityDO.getActivityType().equals(ActivityTypeEnum.INFLUENCER.getValue())) {
                            title = "恭喜你，获得「风云人物」" + buildTitleTail(i);
                            sendRewardFinishMsg(activityDO, configDO.getAlertStyle(), targetId, title, null);
                        }
                        if (activityDO.getActivityType().equals(ActivityTypeEnum.CHARMING_ROOM.getValue())) {
                            title = "恭喜获得「魅力花房」奖励";
                            sendRewardFinishMsg(activityDO, configDO.getAlertStyle(), null, title, targetId);
                        }
                    }
                }
                i++;
            }
        }
    }

    private String buildTitleTail(int rank) {
        switch (rank) {
            case 1:
                return "冠军奖励";
            case 2:
                return "亚军奖励";
            case 3:
                return "季军奖励";
            case 4:
                return "第4名奖励";
            case 5:
                return "第5名奖励";
            default:
                return "10强奖励";
        }
    }

    /**
     * 发送奖励消息
     */
    private void sendRewardMsg(ActivityAnnualAwardItemDO item, Long userId, ActivityDO activityDO, Integer dateInt, Long now, AddItemToBagSourceEnum bagSourceEnum) {
        Long itemId = Long.valueOf(item.getAwardValue());
        Long giveHisId = giveHisDAO.getGiveHisId(userId, activityDO.getActivityId(), item.getRankType(), itemId, dateInt);

        if (giveHisId == null || giveHisId == 0) {
            giveHisId = identityService.getId();
            ActivityAnnualAwardGiveHisDO giveHisDO = new ActivityAnnualAwardGiveHisDO();
            giveHisDO.setHisId(giveHisId);
            giveHisDO.setUserId(userId);
            giveHisDO.setActivityId(activityDO.getActivityId());
            giveHisDO.setRankType(item.getRankType());
            giveHisDO.setItemId(itemId);
            giveHisDO.setCreateDate(dateInt);
            giveHisDO.setCreateTime(now);
            giveHisDAO.insert(giveHisDO);
        }
        QueueAddItemToBagMsg itemBagMsg = new QueueAddItemToBagMsg();
        itemBagMsg.setSource(bagSourceEnum);
        itemBagMsg.setOrderId(giveHisId);
        itemBagMsg.setItemId(itemId);
        itemBagMsg.setItemName(item.getName());
        if (ActivityAwardTypeEnum.GIFT.getValue() == item.getAwardType()) {
            itemBagMsg.setItemType(BagItemTypeEnum.GIFT);
        } else {
            itemBagMsg.setItemType(BagItemTypeEnum.PROPS);
        }
        itemBagMsg.setQuantity(item.getNum());
        itemBagMsg.setExpireTime(getExpireTime(item.getExpire()));
        itemBagMsg.setUserId(userId);
        itemBagMsg.setRemark("活动奖励发放");
        if (SpringContextUtil.isDev()) {
            log.info("活动奖励发放消息 :" + JSON.toJSONString(itemBagMsg, true));
        }
        boolean sendMsg = rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.ADD_ITEM_TO_BAG, itemBagMsg.serialize());
        String msgStr = JSON.toJSONString(itemBagMsg);
        if (sendMsg) {
            log.info("风云人物活动奖励发放奖品成功 {}", msgStr);
            return;
        }
        log.error("风云人物活动奖励发放奖品失败 " + msgStr);
    }

    /**
     * 发送奖励完成消息
     *
     * @param activityDO
     * @param rewardImgUrl
     * @param userId
     */
    private void sendRewardFinishMsg(ActivityDO activityDO, String rewardImgUrl, Long userId, String title, Long roomId) {
        AnnualGalaFinishMsg finishMsg = new AnnualGalaFinishMsg();
        finishMsg.setActivityType(activityDO.getActivityType());
        finishMsg.setMsgType(0);
        finishMsg.setActivityId(activityDO.getActivityId());
        finishMsg.setImgUrl(rewardImgUrl);
        finishMsg.setTitle(title);
        finishMsg.setUserId(userId);
        finishMsg.setRoomId(roomId);
        imService.sendRtcRoomMsg(ImTargetType.GROUP, null, userId, roomId, finishMsg);
    }

    /**
     * 发送排名消息
     *
     * @param activityDO
     * @param title
     */
    private void sendRankFinishMsg(ActivityDO activityDO, String title, Long subActivityId) {
        AnnualGalaFinishMsg finishMsg = new AnnualGalaFinishMsg();
        finishMsg.setActivityType(activityDO.getActivityType());
        finishMsg.setMsgType(1);
        finishMsg.setActivityId(activityDO.getActivityId());
        finishMsg.setTitle(title);
        finishMsg.setSubActivityId(subActivityId);
        imService.sendRtcRoomMsg(ImTargetType.GROUP, null, null, null, finishMsg);
    }

    /**
     * 获取奖励数据
     *
     * @param activityId
     * @param rankType
     * @return
     */
    @Override
    public Map<Integer, ActivityAnnualAwardConfigDO> getRewardConfig(Long activityId, Integer rankType) {
        Map<Integer, ActivityAnnualAwardConfigDO> res = new HashMap<>();
        List<ActivityAnnualAwardConfigDO> configList = awardConfigDAO.getAwardConfigList(activityId, rankType);
        List<ActivityAnnualAwardItemDO> itemList = awardItemDAO.getAwardItemList(activityId, rankType);
        if (CollectionUtil.isNotEmpty(configList)) {
            for (ActivityAnnualAwardConfigDO configDO : configList) {
                if (CollectionUtil.isNotEmpty(itemList)) {
                    configDO.setItemDoList(itemList.stream().filter(i -> i.getConfigId().equals(configDO.getHisId())).collect(Collectors.toList()));
                }
                res.put(configDO.getSequence(), configDO);
            }
        }
        return res;
    }


    /**
     * 获取需要结算的活动
     *
     * @param activityDO
     * @return
     */
    public AssociatedRoomActivityItem getSettleActivity(ActivityDO activityDO) {
        Long now = System.currentTimeMillis();
        List<AssociatedRoomActivityItem> itemList = JSON.parseArray(activityDO.getAssociatedActivities(), AssociatedRoomActivityItem.class);
        if (CollectionUtil.isNotEmpty(itemList)) {
            // 首先获取当前的
            AssociatedRoomActivityItem currentItem = getCurrentActivity(activityDO);
            // 如果是最后一条数据, 并且超过了结算时间, 就返回最后一条数据
            if (currentItem.getIsFinal() && now > currentItem.getActivityEndTime()) {
                return currentItem;
            }
            // 如果只有一个阶段, 直接返回, 如果不止一个阶段, 说明还不需要结算
            if (currentItem.getStep() == 1) {
                if (itemList.size() == 1) {
                    return currentItem;
                } else {
                    return null;
                }
            }
            // 查找上一个阶段的活动
            Integer step = currentItem.getStep() - 1;
            for (AssociatedRoomActivityItem item : itemList) {
                if (step.equals(item.getStep())) {
                    return item;
                }
            }
        }
        return null;
    }

    /**
     * 巅峰家族排行榜
     *
     * @param activityId
     * @return
     */
    @Override
    public List<AnnualGalaPeakFamilyRankBO> getPeakFamilyRankList(Long activityId) {

        List<UserActivityScoreDO> list = userActivityScoreDAO.selectPeakFamilyRankList(activityId);

        Set<Long> familyIds = list.stream().map(UserActivityScoreDO::getFamilyId).collect(Collectors.toSet());
        Map<Long, FamilyDO> familyMap = familyService.getFamilyMap(familyIds);

        List<AnnualGalaPeakFamilyRankBO> result = list.stream()
                .filter(Objects::nonNull)
                .map(val -> {

                    AnnualGalaPeakFamilyRankBO bo = new AnnualGalaPeakFamilyRankBO();
                    BeanUtils.copyProperties(val, bo);
                    if (familyMap != null) {
                        FamilyDO familyDO = Optional.ofNullable(familyMap.get(val.getFamilyId())).orElseGet(FamilyDO::new);
                        bo.setFamilyName(familyDO.getFamilyName());
                        bo.setFamilyIcon(familyDO.getFamilyLogo());
                        bo.setFamilyNo(familyDO.getFamilyNo());
                    }
                    return bo;
                }).collect(Collectors.toList());

        return result;
    }

    /**
     * 巅峰家族给个人发放奖励
     *
     * @param familyRank      家族排名 1 2 3
     * @param userId          获奖的用户
     * @param rewardConfigMap 奖项配置
     * @param activityId      活动id
     */
    @Override
    public List<ActivityAnnualAwardItemDO> doGivePeakFamilyUserReward(Integer familyRank, Long userId, Long activityId, Map<Integer, ActivityAnnualAwardConfigDO> rewardConfigMap) {
        if (CollectionUtil.isEmpty(rewardConfigMap)) {
            log.info("巅峰家族获取奖励配置为空,停止发放");
            return Collections.EMPTY_LIST;
        }
        Integer dateInt = DateUtil.getIntDate();
        Long now = System.currentTimeMillis();

        // 获取当前排名需要发放的奖品
        ActivityAnnualAwardConfigDO configDO = rewardConfigMap.get(familyRank);
        if (configDO == null) {
            log.info("巅峰家族获取奖励配置为空2,停止发放");
            return Collections.EMPTY_LIST;
        }

        if (CollectionUtil.isEmpty(configDO.getItemDoList())) {
            log.info("巅峰家族获取奖励配置为空3,停止发放");
            return Collections.EMPTY_LIST;
        }

        AddItemToBagSourceEnum bagSourceEnum = AddItemToBagSourceEnum.PEAKFAMILY;

        for (ActivityAnnualAwardItemDO item : configDO.getItemDoList()) {
            // 检查奖励类型, 只有 礼物道具才能发放
            if (ActivityAwardTypeEnum.GIFT.getValue() == item.getAwardType() || ActivityAwardTypeEnum.PROP.getValue() == item.getAwardType()) {
                Long itemId = Long.valueOf(item.getAwardValue());
                Long giveHisId = giveHisDAO.getGiveHisId(userId, activityId, item.getRankType(), itemId, dateInt);

                if (giveHisId == null || giveHisId == 0) {
                    giveHisId = identityService.getId();
                    ActivityAnnualAwardGiveHisDO giveHisDO = new ActivityAnnualAwardGiveHisDO();
                    giveHisDO.setHisId(giveHisId);
                    giveHisDO.setUserId(userId);
                    giveHisDO.setActivityId(activityId);
                    giveHisDO.setRankType(item.getRankType());
                    giveHisDO.setItemId(itemId);
                    giveHisDO.setCreateDate(dateInt);
                    giveHisDO.setCreateTime(now);
                    giveHisDAO.insert(giveHisDO);
                }
                QueueAddItemToBagMsg itemBagMsg = new QueueAddItemToBagMsg();
                itemBagMsg.setSource(bagSourceEnum);
                itemBagMsg.setOrderId(giveHisId);
                itemBagMsg.setItemId(itemId);
                BagItemTypeEnum bagItemTypeEnum = item.getAwardType() == ActivityAwardTypeEnum.GIFT.getValue() ? BagItemTypeEnum.GIFT : BagItemTypeEnum.PROPS;
                itemBagMsg.setItemType(bagItemTypeEnum);
                itemBagMsg.setItemName(Optional.ofNullable(item.getName()).orElse("未知"));
                itemBagMsg.setQuantity(item.getNum());
                itemBagMsg.setExpireTime(getExpireTime(item.getExpire()));
                itemBagMsg.setUserId(userId);
                itemBagMsg.setRemark(String.format("巅峰家族第%d名奖励发放%s", familyRank, item.getName()));
                if (SpringContextUtil.isDev()) {
                    log.info("六活动奖励发放消息:" + JSON.toJSONString(itemBagMsg, true));
                }
                rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.ADD_ITEM_TO_BAG, itemBagMsg.serialize());
            }
        }

        return configDO.getItemDoList();
    }

    /**
     * 获取道具过期时间
     *
     * @param durationTime
     * @return
     */
    private Long getExpireTime(Long durationTime) {
        return durationTime == -1 ? -1 : System.currentTimeMillis() + durationTime;
    }

    /**
     * 巅峰家族积分
     *
     * @return
     */
    @Override
    public Long getPeakFamilyScore(Long activityId, Long familyId) {

        return userActivityScoreDAO.selectPeakFamilyScore(activityId, familyId);
    }

    /**
     * 巅峰家族个人积分
     *
     * @return
     */
    @Override
    public Long getPeakFamilyPersonScore(Long activityId, Long familyId, Long userId) {

        return userActivityScoreDAO.selectPeakFamilyPersonScore(activityId, familyId, userId);
    }

    /**
     * 巅峰家族获奖的用户
     *
     * @return
     */
    @Override
    public List<UserActivityScoreDO> getPeakFamilyRewardUserList(Long activityId, Long familyId) {

        return userActivityScoreDAO.selectPeakFamilyRewardUserList(activityId, familyId);
    }

    /**
     * 清除用户在家族的积分
     *
     * @return
     */
    @Override
    public void clearPeakFamilyUserScore(Long activityId, Long familyId, Long userId) {

        userActivityScoreDAO.updateScoreDel(activityId, familyId, userId);
    }

    /**
     * 获取用户的发奖奖励
     */
    @Override
    public List<ActivityAnnualAwardGiveHisDO> getUserRecordList(Long userId, Long activityId) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq("activityId", activityId);
        qw.eq("userId", userId);
        return giveHisDAO.selectList(qw);
    }

    /**
     * 获取当前进度信息
     *
     * @param userId
     * @param roomId
     */
    @Override
    public ActivityJoinRankVO getPeakFamilyProcess(Long userId, Long activityId, Long roomId) {

        ActivityJoinRankVO res = new ActivityJoinRankVO();
        Long familyId = familyService.getFamilyId(userId);

        ActivityDO activityDO = activityDAO.getActivityById(activityId);
        res.setConfig(JSON.parseObject(activityDO.getRankStyle(), ActivityEntranceConfig.class));

        if (familyId == null) {
            res.setStatus(1);
            res.setName("暂无家族");
            return res;
        }

        String familyName = familyService.getFamilyName(familyId);
        res.setName(familyName);

        List<UserActivityScoreDO> rankTotalList = userActivityScoreDAO.selectPeakFamilyAllRankList(activityId);
        if (CollectionUtil.isEmpty(rankTotalList)) {
            // 当前无排名
            res.setStatus(1);
            res.setTopScore(0L);
            res.setMyselfScore(0L);
            return res;
        }
        // 获取家族当前排名
        Map<Integer, Object> familyRank = getFamilyRank(rankTotalList, familyId);
        if (CollectionUtil.isEmpty(familyRank)) {
            // 当前无排名
            res.setStatus(1);
            res.setMyselfScore(0L);
            res.setTopScore(rankTotalList.get(rankTotalList.size() - 1).getScore());
            return res;
        }

        Integer rank = Integer.valueOf(familyRank.get(1).toString());
        UserActivityScoreDO familyRankScore = (UserActivityScoreDO) familyRank.get(0);
        res.setRank(rank);
        res.setMyselfScore(Optional.ofNullable(familyRankScore.getScore()).orElse(0L));
        res.setStatus(rank <= 5 ? 2 : 1);
        res.setTopScore(rank == 1 ? null : rankTotalList.get(rank - 2).getScore());
        return res;
    }

    private Map<Integer, Object> getFamilyRank(List<UserActivityScoreDO> rankTotalList, Long familyId) {

        if (CollectionUtil.isEmpty(rankTotalList)) {
            return null;
        }

        Map<Integer, Object> res = new HashMap<>();
        for (int i = 0; i < rankTotalList.size(); i++) {
            UserActivityScoreDO scoreDO = rankTotalList.get(i);
            if (familyId.equals(scoreDO.getFamilyId())) {
                res.put(0, scoreDO);
                res.put(1, i + 1);
                return res;
            }
        }

        return null;
    }

    /**
     * 获取当前排名
     *
     * @param userId
     * @param activityId
     * @param roomId
     * @return
     */
    @Override
    public ActivityJoinRankVO getProcess(Long userId, Long activityId, Long roomId) {
        ActivityDO activity = activityDAO.getActivityById(activityId);
        ActivityTypeEnum activityType = ActivityTypeEnum.toEnum(activity.getActivityType());
        if (ActivityTypeEnum.PEAKFAMILY.equals(activityType)) {
            return getPeakFamilyProcess(userId, activityId, roomId);
        } else if (ActivityTypeEnum.CHARMING_ROOM.equals(activityType)) {
            return getFlowerRoomRank(userId, activityId, roomId);
        } else if (ActivityTypeEnum.GORGEOUS.equals(activityType) || ActivityTypeEnum.INFLUENCER.equals(activityType)) {
            return getInfluentialRankInfo(userId, activityId, ActivityRankTypeEnum.TOTAL.getValue());
        }

        return null;
    }

    /**
     * 是否是周年庆活动
     *
     * @param activityType
     * @return
     */
    @Override
    public boolean isAnniversary(Integer activityType) {
        List<Integer> activityTypes = new ArrayList<>();
        activityTypes.add(ActivityTypeEnum.CHARMING_ROOM.getValue());
        activityTypes.add(ActivityTypeEnum.INFLUENCER.getValue());
        activityTypes.add(ActivityTypeEnum.GORGEOUS.getValue());
        activityTypes.add(ActivityTypeEnum.PEAKFAMILY.getValue());
        if (activityTypes.contains(activityType)) {
            return true;
        }
        return false;
    }
}
