package com.starbuds.server.service.activity.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.starbuds.server.common.constant.activity.ActivityGiftsRankCategoryEnum;
import com.starbuds.server.common.constant.activity.ActivityGiftsRankTypeEnum;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rocketmq.RocketTopic;
import com.starbuds.server.common.pojo.activity.vo.ActivityGiftsRankVO;
import com.starbuds.server.common.pojo.activity.vo.GiftsRankUserItem;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.pojo.data.HeadwearVO;
import com.starbuds.server.common.pojo.db.ActivityGiftsRankBindDO;
import com.starbuds.server.common.pojo.db.ActivityGiftsRankReceiveDO;
import com.starbuds.server.common.pojo.db.ActivityGiftsRankSendDO;
import com.starbuds.server.common.pojo.db.ActivityGiftsSaveHisDO;
import com.starbuds.server.common.pojo.mq.QueueActivityGiftsPaidMsg;
import com.starbuds.server.common.pojo.user.UserBasicInfo;
import com.starbuds.server.common.service.activity.ActivityGiftsRankService;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.data.HeadwearService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.common.utils.DateUtil;
import com.starbuds.server.service.activity.dao.ActivityGiftsRankBindDAO;
import com.starbuds.server.service.activity.dao.ActivityGiftsRankReceiveDAO;
import com.starbuds.server.service.activity.dao.ActivityGiftsRankSendDAO;
import com.starbuds.server.service.activity.dao.ActivityGiftsSaveHisDAO;
import com.starbuds.server.service.activity.rocket.RocketProducer;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.beans.BeanUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 多礼物活动聚合接口
 */
@Service(version = "1.0.0")
@Slf4j
public class ActivityGiftsRankServiceImpl implements ActivityGiftsRankService {

    @Reference(version = "1.0.0")
    UserService userService;

    @Reference(version = "1.0.0")
    HeadwearService headwearService;

    @Resource
    ActivityGiftsRankBindDAO activityGiftsRankBindDAO;

    @Resource
    ActivityGiftsRankSendDAO activityGiftsRankSendDAO;

    @Resource
    ActivityGiftsRankReceiveDAO activityGiftsRankReceiveDAO;

    @Resource
    ActivityGiftsSaveHisDAO activityGiftsSaveHisDAO;

    @Resource
    IdentityService identityService;

    @Resource
    RocketProducer rocketProducer;


    public boolean isExistHis(Long orderId, Long activityId, Long giftId) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq("activityId", activityId);
        qw.eq("orderId", orderId);
        qw.eq("giftId", giftId);

        return activityGiftsSaveHisDAO.exist(qw);
    }

    /**
     * 插入或者更新多礼物数据
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean dealData(Long orderId, Long activityId, Long senderUserId, Long receiveUserId, Long total, Long giftId) {

        // 如果订单已经处理过，直接返回true
        if (isExistHis(orderId, activityId, giftId)) {
            return true;
        }

        // 当日没有数据就插入
        if (!currentDayIsExist(activityId, senderUserId, receiveUserId)) {
            return addCurrentDayRecord(orderId, activityId, senderUserId, receiveUserId, total, giftId);
        } else {
            return updateCurrentDayRecord(orderId, activityId, senderUserId, receiveUserId, total, giftId);
        }
    }

    /**
     * 更新当日记录
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean updateCurrentDayRecord(Long orderId, Long activityId, Long senderUserId, Long receiveUserId, Long total, Long giftId) {
        Integer dataInt = DateUtil.getIntDate();
        Long timestamp = System.currentTimeMillis();

        // 先把之前的值查出来
        QueryWrapper qw = new QueryWrapper();
        qw.eq("senderUserId", senderUserId);
        qw.eq("receiveUserId", receiveUserId);
        qw.eq("activityId", activityId);
        Long oldTotalValue = Optional.ofNullable(activityGiftsRankBindDAO.selectRelatedTotalValue(qw)).orElse(0L);

        // 更新关系榜
        if (activityGiftsRankBindDAO.updateTotal(total, activityId, senderUserId, receiveUserId, dataInt.longValue(), timestamp) == 0) {
            return false;
        }

        // 更新赠送榜
        if (activityGiftsRankSendDAO.updateTotal(total, activityId, senderUserId, dataInt.longValue(), timestamp) == 0) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            log.info("[updateCurrentDayRecord]更新赠送榜失败，事务回滚");
            return false;
        }

        // 更新收礼榜
        if (activityGiftsRankReceiveDAO.updateTotal(total, activityId, receiveUserId, dataInt.longValue(), timestamp) == 0) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            log.info("[updateCurrentDayRecord]更新收礼榜，事务回滚");
            return false;
        }

        // 添加历史记录
        ActivityGiftsSaveHisDO hisDO = new ActivityGiftsSaveHisDO();
        hisDO.setHisId(identityService.getId());
        hisDO.setActivityId(activityId);
        hisDO.setTotal(total);
        hisDO.setCreateTime(System.currentTimeMillis());
        hisDO.setOrderId(orderId);
        hisDO.setGiftId(giftId);
        if (activityGiftsSaveHisDAO.insert(hisDO) == 0) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            log.info("[updateCurrentDayRecord]插入历史记录，事务回滚");
            return false;
        }

        QueueActivityGiftsPaidMsg msg = new QueueActivityGiftsPaidMsg();
        msg.setActivityId(activityId);
        msg.setOrderId(orderId);
        msg.setReceiverUserId(receiveUserId);
        msg.setSenderUserId(senderUserId);

        msg.setRelatedTotalValue(oldTotalValue + total);

        rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.ACTIVITY_GIFTS_PAID, msg.serialize());
        return true;
    }


    /**
     * 添加当日记录
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean addCurrentDayRecord(Long orderId, Long activityId, Long senderUserId, Long receiveUserId, Long total, Long giftId) {

        Integer dataInt = DateUtil.getIntDate();
        Long timestamp = System.currentTimeMillis();
        Long id = identityService.getId();

        boolean isCurrentDaySendRecordIsExist = currentDaySendRecordIsExist(activityId, senderUserId);
        boolean isCurrentDayReceiveRecordIsExist = currentDayReceiveRecordIsExist(activityId, receiveUserId);

        // 先把之前的值查出来
        QueryWrapper qw = new QueryWrapper();
        qw.eq("senderUserId", senderUserId);
        qw.eq("receiveUserId", receiveUserId);
        qw.eq("activityId", activityId);
        Long oldTotalValue = Optional.ofNullable(activityGiftsRankBindDAO.selectRelatedTotalValue(qw)).orElse(0L);

        // 添加关系榜
        ActivityGiftsRankBindDO bindDO = new ActivityGiftsRankBindDO();
        bindDO.setActivityGiftsRankBindId(id);
        bindDO.setActivityId(activityId);
        bindDO.setReceiveUserId(receiveUserId);
        bindDO.setSenderUserId(senderUserId);
        bindDO.setCreateDate(dataInt);
        bindDO.setTotal(total);
        bindDO.setCreateTime(timestamp);
        bindDO.setUpdateTime(timestamp);
        if (activityGiftsRankBindDAO.insert(bindDO) == 0) {
            return false;
        }

        if (isCurrentDaySendRecordIsExist) {
            // 更新赠送榜
            if (activityGiftsRankSendDAO.updateTotal(total, activityId, senderUserId, dataInt.longValue(), timestamp) == 0) {
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                log.info("[updateCurrentDayRecord]更新赠送榜失败，事务回滚");
                return false;
            }
        } else {
            // 添加赠送榜
            ActivityGiftsRankSendDO sendDO = new ActivityGiftsRankSendDO();
            BeanUtils.copyProperties(bindDO, sendDO);
            sendDO.setActivityGiftsRankSendId(id);
            if (activityGiftsRankSendDAO.insert(sendDO) == 0) {
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                log.info("[addCurrentDayRecord]插入赠送榜，事务回滚");
                return false;
            }
        }

        if (isCurrentDayReceiveRecordIsExist) {
            // 更新收礼榜
            if (activityGiftsRankReceiveDAO.updateTotal(total, activityId, receiveUserId, dataInt.longValue(), timestamp) == 0) {
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                log.info("[updateCurrentDayRecord]更新收礼榜，事务回滚");
                return false;
            }
        } else {
            // 添加收礼榜
            ActivityGiftsRankReceiveDO receiveDO = new ActivityGiftsRankReceiveDO();
            BeanUtils.copyProperties(bindDO, receiveDO);
            receiveDO.setActivityGiftsRankReceiveId(id);
            if (activityGiftsRankReceiveDAO.insert(receiveDO) == 0) {
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                log.info("[addCurrentDayRecord]插入收礼榜，事务回滚");
                return false;
            }
        }

        // 添加历史记录
        ActivityGiftsSaveHisDO hisDO = new ActivityGiftsSaveHisDO();
        BeanUtils.copyProperties(bindDO, hisDO);
        hisDO.setHisId(id);
        hisDO.setOrderId(orderId);
        hisDO.setGiftId(giftId);
        if (activityGiftsSaveHisDAO.insert(hisDO) == 0) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            log.info("[addCurrentDayRecord]插入历史记录，事务回滚");
            return false;
        }

        QueueActivityGiftsPaidMsg msg = new QueueActivityGiftsPaidMsg();
        msg.setActivityId(activityId);
        msg.setOrderId(orderId);
        msg.setReceiverUserId(receiveUserId);
        msg.setSenderUserId(senderUserId);

        msg.setRelatedTotalValue(oldTotalValue + total);

        rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.ACTIVITY_GIFTS_PAID, msg.serialize());
        return true;
    }

    /**
     * 获取用户亲密值(或者什么值)
     */
    @Override
    public Long getTotalValueByUser(Long activityId, Long userId) {

        QueryWrapper qw = new QueryWrapper();
        qw.eq("activityId", activityId);
        qw.eq("senderUserId", userId);

        return Optional.ofNullable(activityGiftsRankSendDAO.selectTotalValue(qw)).orElse(0L);
    }

    /**
     * 首页排行榜
     */
    @Override
    public List<ActivityGiftsRankVO> getRankTopList(Long activityId, Integer type) {

        ActivityGiftsRankTypeEnum typeEnum = ActivityGiftsRankTypeEnum.toEnum(type);
        if (ActivityGiftsRankTypeEnum.TODAY == typeEnum) {

            return getRankList(activityId, DateUtil.getIntDate());
        }

        if (ActivityGiftsRankTypeEnum.YESTERDAY == typeEnum) {

            return getRankList(activityId, DateUtil.getIntDate(DateUtil.getYesterday(new Date())));
        }

        if (ActivityGiftsRankTypeEnum.TOTAL == typeEnum) {
            return getTotalRankList(activityId);
        }

        return Collections.EMPTY_LIST;
    }

    /**
     * 心动我的，我的心动排行榜
     */
    @Override
    public PageList<ActivityGiftsRankVO> getRankListByUser(Long activityId, Long userId, Integer type, Integer category, Integer pageIndex, Integer pageSize) {

        if (ActivityGiftsRankTypeEnum.TODAY.getValue() == type) {
            ActivityGiftsRankCategoryEnum categoryEnum = ActivityGiftsRankCategoryEnum.toEnum(category);
            if (categoryEnum == ActivityGiftsRankCategoryEnum.LIKE_ME) {
                return getReceiverRankList(activityId, userId, pageIndex, pageSize, DateUtil.getIntDate());
            }

            if (categoryEnum == ActivityGiftsRankCategoryEnum.MY_LIKE) {
                return getSenderRankList(activityId, userId, pageIndex, pageSize, DateUtil.getIntDate());
            }
        }

        if (ActivityGiftsRankTypeEnum.YESTERDAY.getValue() == type) {

            if (category == ActivityGiftsRankCategoryEnum.LIKE_ME.getValue()) {
                return getReceiverRankList(activityId, userId, pageIndex, pageSize, DateUtil.getIntDate(DateUtil.getYesterday(new Date())));
            }

            if (category == ActivityGiftsRankCategoryEnum.MY_LIKE.getValue()) {
                return getSenderRankList(activityId, userId, pageIndex, pageSize, DateUtil.getIntDate(DateUtil.getYesterday(new Date())));
            }
        }

        if (ActivityGiftsRankTypeEnum.TOTAL.getValue() == type) {

            if (category == ActivityGiftsRankCategoryEnum.LIKE_ME.getValue()) {
                return getTotalReceiverRankList(activityId, userId, pageIndex, pageSize);
            }

            if (category == ActivityGiftsRankCategoryEnum.MY_LIKE.getValue()) {
                return getTotalSenderRankList(activityId, userId, pageIndex, pageSize);
            }
        }

        return new PageList<>();
    }


    /**
     * 当日数据是否已存在
     */
    private boolean currentDayIsExist(Long activityId, Long senderUserId, Long receiveUserId) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq("activityId", activityId);
        qw.eq("senderUserId", senderUserId);
        qw.eq("receiveUserId", receiveUserId);
        qw.eq("createDate", DateUtil.getIntDate());

        return activityGiftsRankBindDAO.isExistCurrentDay(qw);
    }

    /**
     * 当日赠送数据是否已存在
     */
    private boolean currentDaySendRecordIsExist(Long activityId, Long senderUserId) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq("activityId", activityId);
        qw.eq("senderUserId", senderUserId);
        qw.eq("createDate", DateUtil.getIntDate());

        return activityGiftsRankSendDAO.isExistCurrentDay(qw);
    }

    /**
     * 当日收礼数据是否已存在
     */
    private boolean currentDayReceiveRecordIsExist(Long activityId, Long receiveUserId) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq("activityId", activityId);
        qw.eq("receiveUserId", receiveUserId);
        qw.eq("createDate", DateUtil.getIntDate());

        return activityGiftsRankReceiveDAO.isExistCurrentDay(qw);
    }

    /**
     * 获取某日排行榜
     */
    private List<ActivityGiftsRankVO> getRankList(Long activityId, Integer dateInt) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq("activityId", activityId);
        qw.eq("createDate", dateInt);

        qw.setPageCursor(0);
        qw.setPageSize(10);

        List<ActivityGiftsRankBindDO> rankPageList = activityGiftsRankBindDAO.selectRankPageList(qw);
        return getVOByDO(rankPageList);
    }


    /**
     * 获取总榜
     */
    private List<ActivityGiftsRankVO> getTotalRankList(Long activityId) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq("activityId", activityId);

        qw.setPageCursor(0);
        qw.setPageSize(10);

        List<ActivityGiftsRankBindDO> rankPageList = activityGiftsRankBindDAO.selectTotalRankPageList(qw);
        return getVOByDO(rankPageList);
    }

    /**
     * 获取某日我的心动排行榜
     * 今日：        Integer nowDate = DateUtil.getIntDate();
     * 昨日：        Integer yesterdayDate = DateUtil.getIntDate(DateUtil.getYesterday(new Date()));
     */
    private PageList<ActivityGiftsRankVO> getSenderRankList(Long activityId, Long userId, Integer pageIndex, Integer pageSize, Integer dateInt) {
        PageList<ActivityGiftsRankVO> page = new PageList<>(pageIndex, pageSize);

        QueryWrapper qw = new QueryWrapper();
        qw.eq("activityId", activityId);
        qw.eq("createDate", dateInt);
        qw.eq("senderUserId", userId);

        qw.setPageCursor(page.getPageCursor());
        qw.setPageSize(page.getPageSize());

        List<ActivityGiftsRankBindDO> rankPageList = activityGiftsRankBindDAO.selectRankPageList(qw);
        List<ActivityGiftsRankVO> resultPageList = getVOByDO(rankPageList);

        page.setList(resultPageList);
        page.setTotal(activityGiftsRankBindDAO.selectRankCount(qw));

        return page;
    }

    /**
     * 获取某日心动我的排行榜
     * * 今日：        Integer nowDate = DateUtil.getIntDate();
     * * 昨日：        Integer yesterdayDate = DateUtil.getIntDate(DateUtil.getYesterday(new Date()));
     */
    private PageList<ActivityGiftsRankVO> getReceiverRankList(Long activityId, Long userId, Integer pageIndex, Integer pageSize, Integer dateInt) {
        PageList<ActivityGiftsRankVO> page = new PageList<>(pageIndex, pageSize);

        QueryWrapper qw = new QueryWrapper();
        qw.eq("activityId", activityId);
        qw.eq("createDate", dateInt);
        qw.eq("receiveUserId", userId);

        qw.setPageCursor(page.getPageCursor());
        qw.setPageSize(page.getPageSize());

        List<ActivityGiftsRankBindDO> rankPageList = activityGiftsRankBindDAO.selectRankPageList(qw);
        List<ActivityGiftsRankVO> resultPageList = getVOByDO(rankPageList);

        page.setList(resultPageList);
        page.setTotal(activityGiftsRankBindDAO.selectRankCount(qw));
        return page;
    }

    /**
     * 获取我的心动总榜
     */
    private PageList<ActivityGiftsRankVO> getTotalSenderRankList(Long activityId, Long userId, Integer pageIndex, Integer pageSize) {
        PageList<ActivityGiftsRankVO> page = new PageList<>(pageIndex, pageSize);

        QueryWrapper qw = new QueryWrapper();
        qw.eq("activityId", activityId);
        qw.eq("senderUserId", userId);

        qw.setPageCursor(page.getPageCursor());
        qw.setPageSize(page.getPageSize());

        List<ActivityGiftsRankBindDO> rankPageList = activityGiftsRankBindDAO.selectTotalMyLikeRankPageList(qw);
        List<ActivityGiftsRankVO> resultPageList = getVOByDO(rankPageList);

        Integer total = activityGiftsRankBindDAO.selectTotalMyLikeRankCount(qw);

        page.setList(resultPageList);
        page.setTotal(total);
        return page;
    }

    /**
     * 获取心动我的总榜
     */
    private PageList<ActivityGiftsRankVO> getTotalReceiverRankList(Long activityId, Long userId, Integer pageIndex, Integer pageSize) {
        PageList<ActivityGiftsRankVO> page = new PageList<>(pageIndex, pageSize);

        QueryWrapper qw = new QueryWrapper();
        qw.eq("activityId", activityId);
        qw.eq("receiveUserId", userId);

        qw.setPageCursor(page.getPageCursor());
        qw.setPageSize(page.getPageSize());

        List<ActivityGiftsRankBindDO> rankPageList = activityGiftsRankBindDAO.selectTotalLikeMeRankPageList(qw);
        List<ActivityGiftsRankVO> resultPageList = getVOByDO(rankPageList);


        Integer total = activityGiftsRankBindDAO.selectTotalLikeMeRankCount(qw);

        page.setList(resultPageList);
        page.setTotal(total);
        return page;
    }

    private List<ActivityGiftsRankVO> getVOByDO(List<ActivityGiftsRankBindDO> rankList) {
        List<ActivityGiftsRankVO> list = new ArrayList<>();

        // 赠礼人
        Set<Long> senderUserIds = rankList.stream().map(ActivityGiftsRankBindDO::getSenderUserId).collect(Collectors.toSet());
        // 收礼人
        Set<Long> receiveUserIds = rankList.stream().map(ActivityGiftsRankBindDO::getReceiveUserId).collect(Collectors.toSet());

        List<Long> senderUserList = new ArrayList<>(senderUserIds);
        List<Long> receiveUserList = new ArrayList<>(receiveUserIds);

        List<Long> allUserIds = CollectionUtil.addAllIfNotContains(senderUserList, receiveUserList);

        Map<Long, UserBasicInfo> allUserBaseInfoMap = userService.getBasicInfoMap(CollectionUtil.newHashSet(allUserIds));
        Map<Long, HeadwearVO> headwearMap = headwearService.getUsersEnabledHeadwear(allUserIds);

        rankList
                .forEach(i -> {

                    ActivityGiftsRankVO vo = new ActivityGiftsRankVO();

                    GiftsRankUserItem receiverItem = new GiftsRankUserItem();
                    GiftsRankUserItem senderItem = new GiftsRankUserItem();

                    receiverItem.setUserAnimationFile(Optional.ofNullable(headwearMap.get(i.getReceiveUserId())).map(HeadwearVO::getAnimationFile).orElse(null));
                    receiverItem.setUserAvatar(Optional.ofNullable(allUserBaseInfoMap.get(i.getReceiveUserId())).map(UserBasicInfo::getUserAvatar).orElse(null));
                    receiverItem.setUserId(Optional.ofNullable(allUserBaseInfoMap.get(i.getReceiveUserId())).map(UserBasicInfo::getUserId).orElse(null));
                    receiverItem.setUserName(Optional.ofNullable(allUserBaseInfoMap.get(i.getReceiveUserId())).map(UserBasicInfo::getUserName).orElse(null));

                    senderItem.setUserAnimationFile(Optional.ofNullable(headwearMap.get(i.getSenderUserId())).map(HeadwearVO::getAnimationFile).orElse(null));
                    senderItem.setUserAvatar(Optional.ofNullable(allUserBaseInfoMap.get(i.getSenderUserId())).map(UserBasicInfo::getUserAvatar).orElse(null));
                    senderItem.setUserId(Optional.ofNullable(allUserBaseInfoMap.get(i.getSenderUserId())).map(UserBasicInfo::getUserId).orElse(null));
                    senderItem.setUserName(Optional.ofNullable(allUserBaseInfoMap.get(i.getSenderUserId())).map(UserBasicInfo::getUserName).orElse(null));

                    vo.setSendUser(senderItem);
                    vo.setReceiveUser(receiverItem);

                    vo.setValue(i.getTotal());

                    list.add(vo);
                });

        return list;
    }
}
