package com.starbuds.server.service.activity.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.activity.ActivityTypeEnum;
import com.starbuds.server.common.constant.activity.RankTypeEnum;
import com.starbuds.server.common.constant.activity.TriggerActionTypeEnum;
import com.starbuds.server.common.constant.activity.TriggerEventEnum;
import com.starbuds.server.common.constant.cache.RedisCacheKeys;
import com.starbuds.server.common.constant.common.AppModuleType;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.common.CommonStatus;
import com.starbuds.server.common.constant.finance.BagItemTypeEnum;
import com.starbuds.server.common.pojo.activity.*;
import com.starbuds.server.common.pojo.activity.vo.ActivityJoinRankVO;
import com.starbuds.server.common.pojo.core.Tuple;
import com.starbuds.server.common.pojo.data.ChatBubbleVO;
import com.starbuds.server.common.pojo.data.MedalVO;
import com.starbuds.server.common.pojo.data.MountsVO;
import com.starbuds.server.common.pojo.data.PropsVO;
import com.starbuds.server.common.pojo.db.*;
import com.starbuds.server.common.pojo.finance.BagItem;
import com.starbuds.server.common.pojo.media.GiftItemVO;
import com.starbuds.server.common.pojo.user.AgentVO;
import com.starbuds.server.common.pojo.user.GiftWallItem;
import com.starbuds.server.common.pojo.user.UserBasicInfo;
import com.starbuds.server.common.pojo.user.WeekUser;
import com.starbuds.server.common.service.activity.ActivityAnniversaryService;
import com.starbuds.server.common.service.activity.ActivityService;
import com.starbuds.server.common.service.activity.GiftRankService;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.data.*;
import com.starbuds.server.common.service.finance.BagService;
import com.starbuds.server.common.service.finance.WalletService;
import com.starbuds.server.common.service.media.GiftService;
import com.starbuds.server.common.service.user.AgentService;
import com.starbuds.server.common.service.user.ScoreService;
import com.starbuds.server.common.service.user.UserGiftWallService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.service.activity.dao.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.beans.BeanUtils;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 活动服务实现
 *
 * @author Lay
 * @date 2020/2/22
 */
@Slf4j
@Service(version = "1.0.0")
public class ActivityServiceImpl implements ActivityService {

    @Reference(version = "1.0.0")
    UserService userService;

    @Reference(version = "1.0.0")
    GiftService giftService;

    @Reference(version = "1.0.0")
    PropsService propsService;

    @Reference(version = "1.0.0")
    MountsService mountsService;

    @Reference(version = "1.0.0")
    MedalService medalService;

    @Reference(version = "1.0.0")
    ChatBubbleService chatBubbleService;

    @Reference(version = "1.0.0")
    IdentityService identityService;

    @Reference(version = "1.0.0")
    BagService bagService;

    @Reference(version = "1.0.0")
    WalletService walletService;

    @Reference(version = "1.0.0")
    ScoreService scoreService;

    @Reference(version = "1.0.0")
    RedisService redisService;

    @Reference(version = "1.0.0")
    AgentService agentService;

    @Reference(version = "1.0.0")
    UserGiftWallService userGiftWallService;

    @Reference(version = "1.0.0")
    ActivityAnniversaryService activityAnniversaryService;

    @Resource
    ActivityDAO activityDAO;

    @Resource
    ActivityTriggerDAO activityTriggerDAO;

    @Resource
    ActivityTriggerUnionDAO activityTriggerUnionDAO;

    @Resource
    ActivityRankDAO activityRankDAO;

    @Resource
    ActivityGiftRankUserDAO activityGiftRankUserDAO;

    @Resource
    ActivityGiftRankAnchorDAO activityGiftRankAnchorDAO;

    @Resource
    ActivityGiftRankAssistsDAO activityGiftRankAssistsDAO;

    @Resource
    ActivityCollectDAO activityCollectDAO;

    @Resource
    ActivityReceiveHisDAO activityReceiveHisDAO;

    @Resource
    ActivityWeekDAO activityWeekDAO;

    @Resource
    GiftRankService giftRankService;

    /**
     * 获取活动
     *
     * @param activityId
     * @return
     */
    @Override
    public ActivityDO getActivity(Long activityId) {
        return activityDAO.selectById(activityId);
    }

    /**
     * 查询直播间活动
     *
     * @return
     */
    @Override
    public List<ActivityVO> getLiveList(Long userId) {
        List<ActivityDO> activityList = activityDAO.selectLiveActivityList(System.currentTimeMillis(), AppModuleType.LIVE);
        if (CollectionUtil.isNotEmpty(activityList)) {
            List<ActivityVO> list = new ArrayList<>();
            Integer recharged = null;
            for (ActivityDO activity : activityList) {
                Integer hasRecharged = Optional.ofNullable(activity.getHasRecharged()).orElse(BoolIntVal.Ignore);
                if (BoolIntVal.Ignore != hasRecharged) {
                    // 有首充要求
                    if (recharged == null) {
                        // 读取用户首充状态
                        recharged = userService.hasRecharged(userId) ? BoolIntVal.True : BoolIntVal.False;
                    }
                    if (!hasRecharged.equals(recharged)) {
                        continue;
                    }
                }

                ActivityVO activityVO = new ActivityVO();
                BeanUtils.copyProperties(activity, activityVO);
                list.add(activityVO);
            }
            return list;
        }
        return Collections.emptyList();
    }

    /**
     * 查询语聊房活动
     *
     * @param userId
     * @return
     */
    @Override
    public List<ActivityVO> getRtcList(Long userId, Long roomId) {

        List<ActivityVO> list = getRtcList(userId);

        list.forEach(val -> {

            if (activityAnniversaryService.isAnniversary(val.getActivityType())) {
                val.setCustomStyleType(BoolIntVal.True);
                ActivityJoinRankVO process = activityAnniversaryService.getProcess(userId, val.getActivityId(), roomId);
                // 魅力花房要获取子活动的图标
                if (ActivityTypeEnum.CHARMING_ROOM.equals(ActivityTypeEnum.toEnum(val.getActivityType()))) {
                    val.setActivityIcon(process.getIconImg());
                }
                val.setRankDetail(process);
            } else {
                val.setCustomStyleType(BoolIntVal.False);
            }

        });
        return list;
    }

    /**
     * 查询语聊房活动
     *
     * @param userId
     * @return
     */
    @Override
    public List<ActivityVO> getRtcList(Long userId) {
        List<ActivityDO> activityList = activityDAO.selectLiveActivityList(System.currentTimeMillis(), AppModuleType.RTC);
        if (CollectionUtil.isNotEmpty(activityList)) {
            int userWealthLevel = userService.getWealthLevelByUserId(userId);
            List<ActivityVO> list = new ArrayList<>();
            Integer recharged = null;
            for (ActivityDO activity : activityList) {
                Integer hasRecharged = Optional.ofNullable(activity.getHasRecharged()).orElse(BoolIntVal.Ignore);
                // 首充要求判断
                if (BoolIntVal.Ignore != hasRecharged) {
                    // 有首充要求
                    if (recharged == null) {
                        // 读取用户首充状态
                        recharged = userService.hasRecharged(userId) ? BoolIntVal.True : BoolIntVal.False;
                    }
                    if (!hasRecharged.equals(recharged)) {
                        continue;
                    }
                }

                int wealthLevelLimit = Optional.ofNullable(activity.getWealthLevelLimit()).orElse(0);
                if (wealthLevelLimit > 0) {
                    // 判断用户财富等级是否达标
                    if (userWealthLevel < wealthLevelLimit) {
                        continue;
                    }
                }

                ActivityVO activityVO = new ActivityVO();
                BeanUtils.copyProperties(activity, activityVO);
                list.add(activityVO);
            }
            return list;
        }
        return Collections.emptyList();
    }

    /**
     * 获取活动联合信息
     *
     * @param activityId
     * @return
     */
    @Override
    public List<ActivityTriggerUnionVO> getActivityUnion(Long activityId) {
        List<ActivityTriggerUnionDO> activityTriggerUnionList = activityTriggerUnionDAO.selectAllByActivityId(activityId);

        if (CollectionUtil.isNotEmpty(activityTriggerUnionList)) {
            List<ActivityTriggerUnionVO> list = new ArrayList<>();
            Set<Long> triggerIdSet = new HashSet<>();
            for (ActivityTriggerUnionDO activityTriggerUnion : activityTriggerUnionList) {
                ActivityTriggerUnionVO activityTriggerUnionVO = new ActivityTriggerUnionVO();
                activityTriggerUnionVO.setTriggerId(activityTriggerUnion.getTriggerId());
                activityTriggerUnionVO.setUnionName(activityTriggerUnion.getUnionName());
                triggerIdSet.add(activityTriggerUnion.getTriggerId());
                list.add(activityTriggerUnionVO);
            }

            // 查询动作组
            if (CollectionUtil.isNotEmpty(triggerIdSet)) {
                List<ActivityTriggerDO> activityTriggerList = activityTriggerDAO.selectActionsByIds(new ArrayList<>(triggerIdSet));
                if (CollectionUtil.isNotEmpty(activityTriggerList)) {
                    // 礼物 id 缓存
                    Set<Long> giftIdSet = new HashSet<>();
                    // 道具 id 缓存
                    Set<Long> propsIdSet = new HashSet<>();
                    // 坐骑 id 缓存
                    Set<Long> mountsIdSet = new HashSet<>();
                    // 勋章 id 缓存
                    Set<Long> medalIdSet = new HashSet<>();
                    // 聊天气泡 id 缓存
                    Set<Long> bubbleIdSet = new HashSet<>();

                    // 触发器缓存
                    Map<Long, List<ActivityTrigger.Action>> triggerActionMap = new HashMap<>();
                    Map<Long, String> triggerNameMap = new HashMap<>();
                    for (ActivityTriggerDO activityTrigger : activityTriggerList) {
                        triggerNameMap.put(activityTrigger.getTriggerId(), activityTrigger.getTriggerName());
                        List<ActivityTrigger.Action> actions = JSON.parseArray(activityTrigger.getTriggerActions(), ActivityTrigger.Action.class);
                        if (CollectionUtil.isNotEmpty(actions)) {
                            triggerActionMap.put(activityTrigger.getTriggerId(), actions);
                            for (ActivityTrigger.Action action : actions) {
                                if (TriggerActionTypeEnum.GIVE_ITEM == TriggerActionTypeEnum.toEnum(action.getActionType())) {
                                    // 如果是赠送物品 需要取出物品的信息
                                    for (BagItem bagItem : action.getBagItems()) {
                                        BagItemTypeEnum bagItemType = BagItemTypeEnum.toEnum(bagItem.getItemType());
                                        if (bagItemType == null) {
                                            continue;
                                        }
                                        switch (bagItemType) {
                                            case GIFT:
                                                giftIdSet.add(bagItem.getItemId());
                                                break;
                                            case PROPS:
                                                propsIdSet.add(bagItem.getItemId());
                                                break;
                                            case MOUNTS:
                                                mountsIdSet.add(bagItem.getItemId());
                                                break;
                                            case MEDAL:
                                                medalIdSet.add(bagItem.getItemId());
                                                break;
                                            case CHAT_BUBBLE:
                                                bubbleIdSet.add(bagItem.getItemId());
                                                break;
                                            default:
                                                break;
                                        }
                                    }
                                }
                            }
                        }
                    }

                    // 处理礼物
                    Map<Long, GiftItemVO> giftMap = new HashMap<>();
                    if (CollectionUtil.isNotEmpty(giftIdSet)) {
                        giftMap = giftService.getGiftItemsByIds(new ArrayList<>(giftIdSet)).stream().collect(Collectors.toMap(GiftItemVO::getGiftId, Function.identity()));
                    }

                    // 处理道具
                    Map<Long, PropsVO> propsMap = new HashMap<>();
                    if (CollectionUtil.isNotEmpty(propsIdSet)) {
                        propsMap = propsService.getPropsByIds(new ArrayList<>(propsIdSet)).stream().collect(Collectors.toMap(PropsVO::getPropsId, Function.identity()));
                    }

                    // 处理坐骑
                    Map<Long, MountsVO> mountsMap = new HashMap<>();
                    if (CollectionUtil.isNotEmpty(mountsIdSet)) {
                        mountsMap = mountsService.getMountsListByIds(new ArrayList<>(mountsIdSet)).stream().collect(Collectors.toMap(MountsVO::getMountsId, Function.identity()));
                    }

                    // 处理勋章
                    Map<Long, MedalVO> medalMap = new HashMap<>();
                    if (CollectionUtil.isNotEmpty(medalIdSet)) {
                        medalMap = medalService.getMedalListByIds(new ArrayList<>(medalIdSet)).stream().collect(Collectors.toMap(MedalVO::getMedalId, Function.identity()));
                    }

                    // 处理聊天气泡
                    Map<Long, ChatBubbleVO> bubbleMap = new HashMap<>();
                    if (CollectionUtil.isNotEmpty(bubbleIdSet)) {
                        bubbleMap = chatBubbleService.getBubbleListByIds(new ArrayList<>(bubbleIdSet)).stream().collect(Collectors.toMap(ChatBubbleVO::getBubbleId, Function.identity()));
                    }

                    // 合并数据
                    for (Map.Entry<Long, List<ActivityTrigger.Action>> entry : triggerActionMap.entrySet()) {
                        for (ActivityTrigger.Action action : entry.getValue()) {
                            if (TriggerActionTypeEnum.GIVE_ITEM == TriggerActionTypeEnum.toEnum(action.getActionType())) {
                                // 如果是赠送物品 需要取出物品的信息
                                for (BagItem bagItem : action.getBagItems()) {
                                    BagItemTypeEnum bagItemType = BagItemTypeEnum.toEnum(bagItem.getItemType());
                                    if (bagItemType == null) {
                                        continue;
                                    }
                                    switch (bagItemType) {
                                        case GIFT:
                                            bagItem.setItem(giftMap.get(bagItem.getItemId()));
                                            break;
                                        case PROPS:
                                            bagItem.setItem(propsMap.get(bagItem.getItemId()));
                                            break;
                                        case MOUNTS:
                                            bagItem.setItem(mountsMap.get(bagItem.getItemId()));
                                            break;
                                        case MEDAL:
                                            bagItem.setItem(medalMap.get(bagItem.getItemId()));
                                            break;
                                        case CHAT_BUBBLE:
                                            bagItem.setItem(bubbleMap.get(bagItem.getItemId()));
                                            break;
                                        default:
                                            break;
                                    }
                                }
                            }
                        }
                    }

                    // 组装数据
                    for (ActivityTriggerUnionVO activityTriggerUnion : list) {
                        activityTriggerUnion.setTriggerName(triggerNameMap.get(activityTriggerUnion.getTriggerId()));
                        activityTriggerUnion.setActions(triggerActionMap.get(activityTriggerUnion.getTriggerId()));
                    }
                }
            }

            return list;
        }

        return Collections.emptyList();
    }

    @Override
    public ActivityVO getActivityByType(Integer activityType) {
        return activityDAO.selectByType(activityType);
    }

    @Override
    public boolean isActivityExist(Long activityId) {
        return activityDAO.countById(activityId) > 0;
    }

    @Override
    public ActivityRankVO getRankTopN(Long activityId, int topN) {

        ActivityRankVO rankVO = new ActivityRankVO();
        List<RankAgentItem> agentList = new ArrayList<>();
        List<RankUserItem> anchorList = new ArrayList<>();
        List<RankUserItem> userList = new ArrayList<>();

        ActivityDO activityDO = activityDAO.selectById(activityId);

        if (activityDO == null) {
            rankVO.setAgentList(agentList);
            rankVO.setAnchorList(anchorList);
            rankVO.setUserList(userList);
            return rankVO;
        }

        Long curTime = System.currentTimeMillis();

        // 是否结束，结束从db查询，否则从缓存查询
        boolean isEnd = false;
        if (activityDO.getActivityEndTime() < curTime) {
            isEnd = true;
        }

        List<ActivityRankDO> rankList = activityRankDAO.selectByActivityId(activityId);
        if (!isEnd || CollectionUtil.isEmpty(rankList)) {
            for (RankTypeEnum rankType : RankTypeEnum.values()) {
                if (RankTypeEnum.INVESTOR_LIST == rankType) {
                    continue;
                }
                Set<Tuple> tuples = redisService.zReverseRangeWithScores(RedisCacheKeys.getActivityRankKey(activityId, rankType.getValue()), 0, topN - 1);
                switch (rankType) {
                    case AGENT_LIST:
                        if (!CollectionUtils.isEmpty(tuples)) {
                            for (Tuple t : tuples) {
                                Long agentId = Convert.toLong(t.getElement());
                                RankAgentItem agentItem = new RankAgentItem();
                                agentItem.setAgentId(agentId);
                                agentItem.setScore(Convert.toLong(t.getScore()));
                                AgentVO agentDetail = agentService.getAgentDetail(agentId);
                                if (agentDetail != null) {
                                    agentItem.setAgentName(agentDetail.getAgentName());
                                    agentItem.setAgentLogo(agentDetail.getAgentLogo());
                                    agentItem.setAgentNo(agentDetail.getAgentNo());
                                }
                                agentList.add(agentItem);
                            }

                            agentList.sort(Comparator.comparingLong(RankAgentItem::getScore).reversed());

                            int rankNo = 1;
                            for (RankAgentItem item : agentList) {
                                item.setRankNo(rankNo);
                                rankNo++;
                            }

                        }
                        break;
                    case ANCHOR_LIST:
                        if (!CollectionUtils.isEmpty(tuples)) {
                            for (Tuple t : tuples) {
                                Long userId = Convert.toLong(t.getElement());
                                RankUserItem userItem = new RankUserItem();
                                userItem.setUserId(userId);
                                userItem.setScore(Convert.toLong(t.getScore()));
                                UserBasicInfo basicInfo = userService.getBasicInfo(userId);
                                if (basicInfo != null) {
                                    userItem.setUserName(basicInfo.getUserName());
                                    userItem.setUserAvatar(basicInfo.getUserAvatar());
                                    userItem.setUserNo(basicInfo.getUserNo());
                                }
                                anchorList.add(userItem);
                            }

                            anchorList.sort(Comparator.comparingLong(RankUserItem::getScore).reversed());
                            int rankNo = 1;
                            for (RankUserItem item : anchorList) {
                                item.setRankNo(rankNo);
                                rankNo++;
                            }
                        }
                        break;
                    case WEALTH_LIST:
                        if (!CollectionUtils.isEmpty(tuples)) {
                            for (Tuple t : tuples) {
                                Long userId = Convert.toLong(t.getElement());
                                RankUserItem userItem = new RankUserItem();
                                userItem.setUserId(userId);
                                userItem.setScore(Convert.toLong(t.getScore()));
                                UserBasicInfo basicInfo = userService.getBasicInfo(userId);
                                if (basicInfo != null) {
                                    userItem.setUserName(basicInfo.getUserName());
                                    userItem.setUserAvatar(basicInfo.getUserAvatar());
                                    userItem.setUserNo(basicInfo.getUserNo());
                                }
                                userList.add(userItem);
                            }

                            userList.sort(Comparator.comparingLong(RankUserItem::getScore).reversed());
                            int rankNo = 1;
                            for (RankUserItem item : userList) {
                                item.setRankNo(rankNo);
                                rankNo++;
                            }
                        }
                        break;
                    default:
                        break;
                }
            }
        } else {
            for (RankTypeEnum rankType : RankTypeEnum.values()) {
                if (RankTypeEnum.INVESTOR_LIST == rankType) {
                    continue;
                }
                List<ActivityRankDO> list = rankList.stream().filter(e -> rankType.getValue() == e.getRankType()).collect(Collectors.toList());
                switch (rankType) {
                    case AGENT_LIST:
                        if (!CollectionUtils.isEmpty(list)) {
                            for (ActivityRankDO t : list) {
                                Long agentId = t.getRankTargetId();
                                RankAgentItem agentItem = new RankAgentItem();
                                agentItem.setAgentId(agentId);
                                agentItem.setScore(Convert.toLong(t.getScore()));
                                AgentVO agentDetail = agentService.getAgentDetail(agentId);
                                if (agentDetail != null) {
                                    agentItem.setAgentName(agentDetail.getAgentName());
                                    agentItem.setAgentLogo(agentDetail.getAgentLogo());
                                    agentItem.setAgentNo(agentDetail.getAgentNo());
                                }
                                agentList.add(agentItem);
                            }

                            agentList.sort(Comparator.comparingLong(RankAgentItem::getScore).reversed());

                            int rankNo = 1;
                            for (RankAgentItem item : agentList) {
                                item.setRankNo(rankNo);
                                rankNo++;
                            }

                        }
                        break;
                    case ANCHOR_LIST:
                        if (!CollectionUtils.isEmpty(list)) {
                            for (ActivityRankDO t : list) {
                                Long userId = Convert.toLong(t.getRankTargetId());
                                RankUserItem userItem = new RankUserItem();
                                userItem.setUserId(userId);
                                userItem.setScore(Convert.toLong(t.getScore()));
                                UserBasicInfo basicInfo = userService.getBasicInfo(userId);
                                if (basicInfo != null) {
                                    userItem.setUserName(basicInfo.getUserName());
                                    userItem.setUserAvatar(basicInfo.getUserAvatar());
                                    userItem.setUserNo(basicInfo.getUserNo());
                                }
                                anchorList.add(userItem);
                            }

                            anchorList.sort(Comparator.comparingLong(RankUserItem::getScore).reversed());
                            int rankNo = 1;
                            for (RankUserItem item : anchorList) {
                                item.setRankNo(rankNo);
                                rankNo++;
                            }
                        }
                        break;
                    case WEALTH_LIST:
                        if (!CollectionUtils.isEmpty(list)) {
                            for (ActivityRankDO t : list) {
                                Long userId = Convert.toLong(t.getRankTargetId());
                                RankUserItem userItem = new RankUserItem();
                                userItem.setUserId(userId);
                                userItem.setScore(Convert.toLong(t.getScore()));
                                UserBasicInfo basicInfo = userService.getBasicInfo(userId);
                                if (basicInfo != null) {
                                    userItem.setUserName(basicInfo.getUserName());
                                    userItem.setUserAvatar(basicInfo.getUserAvatar());
                                    userItem.setUserNo(basicInfo.getUserNo());
                                }
                                userList.add(userItem);
                            }

                            userList.sort(Comparator.comparingLong(RankUserItem::getScore).reversed());
                            int rankNo = 1;
                            for (RankUserItem item : userList) {
                                item.setRankNo(rankNo);
                                rankNo++;
                            }
                        }
                        break;
                    default:
                        break;
                }
            }
        }

        List<RankUserItem> investorList = new ArrayList<>();
        if (CollectionUtil.isNotEmpty(anchorList)) {
            for (RankUserItem anchor : anchorList) {
                String anchorInvestorKey = RedisCacheKeys.getAnchorActivityRankKey(activityId, RankTypeEnum.INVESTOR_LIST.getValue(), anchor.getUserId());
                Set<Tuple> tuples = redisService.zReverseRangeWithScores(anchorInvestorKey, 0, 10);
                if (CollectionUtil.isNotEmpty(tuples)) {
                    for (Tuple t : tuples) {
                        Long userId = Convert.toLong(t.getElement());
                        RankUserItem userItem = new RankUserItem();
                        userItem.setUserId(userId);
                        userItem.setScore(Convert.toLong(t.getScore()));
                        UserBasicInfo basicInfo = userService.getBasicInfo(userId);
                        if (basicInfo != null) {
                            userItem.setUserName(basicInfo.getUserName());
                            userItem.setUserAvatar(basicInfo.getUserAvatar());
                            userItem.setUserNo(basicInfo.getUserNo());
                        }
                        investorList.add(userItem);
                    }

                    investorList.sort(Comparator.comparingLong(RankUserItem::getScore).reversed());
                    int rankNo = 1;
                    for (RankUserItem item : investorList) {
                        item.setRankNo(rankNo);
                        rankNo++;
                    }
                }
                anchor.setInvestorList(investorList);
            }
        }

        rankVO.setAgentList(agentList);
        rankVO.setAnchorList(anchorList);
        rankVO.setUserList(userList);
        return rankVO;
    }

    @Override
    public ActivityInfo getInfo(Long activityId) {
        ActivityInfo info = activityDAO.selectInfo(activityId);
        if (info != null && info.getActivityType() != null) {
            ActivityTypeEnum typeEnum = ActivityTypeEnum.toEnum(info.getActivityType());
            switch (typeEnum) {
                case COLLECT:
                    // 收集活动  活动：一对多
                    List<ActivityCollectDO> collects = activityCollectDAO.selectCollects(activityId);
                    List<ActivityCollectVO> collectVOList = new ArrayList<>();

                    for (ActivityCollectDO item : collects) {

                        ActivityCollectVO collectVO = new ActivityCollectVO();
                        BeanUtils.copyProperties(item, collectVO);

                        // 活动需收集的礼物
                        List<GiftItem> giftItems = new ArrayList<>();
                        if (StringUtils.isNotEmpty(item.getCollectGifts())) {
                            List<GiftItem> giftItemList = JSON.parseArray(item.getCollectGifts(), GiftItem.class);
                            List<GiftItemVO> giftItemsByIds = giftService.getGiftItemsByIds(giftItemList.stream().map(GiftItem::getGiftId).collect(Collectors.toList()));
                            for (GiftItemVO itemVO : giftItemsByIds) {
                                GiftItem giftItem = new GiftItem();

                                List<GiftItem> giftItemFilter = giftItemList.stream().filter(e -> e.getGiftId().equals(itemVO.getGiftId())).limit(1).collect(Collectors.toList());

                                giftItem.setGiftId(CollectionUtils.isEmpty(giftItemFilter) ? null : giftItemFilter.get(0).getGiftId());
                                giftItem.setQuantity(CollectionUtils.isEmpty(giftItemFilter) ? null : giftItemFilter.get(0).getQuantity());
                                giftItem.setGiftName(itemVO.getGiftName());
                                giftItem.setGiftIcon(itemVO.getGiftIcon());
                                giftItem.setGiftPriceType(itemVO.getGiftPriceType());
                                giftItem.setGiftPrice(itemVO.getGiftPrice());
                                giftItems.add(giftItem);
                            }
                            collectVO.setGiftItems(giftItems);
                        }

                        // 奖励
                        if (StringUtils.isNotEmpty(item.getRewards())) {
                            // 礼物 id 缓存
                            Set<Long> giftIdSet = new HashSet<>();
                            // 道具 id 缓存
                            Set<Long> propsIdSet = new HashSet<>();
                            // 坐骑 id 缓存
                            Set<Long> mountsIdSet = new HashSet<>();
                            // 勋章 id 缓存
                            Set<Long> medalIdSet = new HashSet<>();
                            // 聊天气泡 id 缓存
                            Set<Long> bubbleIdSet = new HashSet<>();

                            List<RewardItem> rewardItemList = JSON.parseArray(item.getRewards(), RewardItem.class);
                            for (RewardItem rewardItem : rewardItemList) {
                                if (TriggerActionTypeEnum.GIVE_ITEM == TriggerActionTypeEnum.toEnum(rewardItem.getActionType())) {
                                    // 如果是赠送物品 需要取出物品的信息
                                    for (BagItem bagItem : rewardItem.getBagItems()) {
                                        BagItemTypeEnum bagItemType = BagItemTypeEnum.toEnum(bagItem.getItemType());
                                        if (bagItemType == null) {
                                            continue;
                                        }
                                        switch (bagItemType) {
                                            case GIFT:
                                                giftIdSet.add(bagItem.getItemId());
                                                break;
                                            case PROPS:
                                                propsIdSet.add(bagItem.getItemId());
                                                break;
                                            case MOUNTS:
                                                mountsIdSet.add(bagItem.getItemId());
                                                break;
                                            case MEDAL:
                                                medalIdSet.add(bagItem.getItemId());
                                                break;
                                            case CHAT_BUBBLE:
                                                bubbleIdSet.add(bagItem.getItemId());
                                                break;
                                            default:
                                                break;
                                        }
                                    }
                                }
                            }

                            // 处理礼物
                            Map<Long, GiftItemVO> giftMap = new HashMap<>();
                            if (CollectionUtil.isNotEmpty(giftIdSet)) {
                                giftMap = giftService.getGiftItemsByIds(new ArrayList<>(giftIdSet)).stream().collect(Collectors.toMap(GiftItemVO::getGiftId, Function.identity()));
                            }

                            // 处理道具
                            Map<Long, PropsVO> propsMap = new HashMap<>();
                            if (CollectionUtil.isNotEmpty(propsIdSet)) {
                                propsMap = propsService.getPropsByIds(new ArrayList<>(propsIdSet)).stream().collect(Collectors.toMap(PropsVO::getPropsId, Function.identity()));
                            }

                            // 处理坐骑
                            Map<Long, MountsVO> mountsMap = new HashMap<>();
                            if (CollectionUtil.isNotEmpty(mountsIdSet)) {
                                mountsMap = mountsService.getMountsListByIds(new ArrayList<>(mountsIdSet)).stream().collect(Collectors.toMap(MountsVO::getMountsId, Function.identity()));
                            }

                            // 处理勋章
                            Map<Long, MedalVO> medalMap = new HashMap<>();
                            if (CollectionUtil.isNotEmpty(medalIdSet)) {
                                medalMap = medalService.getMedalListByIds(new ArrayList<>(medalIdSet)).stream().collect(Collectors.toMap(MedalVO::getMedalId, Function.identity()));
                            }

                            // 处理聊天气泡
                            Map<Long, ChatBubbleVO> bubbleMap = new HashMap<>();
                            if (CollectionUtil.isNotEmpty(bubbleIdSet)) {
                                bubbleMap = chatBubbleService.getBubbleListByIds(new ArrayList<>(bubbleIdSet)).stream().collect(Collectors.toMap(ChatBubbleVO::getBubbleId, Function.identity()));
                            }

                            for (RewardItem rewardItem : rewardItemList) {
                                if (TriggerActionTypeEnum.GIVE_ITEM == TriggerActionTypeEnum.toEnum(rewardItem.getActionType())) {
                                    // 如果是赠送物品 需要取出物品的信息
                                    for (BagItem bagItem : rewardItem.getBagItems()) {
                                        BagItemTypeEnum bagItemType = BagItemTypeEnum.toEnum(bagItem.getItemType());
                                        if (bagItemType == null) {
                                            continue;
                                        }
                                        switch (bagItemType) {
                                            case GIFT:
                                                bagItem.setItem(giftMap.get(bagItem.getItemId()));
                                                break;
                                            case PROPS:
                                                bagItem.setItem(propsMap.get(bagItem.getItemId()));
                                                break;
                                            case MOUNTS:
                                                bagItem.setItem(mountsMap.get(bagItem.getItemId()));
                                                break;
                                            case MEDAL:
                                                bagItem.setItem(medalMap.get(bagItem.getItemId()));
                                                break;
                                            case CHAT_BUBBLE:
                                                bagItem.setItem(bubbleMap.get(bagItem.getItemId()));
                                                break;
                                            default:
                                                break;
                                        }
                                    }
                                }
                            }
                            collectVO.setRewardItems(rewardItemList);
                            collectVOList.add(collectVO);
                        }
                    }
                    info.setCollects(collectVOList);
                    break;
                case AGGREGATION:
                    break;
                default:
                    break;
            }
        }
        return info;
    }

    /**
     * 获取礼物活动用户排行
     *
     * @param activityId
     * @param num
     * @return
     */
    @Override
    public List<ActivityGiftRankUser> getGiftUserRankList(Long activityId, int num) {
        return activityGiftRankUserDAO.selectAll(activityId, num);
    }

    /**
     * 获取礼物活动主播排行
     *
     * @param activityId
     * @param num
     * @return
     */
    @Override
    public List<ActivityGiftRankUser> getGiftAnchorRankList(Long activityId, int num) {
        List<ActivityGiftRankUser> list = activityGiftRankAnchorDAO.selectAll(activityId, num);
        for (ActivityGiftRankUser rankUser : list) {
            rankUser.setAssistsUsers(Arrays.asList(activityGiftRankAssistsDAO.selectInvestor(activityId, rankUser.getUserId())));
        }
        return list;
    }

    /**
     * 获取活动达标情况
     *
     * @param userId
     * @param collectId
     */
    @Override
    public List<GiftWallItem> getCollectStatus(Long userId, String collectId) {
        ActivityCollectDO collectDO = activityCollectDAO.selectById(collectId);
        if (collectDO != null) {
            ActivityDO activityDO = activityDAO.selectById(collectDO.getActivityId());
            if (activityDO != null && activityDO.getActivityBeginTime() != null && activityDO.getActivityEndTime() != null) {
                List<GiftItem> giftItemList = JSON.parseArray(collectDO.getCollectGifts(), GiftItem.class);
                List<Long> giftIds = giftItemList.stream().map(GiftItem::getGiftId).collect(Collectors.toList());
                // 礼物墙
                List<GiftWallItem> wallItems = userGiftWallService.getWallItems(userId,
                        Convert.toLong(DateUtil.format(new Date(activityDO.getActivityBeginTime()), "yyyyMMdd")),
                        Convert.toLong(DateUtil.format(new Date(activityDO.getActivityEndTime()), "yyyyMMdd")),
                        giftIds);

                // 礼物墙map
                Map<Long, GiftWallItem> wallItemMap = wallItems.stream().collect(Collectors.toMap(GiftWallItem::getGiftId, Function.identity()));
                // 礼物
                List<GiftItemVO> giftItemsByIds = giftService.getGiftItemsByIds(giftItemList.stream().map(GiftItem::getGiftId).collect(Collectors.toList()));
                // 礼物Map
                Map<Long, GiftItemVO> giftItemMap = giftItemsByIds.stream().collect(Collectors.toMap(GiftItemVO::getGiftId, Function.identity()));

                // 结果集
                List<GiftWallItem> resultList = new ArrayList<>();
                giftItemList.forEach(e -> {
                    GiftWallItem giftWallItem = wallItemMap.get(e.getGiftId());
                    GiftItemVO giftItemVO = giftItemMap.get(e.getGiftId());
                    if (giftWallItem == null) {
                        giftWallItem = new GiftWallItem();
                        giftWallItem.setGiftId(e.getGiftId());
                        giftWallItem.setQuantitySum(0);
                    }
                    giftWallItem.setRequireQuantity(e.getQuantity());
                    giftWallItem.setGiftName(giftItemVO.getGiftName());
                    giftWallItem.setGiftIcon(giftItemVO.getGiftIcon());
                    giftWallItem.setGiftPriceType(giftItemVO.getGiftPriceType());
                    giftWallItem.setGiftPrice(giftItemVO.getGiftPrice());
                    resultList.add(giftWallItem);
                });
                return resultList;
            }
        }
        return Collections.EMPTY_LIST;
    }

    /**
     * 领取奖励
     *
     * @param userId
     * @param collectId
     * @param activityBeginTime
     * @param activityEndTime
     * @return
     */
    @Override
    public boolean receiveReward(Long userId, String collectId, Long activityBeginTime, Long activityEndTime) {
        // 达标情况
        ActivityCollectDO collectDO = activityCollectDAO.selectById(collectId);
        if (collectDO != null) {
            // 目标
            List<GiftItem> itemList = JSON.parseArray(collectDO.getCollectGifts(), GiftItem.class);
            if (CollectionUtil.isEmpty(itemList)) {
                log.info("未配置任何礼物");
                return false;
            }
            List<Long> giftIds = itemList.stream().map(GiftItem::getGiftId).collect(Collectors.toList());
            // 实际达标情况
            List<GiftWallItem> wallItems = userGiftWallService.getWallItems(userId,
                    Convert.toLong(DateUtil.format(new Date(activityBeginTime), "yyyyMMdd")),
                    Convert.toLong(DateUtil.format(new Date(activityEndTime), "yyyyMMdd")),
                    giftIds);
            if (CollectionUtils.isEmpty(wallItems)) {
                log.info("未收集任何礼物");
                return false;
            }

            // 达标且获取到锁
            if (isMatch(userId, collectId)) {
                boolean lock = redisService.setIfAbsent(String.format("%s:%d", collectId, userId), "", 30, TimeUnit.SECONDS);
                // 抢锁
                if (lock) {
                    Long refId = identityService.getId();
                    // 创建领取记录
                    ActivityReceiveHisDO hisDO = new ActivityReceiveHisDO();
                    hisDO.setActivityId(collectDO.getActivityId());
                    hisDO.setCollectId(collectId);
                    hisDO.setBizId(refId);
                    hisDO.setUserId(userId);
                    hisDO.setCreateTime(System.currentTimeMillis());
                    hisDO.setRewards(collectDO.getRewards());
                    hisDO.setStatus(CommonStatus.Normal);
                    activityReceiveHisDAO.insert(hisDO);

                    // 领取动作
                    List<RewardItem> rewardItemList = JSON.parseArray(collectDO.getRewards(), RewardItem.class);
                    for (RewardItem item : rewardItemList) {
                        TriggerActionTypeEnum triggerActionType = TriggerActionTypeEnum.toEnum(item.getActionType());
                        String remarks = String.format("收集活动%s领取奖励", collectDO.getCollectName());
                        switch (triggerActionType) {
                            case GIVE_ITEM:
                                for (BagItem bagItem : item.getBagItems()) {
                                    bagService.addItem(userId, bagItem.getItemId(), bagItem.getItemType(), bagItem.getQuantity(), bagItem.getExpireTime().equals(-1L) ? -1L : System.currentTimeMillis() + bagItem.getExpireTime(), remarks, refId);
                                }
                                break;
                            case GIVE_COIN:
                                walletService.addCoin(userId, item.getWalletType(), item.getWalletAmount(), remarks, refId);
                                break;
                            case GIVE_SCORE:
                                scoreService.addScore(userId, item.getScoreType(), item.getScoreValue(), remarks, refId, 0L);
                                break;
                            default:
                                break;
                        }
                    }
                    return true;
                }
            }
            return false;
        }
        return false;
    }

    /**
     * 是否达成
     *
     * @param userId
     * @param collectId
     * @return
     */
    @Override
    public boolean isMatch(Long userId, String collectId) {
        ActivityCollectDO collectDO = activityCollectDAO.selectById(collectId);
        if (collectDO == null) {
            log.info("未找到活动");
            return false;
        }
        ActivityDO activityDO = activityDAO.selectById(collectDO.getActivityId());
        if (activityDO == null) {
            log.info("未找到活动");
            return false;
        }
        // 目标
        List<GiftItem> itemList = JSON.parseArray(collectDO.getCollectGifts(), GiftItem.class);
        if (CollectionUtil.isEmpty(itemList)) {
            log.info("未配置收集的礼物");
            return false;
        }
        List<Long> giftIds = itemList.stream().map(GiftItem::getGiftId).collect(Collectors.toList());
        // 实际达标情况
        List<GiftWallItem> wallItems = userGiftWallService.getWallItems(userId,
                Convert.toLong(DateUtil.format(new Date(activityDO.getActivityBeginTime()), "yyyyMMdd")),
                Convert.toLong(DateUtil.format(new Date(activityDO.getActivityEndTime()), "yyyyMMdd")),
                giftIds);
        if (CollectionUtil.isEmpty(wallItems)) {
            log.info("未收到活动的礼物");
            return false;
        }

        boolean isMatch = true;
        for (GiftItem item : itemList) {
            // 未达标
            if (wallItems.stream().filter(e -> e.getGiftId().equals(item.getGiftId())).mapToInt(GiftWallItem::getQuantitySum).sum() < item.getQuantity()) {
                isMatch = false;
                break;
            }
        }
        return isMatch;
    }

    /**
     * 判断收集活动是否存在
     *
     * @param collectId
     * @return
     */
    @Override
    public boolean isCollectExist(String collectId) {
        return activityCollectDAO.countById(collectId) > 0;
    }

    /**
     * 获取收集活动
     *
     * @param collectId
     * @return
     */
    @Override
    public ActivityCollectDO getCollect(String collectId) {
        return activityCollectDAO.selectById(collectId);
    }

    /**
     * 判断是否领取
     *
     * @param userId
     * @param collectId
     * @return
     */
    @Override
    public boolean isReceive(Long userId, String collectId) {
        return activityCollectDAO.countByUserId(collectId, userId) > 0;
    }

    /**
     * 获取我的排行
     *
     * @param activityId
     * @param userId
     * @return
     */
    @Override
    public RankItem getMyRank(Long activityId, Long userId) {
        RankItem rankItem = activityGiftRankUserDAO.selectByActivityId(activityId, userId);
        if (rankItem == null) {
            rankItem = new RankItem();
            rankItem.setRankScore(0L);
            rankItem.setUserId(userId);
            UserBasicInfo basicInfo = userService.getBasicInfo(userId);
            rankItem.setUserNo(basicInfo != null ? basicInfo.getUserNo() : null);
            rankItem.setUserName(basicInfo != null ? basicInfo.getUserName() : "");
            rankItem.setUserAvatar(basicInfo != null ? basicInfo.getUserAvatar() : "");
        }
        return rankItem;
    }

    /**
     * 获取等级奖励
     *
     * @param scoreType
     */
    @Override
    public List<ActivityTriggerVO> getLevelReward(Integer scoreType) {
        List<ActivityTriggerVO> triggerVOList = new ArrayList<>();

        List<ActivityTriggerDO> list = activityTriggerDAO.selectByEvent(TriggerEventEnum.SCORE_LEVEL_UP.getValue(), System.currentTimeMillis());
        List<ActivityTriggerDO> triggerList = list.stream().filter(e -> e.getTriggerConditions().contains(Convert.toStr(scoreType))).collect(Collectors.toList());
        if (CollectionUtil.isNotEmpty(triggerList)) {

            // 礼物 id 缓存
            Set<Long> giftIdSet = new HashSet<>();
            // 道具 id 缓存
            Set<Long> propsIdSet = new HashSet<>();
            // 坐骑 id 缓存
            Set<Long> mountsIdSet = new HashSet<>();
            // 勋章 id 缓存
            Set<Long> medalIdSet = new HashSet<>();
            // 聊天气泡 id 缓存
            Set<Long> bubbleIdSet = new HashSet<>();

            for (ActivityTriggerDO trigger : triggerList) {
                List<ActivityTrigger.Action> actions = JSON.parseArray(trigger.getTriggerActions(), ActivityTrigger.Action.class);
                if (CollectionUtil.isNotEmpty(actions)) {
                    for (ActivityTrigger.Action action : actions) {
                        TriggerActionTypeEnum actionTypeEnum = TriggerActionTypeEnum.toEnum(action.getActionType());
                        switch (actionTypeEnum) {
                            case GIVE_ITEM:
                                List<BagItem> bagItems = action.getBagItems();
                                if (CollectionUtil.isNotEmpty(bagItems)) {
                                    for (BagItem bagItem : bagItems) {
                                        BagItemTypeEnum bagItemType = BagItemTypeEnum.toEnum(bagItem.getItemType());
                                        if (bagItemType == null) {
                                            continue;
                                        }
                                        switch (bagItemType) {
                                            case GIFT:
                                                giftIdSet.add(bagItem.getItemId());
                                                break;
                                            case MOUNTS:
                                                mountsIdSet.add(bagItem.getItemId());
                                                break;
                                            case PROPS:
                                                propsIdSet.add(bagItem.getItemId());
                                                break;
                                            case MEDAL:
                                                medalIdSet.add(bagItem.getItemId());
                                                break;
                                            case CHAT_BUBBLE:
                                                bubbleIdSet.add(bagItem.getItemId());
                                                break;
                                            default:
                                                break;
                                        }
                                    }
                                }
                            case GIVE_COIN:
                                break;
                            case GIVE_SCORE:
                                break;
                            default:
                                break;
                        }
                    }
                }
            }

            // 处理礼物
            Map<Long, GiftItemVO> giftMap = new HashMap<>();
            if (CollectionUtil.isNotEmpty(giftIdSet)) {
                giftMap = giftService.getGiftItemsByIds(new ArrayList<>(giftIdSet)).stream().collect(Collectors.toMap(GiftItemVO::getGiftId, Function.identity()));
            }

            // 处理道具
            Map<Long, PropsVO> propsMap = new HashMap<>();
            if (CollectionUtil.isNotEmpty(propsIdSet)) {
                propsMap = propsService.getPropsByIds(new ArrayList<>(propsIdSet)).stream().collect(Collectors.toMap(PropsVO::getPropsId, Function.identity()));
            }

            // 处理坐骑
            Map<Long, MountsVO> mountsMap = new HashMap<>();
            if (CollectionUtil.isNotEmpty(mountsIdSet)) {
                mountsMap = mountsService.getMountsListByIds(new ArrayList<>(mountsIdSet)).stream().collect(Collectors.toMap(MountsVO::getMountsId, Function.identity()));
            }

            // 处理勋章
            Map<Long, MedalVO> medalMap = new HashMap<>();
            if (CollectionUtil.isNotEmpty(medalIdSet)) {
                medalMap = medalService.getMedalListByIds(new ArrayList<>(medalIdSet)).stream().collect(Collectors.toMap(MedalVO::getMedalId, Function.identity()));
            }

            // 处理聊天气泡
            Map<Long, ChatBubbleVO> bubbleMap = new HashMap<>();
            if (CollectionUtil.isNotEmpty(bubbleIdSet)) {
                bubbleMap = chatBubbleService.getBubbleListByIds(new ArrayList<>(bubbleIdSet)).stream().collect(Collectors.toMap(ChatBubbleVO::getBubbleId, Function.identity()));
            }

            for (ActivityTriggerDO trigger : triggerList) {
                List<ActivityTrigger.Action> actions = JSON.parseArray(trigger.getTriggerActions(), ActivityTrigger.Action.class);
                if (CollectionUtil.isNotEmpty(actions)) {
                    for (ActivityTrigger.Action action : actions) {
                        if (TriggerActionTypeEnum.GIVE_ITEM == TriggerActionTypeEnum.toEnum(action.getActionType())) {
                            // 如果是赠送物品 需要取出物品的信息
                            for (BagItem bagItem : action.getBagItems()) {
                                BagItemTypeEnum bagItemType = BagItemTypeEnum.toEnum(bagItem.getItemType());
                                if (bagItemType == null) {
                                    continue;
                                }
                                log.info("bagItem {}", JSON.toJSONString(bagItem));
                                switch (bagItemType) {
                                    case GIFT:
                                        bagItem.setName(CollectionUtil.isNotEmpty(giftMap) ? giftMap.get(bagItem.getItemId()).getGiftName() : "");
                                        bagItem.setUrl(CollectionUtil.isNotEmpty(giftMap) ? giftMap.get(bagItem.getItemId()).getGiftIcon() : "");
                                        break;
                                    case PROPS:
                                        bagItem.setName(CollectionUtil.isNotEmpty(propsMap) ? propsMap.get(bagItem.getItemId()).getPropsName() : "");
                                        bagItem.setUrl(CollectionUtil.isNotEmpty(propsMap) ? propsMap.get(bagItem.getItemId()).getPropsIcon() : "");
                                        break;
                                    case MOUNTS:
                                        bagItem.setName(CollectionUtil.isNotEmpty(mountsMap) ? mountsMap.get(bagItem.getItemId()).getMountsName() : "");
                                        bagItem.setUrl(CollectionUtil.isNotEmpty(mountsMap) ? mountsMap.get(bagItem.getItemId()).getMountsPreview() : "");
                                        break;
                                    case MEDAL:
                                        bagItem.setName(CollectionUtil.isNotEmpty(medalMap) ? medalMap.get(bagItem.getItemId()).getMedalName() : "");
                                        bagItem.setUrl(CollectionUtil.isNotEmpty(medalMap) ? medalMap.get(bagItem.getItemId()).getMedalIcon() : "");
                                        break;
                                    case CHAT_BUBBLE:
                                        bagItem.setName(CollectionUtil.isNotEmpty(bubbleMap) ? bubbleMap.get(bagItem.getItemId()).getBubbleName() : "");
                                        bagItem.setUrl(CollectionUtil.isNotEmpty(bubbleMap) ? bubbleMap.get(bagItem.getItemId()).getBubbleImage() : "");
                                        break;
                                    default:
                                        break;
                                }
                            }
                        }
                    }
                }
                ActivityTriggerVO vo = new ActivityTriggerVO();
                BeanUtils.copyProperties(trigger, vo);
                vo.setActionJSONObject(actions);
                triggerVOList.add(vo);
            }
        }
        return triggerVOList;
    }

    /**
     * 获取周星礼物
     *
     * @return
     */
    @Override
    public List<GiftWeekItem> getWeekGifts() {
        String gifts = activityWeekDAO.selectWeekGifts();
        if (StringUtils.isNotEmpty(gifts)) {
            List<GiftItem> giftItems = JSON.parseArray(gifts, GiftItem.class);
            if (CollectionUtil.isNotEmpty(giftItems)) {
                List<GiftWeekItem> weekItems = new ArrayList<>();
                for (GiftItem giftItem : giftItems) {
                    GiftWeekItem weekItem = new GiftWeekItem();
                    BeanUtils.copyProperties(giftItem, weekItem);
                    weekItems.add(weekItem);
                }
                return weekItems.stream().sorted(Comparator.comparingLong(GiftWeekItem::getGiftPrice)).collect(Collectors.toList());
            }
        }
        return Collections.EMPTY_LIST;
    }

    /**
     * 获取标签id
     *
     * @return
     */
    @Override
    public Long getTagId() {
        return activityWeekDAO.selectTagId();
    }

    /**
     * 获取周星礼物  上期
     *
     * @return
     */
    @Override
    public List<GiftWeekItem> getWeekGiftsPrevious() {
        String gifts = activityWeekDAO.selectLastWeekGifts();
        if (StringUtils.isNotEmpty(gifts)) {
            List<GiftItem> giftItems = JSON.parseArray(gifts, GiftItem.class);
            if (CollectionUtil.isNotEmpty(giftItems)) {
                // 礼物id
                Set<Long> giftIds = giftItems.stream().map(GiftItem::getGiftId).collect(Collectors.toSet());
                // 周星榜
                Map<Long, GiftRankItem> giftWeekTop = giftRankService.getGiftWeekStarRankWeekTop(giftIds, true);
                // 壕星榜
                Map<Long, GiftRankItem> giftRichTop = giftRankService.getGiftRichStarRankWeekTop(giftIds, true);
                List<GiftWeekItem> weekItems = new ArrayList<>();
                for (GiftItem giftItem : giftItems) {
                    GiftWeekItem weekItem = new GiftWeekItem();
                    BeanUtils.copyProperties(giftItem, weekItem);
                    // 周星榜
                    if (CollectionUtil.isNotEmpty(giftWeekTop)) {
                        GiftRankItem giftRankItem = giftWeekTop.get(giftItem.getGiftId());
                        if (giftRankItem != null) {
                            // 榜首用户
                            WeekUser weekTopUser = new WeekUser();
                            weekTopUser.setUserId(giftRankItem.getUserId());
                            weekTopUser.setUserName(giftRankItem.getUserName());
                            weekTopUser.setQuantity(giftRankItem.getQuantity());
                            weekTopUser.setUserAvatar(giftRankItem.getUserAvatar());
                            weekItem.setWeekTopUser(weekTopUser);
                        }
                    }
                    // 壕星榜
                    if (CollectionUtil.isNotEmpty(giftRichTop)) {
                        GiftRankItem giftRankItem = giftRichTop.get(giftItem.getGiftId());
                        if (giftRankItem != null) {
                            // 榜首用户
                            WeekUser richTopUser = new WeekUser();
                            richTopUser.setUserId(giftRankItem.getUserId());
                            richTopUser.setUserName(giftRankItem.getUserName());
                            richTopUser.setQuantity(giftRankItem.getQuantity());
                            richTopUser.setUserAvatar(giftRankItem.getUserAvatar());
                            weekItem.setRichTopUser(richTopUser);
                        }
                    }

                    weekItems.add(weekItem);
                }
                return weekItems.stream().sorted(Comparator.comparingLong(GiftWeekItem::getGiftPrice)).collect(Collectors.toList());
            }
        }
        return Collections.EMPTY_LIST;
    }

}
