package com.starbuds.server.service.activity.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUtil;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.pojo.activity.SignInDateItem;
import com.starbuds.server.common.pojo.db.DailySignInDO;
import com.starbuds.server.common.service.activity.DailySignInService;
import com.starbuds.server.service.activity.dao.DailySignInDAO;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Service;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * 每日签到服务
 *
 * @author Lay
 * @date 2020/9/23
 */
@Service(version = "1.0.0")
@Slf4j
public class DailySignInServiceImpl implements DailySignInService {

    @Resource
    DailySignInDAO dailySignInDAO;

    /**
     * 签到
     *
     * @param userId
     * @param channelId
     * @return
     */
    @Override
    public boolean signIn(Long userId, Long channelId) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
        Integer dateNum = Convert.toInt(sdf.format(new Date()));
        return signIn(userId, dateNum, channelId);
    }

    /**
     * 签到
     *
     * @param userId
     * @param dateNum
     * @param channelId
     * @return
     */
    @Override
    public boolean signIn(Long userId, Integer dateNum, Long channelId) {
        Long curTime = System.currentTimeMillis();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
        Integer signInDay = BoolIntVal.False;
        if (sdf.format(curTime).equals(String.valueOf(dateNum))) {
            signInDay = BoolIntVal.True;
        }

        DailySignInDO dailySignIn = new DailySignInDO();
        dailySignIn.setDateNum(dateNum);
        dailySignIn.setUserId(userId);
        dailySignIn.setChannelId(channelId);
        dailySignIn.setSignInDay(signInDay);
        dailySignIn.setSignInTime(curTime);

        try {
            dailySignInDAO.insert(dailySignIn);
            return true;
        } catch (Exception e) {
            log.error("签到失败", e);
        }
        return false;
    }

    /**
     * 是否签到
     *
     * @param userId
     * @param dateNum
     * @return
     */
    @Override
    public boolean isSigned(Long userId, Integer dateNum) {
        return dailySignInDAO.countById(dateNum, userId) > 0;
    }

    /**
     * 今日是否签到
     *
     * @param userId
     * @return
     */
    @Override
    public boolean isSigned(Long userId) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
        Integer dateNum = Convert.toInt(sdf.format(new Date()));
        return isSigned(userId, dateNum);
    }

    /**
     * 获取本周签到记录
     *
     * @param userId
     * @return
     */
    @Override
    public List<SignInDateItem> getWeekSignInItems(Long userId) {
        Calendar calendar = Calendar.getInstance();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
        Integer beginDate = Convert.toInt(sdf.format(DateUtil.beginOfWeek(calendar).getTime()));
        Integer endDate = Convert.toInt(sdf.format(DateUtil.endOfWeek(calendar).getTime()));

        List<DailySignInDO> list = dailySignInDAO.selectByDateRange(userId, beginDate, endDate);
        if (CollectionUtil.isNotEmpty(list)) {
            List<SignInDateItem> signInDateItems = new ArrayList<>();
            for (DailySignInDO dailySignIn : list) {
                SignInDateItem signInDateItem = new SignInDateItem();
                signInDateItem.setDateNum(dailySignIn.getDateNum());
                signInDateItem.setSignInTime(dailySignIn.getSignInTime());
                signInDateItems.add(signInDateItem);
            }
            return signInDateItems;
        }

        return Collections.emptyList();
    }
}
