package com.starbuds.server.service.activity.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import com.starbuds.server.common.constant.cache.RedisGiftKeys;
import com.starbuds.server.common.pojo.activity.GiftAirRankItem;
import com.starbuds.server.common.pojo.activity.GiftLoveRankItem;
import com.starbuds.server.common.pojo.activity.GiftRankItem;
import com.starbuds.server.common.pojo.data.HeadwearVO;
import com.starbuds.server.common.pojo.db.GiftAirHisDO;
import com.starbuds.server.common.pojo.db.GiftDO;
import com.starbuds.server.common.pojo.rtc.RtcRoom;
import com.starbuds.server.common.pojo.user.UserCardVO;
import com.starbuds.server.common.service.activity.GiftRankService;
import com.starbuds.server.common.service.data.HeadwearService;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.media.GiftService;
import com.starbuds.server.common.service.rtc.RtcRoomService;
import com.starbuds.server.common.service.user.RtcUserService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.service.activity.dao.ActivityWeekDAO;
import com.starbuds.server.service.activity.dao.GiftAirHisDAO;
import com.starbuds.server.service.activity.dao.GiftLoveHisDAO;
import com.starbuds.server.service.activity.provider.GiftRankProvider;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Author jiangxiaowei
 * @DateTime 2021/1/25 17:55
 * @Description 礼物排行榜
 */
@Slf4j
@Service(version = "1.0.0")
public class GiftRankServiceImpl implements GiftRankService {

    @Resource
    ActivityWeekDAO activityWeekDAO;

    @Resource
    GiftRankProvider giftRankProvider;

    @Resource
    GiftLoveHisDAO giftLoveHisDAO;

    @Resource
    GiftAirHisDAO giftAirHisDAO;

    @Reference(version = "1.0.0")
    RtcRoomService rtcRoomService;

    @Reference(version = "1.0.0")
    RtcUserService rtcUserService;

    @Reference(version = "1.0.0")
    RedisService redisService;

    @Reference(version = "1.0.0")
    UserService userService;

    @Reference(version = "1.0.0")
    GiftService giftService;

    @Reference(version = "1.0.0")
    HeadwearService headwearService;

    /**
     * 获取周星榜日榜
     *
     * @param giftId
     * @param start
     * @param end
     * @param isPrevious
     * @return
     */
    @Override
    public List<GiftRankItem> getGiftWeekStarRankDay(Long giftId, long start, long end, boolean isPrevious) {
        String[] rankTimes = isPrevious ? giftRankProvider.getPreviousRankTimes() : giftRankProvider.getRankTimes();
        return giftRankProvider.getRankItems(RedisGiftKeys.getGiftRankWeekStarDay(giftId, rankTimes[0]), start, end, giftId);
    }

    /**
     * 获取周星榜周榜
     *
     * @param giftId
     * @param start
     * @param end
     * @param isPrevious
     * @return
     */
    @Override
    public List<GiftRankItem> getGiftWeekStarRankWeek(Long giftId, long start, long end, boolean isPrevious) {
        String[] rankTimes = isPrevious ? giftRankProvider.getPreviousRankTimes() : giftRankProvider.getRankTimes();
        return giftRankProvider.getRankItems(RedisGiftKeys.getGiftRankWeekStarWeek(giftId, rankTimes[1]), start, end, giftId);
    }

    /**
     * 获取周星榜当期的榜单
     *
     * @param giftId
     * @param start
     * @param end
     * @return
     */
    @Override
    public List<GiftRankItem> getCurrentGiftWeekStarRankWeek(Long giftId, long start, long end, String weekId) {
        Long startDate = activityWeekDAO.selectStartDateById(weekId);
        if (startDate != null) {
            return giftRankProvider.getRankItems(RedisGiftKeys.getGiftRankWeekStarWeek(giftId, Convert.toStr(giftRankProvider.getWeek(startDate))), start, end, giftId);
        }
        return Collections.EMPTY_LIST;
    }

    /**
     * 获取周星榜月榜
     *
     * @param giftId
     * @param start
     * @param end
     * @param isPrevious
     * @return
     */
    @Override
    public List<GiftRankItem> getGiftWeekStarRankMonth(Long giftId, long start, long end, boolean isPrevious) {
        String[] rankTimes = isPrevious ? giftRankProvider.getPreviousRankTimes() : giftRankProvider.getRankTimes();
        return giftRankProvider.getRankItems(RedisGiftKeys.getGiftRankWeekStarMonth(giftId, rankTimes[2]), start, end, giftId);
    }

    /**
     * 获取壕星榜日榜
     *
     * @param giftId
     * @param start
     * @param end
     * @param isPrevious
     * @return
     */
    @Override
    public List<GiftRankItem> getGiftRichStarRankDay(Long giftId, long start, long end, boolean isPrevious) {
        String[] rankTimes = isPrevious ? giftRankProvider.getPreviousRankTimes() : giftRankProvider.getRankTimes();
        return giftRankProvider.getRankItems(RedisGiftKeys.getGiftRankRichStarDay(giftId, rankTimes[0]), start, end, giftId);
    }

    /**
     * 获取壕星榜当期的榜单
     *
     * @param giftId
     * @param start
     * @param end
     * @param weekId
     * @return
     */
    @Override
    public List<GiftRankItem> getCurrentGiftRichStarRankWeek(Long giftId, long start, long end, String weekId) {
        Long startDate = activityWeekDAO.selectStartDateById(weekId);
        if (startDate != null) {
            return giftRankProvider.getRankItems(RedisGiftKeys.getGiftRankRichStarWeek(giftId, Convert.toStr(giftRankProvider.getWeek(startDate))), start, end, giftId);
        }
        return Collections.EMPTY_LIST;
    }

    /**
     * 获取壕星榜周榜
     *
     * @param giftId
     * @param start
     * @param end
     * @param isPrevious
     * @return
     */
    @Override
    public List<GiftRankItem> getGiftRichStarRankWeek(Long giftId, long start, long end, boolean isPrevious) {
        String[] rankTimes = isPrevious ? giftRankProvider.getPreviousRankTimes() : giftRankProvider.getRankTimes();
        return giftRankProvider.getRankItems(RedisGiftKeys.getGiftRankRichStarWeek(giftId, rankTimes[1]), start, end, giftId);
    }

    /**
     * 获取周星榜周榜 top
     *
     * @param giftIds
     * @param isPrevious
     * @return
     */
    @Override
    public Map<Long, GiftRankItem> getGiftWeekStarRankWeekTop(Set<Long> giftIds, boolean isPrevious) {
        if (CollectionUtil.isEmpty(giftIds)) {
            return Collections.EMPTY_MAP;
        }
        String[] rankTimes = isPrevious ? giftRankProvider.getPreviousRankTimes() : giftRankProvider.getRankTimes();
        Map<Long, GiftRankItem> giftRankItemMap = new HashMap<>();
        for (Long giftId : giftIds) {
            List<GiftRankItem> rankItems = giftRankProvider.getRankItems(RedisGiftKeys.getGiftRankWeekStarWeek(giftId, rankTimes[1]), 0, 0, giftId);
            giftRankItemMap.put(giftId, CollectionUtil.isNotEmpty(rankItems) ? rankItems.get(0) : null);
        }
        return giftRankItemMap;
    }

    /**
     * 获取壕星榜周榜 top
     *
     * @param giftIds
     * @param isPrevious
     * @return
     */
    @Override
    public Map<Long, GiftRankItem> getGiftRichStarRankWeekTop(Set<Long> giftIds, boolean isPrevious) {
        if (CollectionUtil.isEmpty(giftIds)) {
            return Collections.EMPTY_MAP;
        }
        String[] rankTimes = isPrevious ? giftRankProvider.getPreviousRankTimes() : giftRankProvider.getRankTimes();
        Map<Long, GiftRankItem> giftRankItemMap = new HashMap<>();
        for (Long giftId : giftIds) {
            List<GiftRankItem> rankItems = giftRankProvider.getRankItems(RedisGiftKeys.getGiftRankRichStarWeek(giftId, rankTimes[1]), 0, 0, giftId);
            giftRankItemMap.put(giftId, CollectionUtil.isNotEmpty(rankItems) ? rankItems.get(0) : null);
        }
        return giftRankItemMap;
    }

    /**
     * 获取壕星榜月榜
     *
     * @param giftId
     * @param start
     * @param end
     * @param isPrevious
     * @return
     */
    @Override
    public List<GiftRankItem> getGiftRichStarRankMonth(Long giftId, long start, long end, boolean isPrevious) {
        String[] rankTimes = isPrevious ? giftRankProvider.getPreviousRankTimes() : giftRankProvider.getRankTimes();
        return giftRankProvider.getRankItems(RedisGiftKeys.getGiftRankRichStarMonth(giftId, rankTimes[2]), start, end, giftId);
    }

    /**
     * 获取周星榜
     *
     * @param giftId
     * @param userId
     * @param isPrevious
     * @return
     */
    @Override
    public GiftRankItem getGiftWeekStarRankItemDay(Long giftId, Long userId, boolean isPrevious) {
        GiftRankItem item = new GiftRankItem();
        String[] rankTimes = isPrevious ? giftRankProvider.getPreviousRankTimes() : giftRankProvider.getRankTimes();
        String redisKey = RedisGiftKeys.getGiftRankWeekStarDay(giftId, rankTimes[0]);
        Integer rankNo = Optional.ofNullable(redisService.zRevRank(redisKey, String.valueOf(userId))).map(Long::intValue).orElse(-1);
        Long amount = Optional.ofNullable(redisService.zScore(redisKey, String.valueOf(userId))).map(Convert::toLong).orElse(0L);
        List<UserCardVO> userCardVOList = userService.getNameAndAvatarByIds(Collections.singletonList(userId));
        if (!CollectionUtils.isEmpty(userCardVOList)) {
            UserCardVO userCardVO = userCardVOList.get(0);
            item.setUserName(userCardVO.getUserName());
            item.setUserAvatar(userCardVO.getUserAvatar());
            item.setWealthLevel(userCardVO.getWealthLevel());
        }

        GiftDO gift = giftService.getGift(giftId, null);
        item.setQuantity(amount);
        item.setGiftId(giftId);
        item.setGiftName(gift.getGiftName());
        item.setValue(item.getQuantity() * gift.getGiftPrice());
        item.setRankNo(rankNo + 1);
        item.setUserId(userId);
        return item;
    }

    /**
     * 获取周星榜
     *
     * @param giftId
     * @param userId
     * @param isPrevious
     * @return
     */
    @Override
    public GiftRankItem getGiftWeekStarRankItemWeek(Long giftId, Long userId, boolean isPrevious) {
        GiftRankItem item = new GiftRankItem();
        String[] rankTimes = isPrevious ? giftRankProvider.getPreviousRankTimes() : giftRankProvider.getRankTimes();
        String redisKey = RedisGiftKeys.getGiftRankWeekStarWeek(giftId, rankTimes[1]);
        Integer rankNo = Optional.ofNullable(redisService.zRevRank(redisKey, String.valueOf(userId))).map(Long::intValue).orElse(-1);
        Long quantity = Optional.ofNullable(redisService.zScore(redisKey, String.valueOf(userId))).map(Convert::toLong).orElse(0L);
        List<UserCardVO> userCardVOList = userService.getNameAndAvatarByIds(Collections.singletonList(userId));
        if (!CollectionUtils.isEmpty(userCardVOList)) {
            UserCardVO userCardVO = userCardVOList.get(0);
            item.setUserName(userCardVO.getUserName());
            item.setUserAvatar(userCardVO.getUserAvatar());
            item.setWealthLevel(userCardVO.getWealthLevel());
        }

        GiftDO gift = giftService.getGift(giftId, null);
        item.setQuantity(quantity);
        item.setGiftId(giftId);
        item.setRankNo(rankNo + 1);
        item.setUserId(userId);
        if (gift != null) {
            item.setGiftName(gift.getGiftName());
            item.setValue(item.getQuantity() * gift.getGiftPrice());
        }
        return item;
    }

    /**
     * 获取周星榜
     *
     * @param giftId
     * @param userId
     * @param isPrevious
     * @return
     */
    @Override
    public GiftRankItem getGiftWeekStarRankItemMonth(Long giftId, Long userId, boolean isPrevious) {
        GiftRankItem item = new GiftRankItem();
        String[] rankTimes = isPrevious ? giftRankProvider.getPreviousRankTimes() : giftRankProvider.getRankTimes();
        String redisKey = RedisGiftKeys.getGiftRankWeekStarMonth(giftId, rankTimes[2]);
        Integer rankNo = Optional.ofNullable(redisService.zRevRank(redisKey, String.valueOf(userId))).map(Long::intValue).orElse(-1);
        Long amount = Optional.ofNullable(redisService.zScore(redisKey, String.valueOf(userId))).map(Convert::toLong).orElse(0L);
        List<UserCardVO> userCardVOList = userService.getNameAndAvatarByIds(Collections.singletonList(userId));
        if (!CollectionUtils.isEmpty(userCardVOList)) {
            UserCardVO userCardVO = userCardVOList.get(0);
            item.setUserName(userCardVO.getUserName());
            item.setUserAvatar(userCardVO.getUserAvatar());
            item.setWealthLevel(userCardVO.getWealthLevel());
        }

        GiftDO gift = giftService.getGift(giftId, null);
        item.setQuantity(amount);
        item.setGiftId(giftId);
        item.setGiftName(gift.getGiftName());
        item.setValue(item.getQuantity() * gift.getGiftPrice());
        item.setRankNo(rankNo + 1);
        item.setUserId(userId);
        return item;
    }

    /**
     * 获取壕星榜
     *
     * @param giftId
     * @param userId
     * @param isPrevious
     * @return
     */
    @Override
    public GiftRankItem getGiftRichStarRankItemDay(Long giftId, Long userId, boolean isPrevious) {
        GiftRankItem item = new GiftRankItem();
        String[] rankTimes = isPrevious ? giftRankProvider.getPreviousRankTimes() : giftRankProvider.getRankTimes();
        String redisKey = RedisGiftKeys.getGiftRankRichStarDay(giftId, rankTimes[0]);
        Integer rankNo = Optional.ofNullable(redisService.zRevRank(redisKey, String.valueOf(userId))).map(Long::intValue).orElse(-1);
        Long amount = Optional.ofNullable(redisService.zScore(redisKey, String.valueOf(userId))).map(Convert::toLong).orElse(0L);
        List<UserCardVO> userCardVOList = userService.getNameAndAvatarByIds(Collections.singletonList(userId));
        if (!CollectionUtils.isEmpty(userCardVOList)) {
            UserCardVO userCardVO = userCardVOList.get(0);
            item.setUserName(userCardVO.getUserName());
            item.setUserAvatar(userCardVO.getUserAvatar());
            item.setWealthLevel(userCardVO.getWealthLevel());
        }

        GiftDO gift = giftService.getGift(giftId, null);
        item.setQuantity(amount);
        item.setGiftId(giftId);
        item.setGiftName(gift.getGiftName());
        item.setValue(item.getQuantity() * gift.getGiftPrice());
        item.setRankNo(rankNo + 1);
        item.setUserId(userId);
        return item;
    }


    /**
     * 获取壕星榜
     *
     * @param giftId
     * @param userId
     * @param isPrevious
     * @return
     */
    @Override
    public GiftRankItem getGiftRichStarRankItemWeek(Long giftId, Long userId, boolean isPrevious) {
        GiftRankItem item = new GiftRankItem();
        String[] rankTimes = isPrevious ? giftRankProvider.getPreviousRankTimes() : giftRankProvider.getRankTimes();
        String redisKey = RedisGiftKeys.getGiftRankRichStarWeek(giftId, rankTimes[1]);
        Integer rankNo = Optional.ofNullable(redisService.zRevRank(redisKey, String.valueOf(userId))).map(Long::intValue).orElse(-1);
        Long amount = Optional.ofNullable(redisService.zScore(redisKey, String.valueOf(userId))).map(Convert::toLong).orElse(0L);
        List<UserCardVO> userCardVOList = userService.getNameAndAvatarByIds(Collections.singletonList(userId));
        if (!CollectionUtils.isEmpty(userCardVOList)) {
            UserCardVO userCardVO = userCardVOList.get(0);
            item.setUserName(userCardVO.getUserName());
            item.setUserAvatar(userCardVO.getUserAvatar());
            item.setWealthLevel(userCardVO.getWealthLevel());
        }

        GiftDO gift = giftService.getGift(giftId, null);
        item.setQuantity(amount);
        item.setGiftId(giftId);
        item.setGiftName(gift.getGiftName());
        item.setValue(item.getQuantity() * gift.getGiftPrice());
        item.setRankNo(rankNo + 1);
        item.setUserId(userId);
        return item;
    }

    /**
     * 获取壕星榜
     *
     * @param giftId
     * @param userId
     * @param isPrevious
     * @return
     */
    @Override
    public GiftRankItem getGiftRichStarRankItemMonth(Long giftId, Long userId, boolean isPrevious) {
        GiftRankItem item = new GiftRankItem();
        String[] rankTimes = isPrevious ? giftRankProvider.getPreviousRankTimes() : giftRankProvider.getRankTimes();
        String redisKey = RedisGiftKeys.getGiftRankRichStarWeek(giftId, rankTimes[2]);
        Integer rankNo = Optional.ofNullable(redisService.zRevRank(redisKey, String.valueOf(userId))).map(Long::intValue).orElse(-1);
        Long amount = Optional.ofNullable(redisService.zScore(redisKey, String.valueOf(userId))).map(Convert::toLong).orElse(0L);
        List<UserCardVO> userCardVOList = userService.getNameAndAvatarByIds(Collections.singletonList(userId));
        if (!CollectionUtils.isEmpty(userCardVOList)) {
            UserCardVO userCardVO = userCardVOList.get(0);
            item.setUserName(userCardVO.getUserName());
            item.setUserAvatar(userCardVO.getUserAvatar());
            item.setWealthLevel(userCardVO.getWealthLevel());
        }

        GiftDO gift = giftService.getGift(giftId, null);
        item.setQuantity(amount);
        item.setGiftId(giftId);
        item.setGiftName(gift.getGiftName());
        item.setValue(item.getQuantity() * gift.getGiftPrice());
        item.setRankNo(rankNo + 1);
        item.setUserId(userId);
        return item;
    }

    /**
     * 获取礼物真爱榜
     *
     * @param giftId
     * @param num
     * @return
     */
    @Override
    public List<GiftLoveRankItem> getTrueLoveGiftRank(Long giftId, int num) {
        List<GiftLoveRankItem> rankItems = giftLoveHisDAO.selectListByGiftId(giftId, num);
        if (CollectionUtil.isNotEmpty(rankItems)) {
            // 送礼人
            Set<Long> senderUserIds = rankItems.stream().map(GiftLoveRankItem::getSenderUserId).collect(Collectors.toSet());
            // 赠礼人
            Set<Long> receiveUserIds = rankItems.stream().map(GiftLoveRankItem::getReceiveUserId).collect(Collectors.toSet());
            List<Long> senderUserList = new ArrayList<>(senderUserIds);
            List<Long> receiveUserList = new ArrayList<>(receiveUserIds);
            // 用户对应的房间id
            Map<Long, Long> roomIdMap = rtcUserService.getUserRoomIdMap(senderUserIds);
            // 房间信息
            Map<Long, RtcRoom> roomMap = rtcRoomService.getRoomMap(roomIdMap.values().stream().collect(Collectors.toSet()));
            // 用户id=>房间
            Map<Long, RtcRoom> userRoomMap = new HashMap<>();
            senderUserList.forEach(e -> {
                userRoomMap.put(e, roomMap.get(roomIdMap.get(e)));
            });
            List<Long> allUserIds = CollectionUtil.addAllIfNotContains(senderUserList, receiveUserList);
            // 头饰
            Map<Long, HeadwearVO> headwearMap = headwearService.getUsersEnabledHeadwear(new ArrayList<>(allUserIds));
            rankItems.forEach(e -> {
                e.setSenderUserAnimationFile(Optional.ofNullable(headwearMap.get(e.getSenderUserId())).map(HeadwearVO::getAnimationFile).orElse(null));
                e.setSenderUserRoomId(Optional.ofNullable(userRoomMap.get(e.getSenderUserId())).map(RtcRoom::getRoomId).orElse(null));
                e.setSenderUserRoomName(Optional.ofNullable(userRoomMap.get(e.getSenderUserId())).map(RtcRoom::getRoomName).orElse(null));
                e.setSenderUserRoomCover(Optional.ofNullable(userRoomMap.get(e.getSenderUserId())).map(RtcRoom::getRoomCover).orElse(null));
                e.setReceiveUserAnimationFile(Optional.ofNullable(headwearMap.get(e.getReceiveUserId())).map(HeadwearVO::getAnimationFile).orElse(null));
                e.setReceiveUserRoomId(Optional.ofNullable(userRoomMap.get(e.getReceiveUserId())).map(RtcRoom::getRoomId).orElse(null));
                e.setReceiveUserRoomName(Optional.ofNullable(userRoomMap.get(e.getReceiveUserId())).map(RtcRoom::getRoomName).orElse(null));
                e.setReceiveUserRoomCover(Optional.ofNullable(userRoomMap.get(e.getReceiveUserId())).map(RtcRoom::getRoomCover).orElse(null));
            });
        }
        return rankItems;
    }

    /**
     * 写入欧气榜
     *
     * @param airList
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean addGiftAirRank(List<GiftAirHisDO> airList) {
        return giftAirHisDAO.batchInsert(airList) > 0;
    }

    /**
     * 获取欧气榜
     *
     * @param num
     * @param startTime
     * @param endTime
     * @return
     */
    @Override
    public List<GiftAirRankItem> getGiftAirRank(int num, Long startTime, Long endTime) {
        List<GiftAirRankItem> airRankItems = giftAirHisDAO.selectGiftAirRank(num, startTime, endTime);
        List<Long> senderUserIds = airRankItems.stream().map(GiftAirRankItem::getSenderUserId).collect(Collectors.toList());
        List<Long> receiveUserIds = airRankItems.stream().map(GiftAirRankItem::getReceiveUserId).collect(Collectors.toList());
        List<Long> allUserIds = CollectionUtil.addAllIfNotContains(senderUserIds, receiveUserIds);
        // 头饰
        Map<Long, HeadwearVO> headwearMap = headwearService.getUsersEnabledHeadwear(new ArrayList<>(allUserIds));
        airRankItems.forEach(e -> {
            e.setSenderUserAnimationFile(Optional.ofNullable(headwearMap.get(e.getSenderUserId())).map(HeadwearVO::getAnimationFile).orElse(null));
            e.setReceiveUserAnimationFile(Optional.ofNullable(headwearMap.get(e.getReceiveUserId())).map(HeadwearVO::getAnimationFile).orElse(null));
        });
        return airRankItems;
    }

    /**
     * 获取欧皇
     *
     * @param startTime
     * @param endTime
     * @return
     */
    @Override
    public GiftAirRankItem getGiftAirTop(Long startTime, Long endTime) {
        GiftAirRankItem top = giftAirHisDAO.selectGiftAirTop(startTime, endTime);
        if (top != null) {
            // 头饰
            Map<Long, HeadwearVO> headwearMap = headwearService.getUsersEnabledHeadwear(Arrays.asList(top.getSenderUserId(), top.getReceiveUserId()));
            top.setSenderUserAnimationFile(Optional.ofNullable(headwearMap.get(top.getSenderUserId())).map(HeadwearVO::getAnimationFile).orElse(null));
            top.setReceiveUserAnimationFile(Optional.ofNullable(headwearMap.get(top.getReceiveUserId())).map(HeadwearVO::getAnimationFile).orElse(null));
        }
        return top;
    }
}
