package com.starbuds.server.service.activity.impl;

import com.starbuds.server.common.pojo.activity.ActivityGrabPeriod;
import com.starbuds.server.common.pojo.activity.ActivityGrabPeriodVO;
import com.starbuds.server.common.pojo.activity.GrabPlayerWinQuantity;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.service.activity.GrabActivityService;
import com.starbuds.server.service.activity.dao.ActivityGrabPeriodDAO;
import com.starbuds.server.service.activity.dao.ActivityGrabPeriodItemDAO;
import com.starbuds.server.service.activity.grab.GrabPeriodProvider;
import com.starbuds.server.service.activity.grab.GrabSettleProvider;
import org.apache.commons.collections.CollectionUtils;
import org.apache.dubbo.config.annotation.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 抢龙活动服务实现
 *
 * @author Lay
 * @date 2021/3/29
 */
@Service(version = "1.0.0")
public class GrabActivityServiceImpl implements GrabActivityService {

    @Resource
    GrabPeriodProvider grabPeriodProvider;

    @Resource
    GrabSettleProvider grabSettleProvider;

    @Resource
    ActivityGrabPeriodDAO activityGrabPeriodDAO;

    @Resource
    ActivityGrabPeriodItemDAO activityGrabPeriodItemDAO;

    /**
     * 获取进行中期数
     *
     * @param activityId
     * @return
     */
    @Override
    public ActivityGrabPeriodVO getGoingPeriod(long activityId) {
        // 获取最新进行中活动
        return grabPeriodProvider.getPeriod(activityId);
    }

    /**
     * 加入活动
     *
     * @param periodId
     * @param userId
     * @return
     */
    @Override
    public Long join(long periodId, long userId) {
        return grabPeriodProvider.join(periodId, userId);
    }

    /**
     * 判断期数是否存在
     *
     * @param periodId
     * @return
     */
    @Override
    public boolean isPeriodExists(long periodId) {
        return activityGrabPeriodDAO.countById(periodId) > 0;
    }

    /**
     * 获取用户参与的期数列表
     *
     * @param activityId
     * @param userId
     * @param pageIndex
     * @param pageSize
     * @return
     */
    @Override
    public PageList<ActivityGrabPeriod> getUserPeriods(long activityId, long userId, int pageIndex, int pageSize) {
        PageList<ActivityGrabPeriod> pageList = new PageList<>(pageIndex, pageSize);
        pageList.setTotal(activityGrabPeriodDAO.countUserPeriods(activityId, userId));

        if (pageList.getTotal() > 0) {
            List<ActivityGrabPeriod> list = activityGrabPeriodDAO.selectUserPeriods(activityId, userId, pageList.getPageCursor(), pageSize);
            if (CollectionUtils.isNotEmpty(list)) {
                // 查询
                List<Long> periodIds = list.stream().map(ActivityGrabPeriod::getPeriodId).collect(Collectors.toList());
                Map<Long, List<GrabPlayerWinQuantity>> map = new HashMap<>();
                List<GrabPlayerWinQuantity> winQuantityList = activityGrabPeriodItemDAO.selectWinQuantities(periodIds);
                for (GrabPlayerWinQuantity winQuantity : winQuantityList) {
                    if (!map.containsKey(winQuantity.getPeriodId())) {
                        map.put(winQuantity.getPeriodId(), new ArrayList<>());
                    }
                    map.get(winQuantity.getPeriodId()).add(winQuantity);
                }

                for (ActivityGrabPeriod period : list) {
                    period.setWinQuantityList(map.get(period.getPeriodId()));
                }
            }

            pageList.setList(list);
        }

        return pageList;
    }

    /**
     * 是否玩家
     *
     * @param periodId
     * @param userId
     * @return
     */
    @Override
    public boolean isPlayer(long periodId, long userId) {
        return activityGrabPeriodItemDAO.countById(periodId, userId) > 0;
    }

    /**
     * 结算
     *
     * @param periodId
     * @param userIds
     */
    @Override
    public void settle(Long periodId, List<Long> userIds) {
        grabSettleProvider.settleAsync(periodId, userIds);
    }
}
