package com.starbuds.server.service.activity.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.activity.ActivityLotteryRankTypeEnum;
import com.starbuds.server.common.constant.activity.LotteryRewardTypeEnum;
import com.starbuds.server.common.constant.data.BroadcastTypeEnum;
import com.starbuds.server.common.constant.finance.WalletTypeEnum;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rocketmq.RocketTopic;
import com.starbuds.server.common.pojo.activity.*;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;
import com.starbuds.server.common.pojo.data.PropsVO;
import com.starbuds.server.common.pojo.db.ActivityLotteryDO;
import com.starbuds.server.common.pojo.db.SendGiftSnapshotDO;
import com.starbuds.server.common.pojo.media.GiftItemVO;
import com.starbuds.server.common.pojo.mq.QueueLotteryGameCompleteMsg;
import com.starbuds.server.common.service.activity.LotteryActivityService;
import com.starbuds.server.common.service.data.BroadcastHisService;
import com.starbuds.server.common.service.data.PropsService;
import com.starbuds.server.common.service.finance.SendGiftSnapshotMsgService;
import com.starbuds.server.common.service.media.GiftService;
import com.starbuds.server.common.service.user.RtcUserService;
import com.starbuds.server.service.activity.dao.ActivityLotteryDAO;
import com.starbuds.server.service.activity.dao.ActivityLotteryHisDAO;
import com.starbuds.server.service.activity.dao.ActivityLotteryHisRewardsDAO;
import com.starbuds.server.service.activity.dao.SendGiftSnapshotDAO;
import com.starbuds.server.service.activity.lottery.LotteryProvider;
import com.starbuds.server.service.activity.lottery.LotteryResult;
import com.starbuds.server.service.activity.rocket.RocketProducer;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 抽奖活动服务实现
 *
 * @author Lay
 * @date 2020/2/18
 */
@Service(version = "1.0.0")
@Slf4j
public class LotteryActivityServiceImpl implements LotteryActivityService {

    @Autowired
    ActivityLotteryDAO activityLotteryDAO;

    @Autowired
    ActivityLotteryHisDAO activityLotteryHisDAO;

    @Autowired
    ActivityLotteryHisRewardsDAO activityLotteryHisRewardsDAO;

    @Autowired
    LotteryProvider lotteryProvider;

    @Reference(version = "1.0.0")
    GiftService giftService;

    @Reference(version = "1.0.0")
    PropsService propsService;

    @Reference(version = "1.0.0")
    SendGiftSnapshotMsgService sendGiftSnapshotMsgService;

    @Resource
    private SendGiftSnapshotDAO sendGiftSnapshotDAO;

    @Resource
    private RocketProducer rocketProducer;

    /**
     * 获取抽奖配置
     *
     * @param lotteryId
     * @return
     */
    @Override
    public ActivityLotteryDO getLottery(String lotteryId) {
        return activityLotteryDAO.selectById(lotteryId);
    }

    /**
     * 抽奖
     *
     * @param lotteryId
     * @param userId
     * @param times
     * @param channelId
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public ActivityLotteryResult lottery(String lotteryId, Long userId, Integer times, Long channelId) {
        LotteryResult lotteryResult = lotteryProvider.lottery(lotteryId, userId, times, channelId, false, userId);
        if (lotteryResult != null) {
            ActivityLotteryResult activityLotteryResult = new ActivityLotteryResult();
            activityLotteryResult.setBalance(lotteryResult.getBalance());

            if (lotteryResult.getBalance() < 0) {
                return activityLotteryResult;
            }

            List<LotteryReward> winRewards = lotteryResult.getWinRewards();
            if (CollectionUtil.isNotEmpty(winRewards)) {
                List<LotteryHisRewardVO> list = new ArrayList<>();

                for (LotteryReward lotteryReward : winRewards) {
                    LotteryHisRewardVO rewardVO = new LotteryHisRewardVO();
                    rewardVO.setRewardId(lotteryReward.getRewardId());
                    rewardVO.setRewardType(lotteryReward.getRewardType());
                    rewardVO.setQuantity(lotteryReward.getQuantity());
                    rewardVO.setRewardValue(lotteryReward.getRewardValue());
                    list.add(rewardVO);
                }

                // 异步结算
                lotteryProvider.settleAsync(lotteryResult.getLotteryName(), lotteryResult.getHisId(), userId, list, channelId, 0, null, null);

                activityLotteryResult.setWinRewards(list);
            } else {
                activityLotteryResult.setWinRewards(Collections.emptyList());
            }

            // 抽奖活动发出消息
            QueueLotteryGameCompleteMsg msg = new QueueLotteryGameCompleteMsg();
            msg.setLotteryId(lotteryId);
            msg.setTimes(times);
            msg.setChannelId(channelId);
            msg.setUserId(userId);
            msg.setHisId(msg.getHisId());
            rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.LOTTERY_GAME_COMPLETE, msg.serialize());
            return activityLotteryResult;
        }

        return null;
    }

    /**
     * 抽奖并赠送主播
     *
     * @param lotteryId
     * @param userId
     * @param times
     * @param channelId
     * @param targetUserId
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public ActivityLotteryResult lottery(String lotteryId, Long userId, Integer times, Long channelId, int giftSceneType, String channelName, Long targetUserId) {
        LotteryResult lotteryResult = lotteryProvider.lottery(lotteryId, userId, times, channelId, true, targetUserId);
        if (lotteryResult != null) {
            log.info("抽奖完成结果：{}", JSON.toJSONString(lotteryResult));
            ActivityLotteryResult activityLotteryResult = new ActivityLotteryResult();
            activityLotteryResult.setBalance(lotteryResult.getBalance());

            if (lotteryResult.getBalance() < 0) {
                return activityLotteryResult;
            }

            List<LotteryReward> winRewards = lotteryResult.getWinRewards();
            if (CollectionUtil.isNotEmpty(winRewards)) {
                List<LotteryHisRewardVO> list = new ArrayList<>();

                for (LotteryReward lotteryReward : winRewards) {
                    LotteryHisRewardVO rewardVO = new LotteryHisRewardVO();
                    rewardVO.setRewardId(lotteryReward.getRewardId());
                    rewardVO.setRewardType(lotteryReward.getRewardType());
                    rewardVO.setQuantity(lotteryReward.getQuantity());
                    list.add(rewardVO);
                }

                // 异步结算
                lotteryProvider.settleAsync(lotteryResult.getLotteryName(), lotteryResult.getHisId(), userId, list, channelId, giftSceneType, channelName, targetUserId);

                activityLotteryResult.setWinRewards(list);
            } else {
                activityLotteryResult.setWinRewards(Collections.emptyList());
            }

            return activityLotteryResult;
        }

        return null;
    }

    /**
     * 抽奖并赠送
     *
     * @param lotteryId
     * @param senderUserId    赠送人id
     * @param times
     * @param channelId
     * @param roomId
     * @param targetUserSeats
     * @param sourceOrderId   源头订单id
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public void lottery(String lotteryId, Long senderUserId, Integer times, Long channelId, Long roomId,
                        Map<Long, Integer> targetUserSeats, Long sourceOrderId) {
        List<LotteryUserRewards> userRewardsList = new ArrayList<>();
        List<Long> lotteryResultIds = new ArrayList<>(targetUserSeats.size());
        // userId => seatNo
        for (Map.Entry<Long, Integer> userSeat : targetUserSeats.entrySet()) {
            //抽奖
            LotteryResult lotteryResult = lotteryProvider.lottery(lotteryId, senderUserId, times, channelId, true, userSeat.getKey());
            if (lotteryResult != null) {
                lotteryResultIds.add(lotteryResult.getHisId());

                LotteryUserRewards userRewards = new LotteryUserRewards();
                userRewards.setTargetUserId(userSeat.getKey());
                userRewards.setLotteryName(lotteryResult.getLotteryName());
                userRewards.setSeatNo(userSeat.getValue());

                List<LotteryReward> winRewards = lotteryResult.getWinRewards();
                if (CollectionUtil.isNotEmpty(winRewards)) {
                    List<LotteryHisRewardVO> list = new ArrayList<>();

                    for (LotteryReward lotteryReward : winRewards) {
                        LotteryHisRewardVO rewardVO = new LotteryHisRewardVO();
                        rewardVO.setHisId(lotteryResult.getHisId());
                        rewardVO.setRewardId(lotteryReward.getRewardId());
                        rewardVO.setRewardType(lotteryReward.getRewardType());
                        rewardVO.setQuantity(lotteryReward.getQuantity());
                        list.add(rewardVO);
                    }

                    userRewards.setRewardList(list);
                }

                userRewardsList.add(userRewards);
            }
        }

        sendGiftSnapshotDAO.updateActivityLotteryHisId(sourceOrderId,
                StringUtils.join(lotteryResultIds, ","), System.currentTimeMillis());

        SendGiftSnapshotDO snapshotDO = sendGiftSnapshotDAO.selectActivityLotterySnap(sourceOrderId);
        sendGiftSnapshotMsgService.sendMQMsg(snapshotDO);

        if (CollectionUtil.isNotEmpty(userRewardsList)) {
            lotteryProvider.settleAsync(senderUserId, userRewardsList, channelId, roomId, targetUserSeats);
        }
    }

    /**
     * 抽奖并赠送-语音速配礼物
     *
     * @param lotteryId
     * @param userId
     * @param times         礼物数量
     * @param channelId
     * @param hisId
     * @param receiveUserId
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public void lotteryForVoiceChat(String lotteryId, Long userId, Integer times, Long channelId, Long hisId, Long receiveUserId) {
        LotteryResult lotteryResult = lotteryProvider.lottery(lotteryId, userId, times, channelId, true, receiveUserId);
        if (lotteryResult != null) {
            LotteryUserRewards userRewards = lotteryProvider.getLotteryUserRewards(receiveUserId, lotteryResult);
            lotteryProvider.settleAsyncForVoiceChat(userId, userRewards, channelId, hisId, receiveUserId);
        }
    }

    /**
     * 抽奖并赠送  私聊
     *
     * @param lotteryId     抽奖活动id
     * @param senderUserId  发送人id
     * @param quantity      礼物数量
     * @param channelId
     * @param orderId
     * @param receiveUserId 中奖人id
     */
    @Override
    public void lotteryForPrivateChat(String lotteryId, Long senderUserId, Integer quantity, Long channelId, Long orderId, Long receiveUserId) {
        LotteryResult lotteryResult = lotteryProvider.lottery(lotteryId, senderUserId, quantity, channelId, true, receiveUserId);
        if (lotteryResult != null) {
            LotteryUserRewards userRewards = lotteryProvider.getLotteryUserRewards(receiveUserId, lotteryResult);
            lotteryProvider.settleAsyncForPrivateChat(senderUserId, userRewards, channelId, orderId, receiveUserId);
        }
    }

    /**
     * 获取中奖记录
     *
     * @param lotteryId
     * @param num
     * @return
     */
    @Override
    public List<LotteryHisRewardVO> getLotteryHisRewards(String lotteryId, int num) {
        return activityLotteryHisRewardsDAO.selectHisRewards(lotteryId, num);
    }

    /**
     * 获取中奖纪录
     *
     * @param activityId
     * @param num
     * @return
     */
    @Override
    public List<LotteryHisRewardVO> getLotteryHisRewards(Long activityId, int num) {
        return activityLotteryHisRewardsDAO.selectHisRewardsByActivityId(activityId, num);
    }

    /**
     * 获取用户抽奖记录
     *
     * @param pageIndex
     * @param pageSize
     * @param lotteryId
     * @param userId
     * @return
     */
    @Override
    public PageList<ActivityLotteryHisVO> getLotteryHisList(int pageIndex, int pageSize, String lotteryId, Long userId) {
        PageList<ActivityLotteryHisVO> pageList = new PageList<>(pageIndex, pageSize);
        pageList.setList(activityLotteryHisDAO.selectAll(pageList.getPageCursor(), pageSize, null, lotteryId, userId));
        pageList.setTotal(activityLotteryHisDAO.count(null, lotteryId, userId));
        return pageList;
    }

    /**
     * 获取用户抽奖记录
     *
     * @param pageIndex
     * @param pageSize
     * @param activityId
     * @param userId
     * @return
     */
    @Override
    public PageList<ActivityLotteryHisVO> getLotteryHisList(int pageIndex, int pageSize, Long activityId, Long userId) {
        PageList<ActivityLotteryHisVO> pageList = new PageList<>(pageIndex, pageSize);
        pageList.setList(activityLotteryHisDAO.selectAll(pageList.getPageCursor(), pageSize, activityId, null, userId));
        pageList.setTotal(activityLotteryHisDAO.count(activityId, null, userId));
        return pageList;
    }

    /**
     * 获取用户中奖记录
     *
     * @param hisId
     * @param userId
     * @return
     */
    @Override
    public List<LotteryHisRewardVO> getLotteryHisRewardsListByHisId(Long hisId, Long userId) {
        return activityLotteryHisRewardsDAO.selectHisRewardsByHisId(hisId, userId);
    }

    /**
     * 获取用户中奖记录
     *
     * @param pageIndex
     * @param pageSize
     * @param lotteryId
     * @param userId
     * @return
     */
    @Override
    public PageList<LotteryHisRewardVO> getLotteryHisRewardsList(int pageIndex, int pageSize, String lotteryId, Long userId) {
        PageList<LotteryHisRewardVO> pageList = new PageList<>(pageIndex, pageSize);
        pageList.setList(activityLotteryHisRewardsDAO.selectAll(pageList.getPageCursor(), pageSize, null, lotteryId, userId));
        pageList.setTotal(activityLotteryHisRewardsDAO.count(null, lotteryId, userId));
        return pageList;
    }

    /**
     * 获取用户中奖记录
     *
     * @param pageIndex
     * @param pageSize
     * @param activityId
     * @param userId
     * @return
     */
    @Override
    public PageList<LotteryHisRewardVO> getLotteryHisRewardsList(int pageIndex, int pageSize, Long activityId, Long userId) {
        PageList<LotteryHisRewardVO> pageList = new PageList<>(pageIndex, pageSize);
        pageList.setList(activityLotteryHisRewardsDAO.selectAll(pageList.getPageCursor(), pageSize, activityId, null, userId));
        pageList.setTotal(activityLotteryHisRewardsDAO.count(activityId, null, userId));
        return pageList;
    }

    /**
     * 判断抽奖记录是否存在
     *
     * @param hisId
     * @param userId
     * @return
     */
    @Override
    public boolean isLotteryHisExist(Long hisId, Long userId) {
        return activityLotteryHisDAO.countById(hisId, userId) > 0;
    }

    @Override
    public List<LotteryRewardVO> getRewardItem(String lotteryId) {
        // 活动关联的奖项集合
        ActivityLotteryDO lotteryDO = activityLotteryDAO.selectById(lotteryId);

        if (lotteryDO == null) {
            return new ArrayList<>();
        }

        String lotteryReward = lotteryDO.getLotteryRewards();
        if (StringUtils.isEmpty(lotteryReward)) {
            return new ArrayList<>();
        }

        List<LotteryReward> rewards = JSON.parseArray(lotteryReward, LotteryReward.class);

        List<LotteryRewardVO> list = new ArrayList<>();
        if (!CollectionUtil.isEmpty(rewards)) {
            Map<Long, List<LotteryReward>> groupByReward = rewards.stream().collect(Collectors.groupingBy(LotteryReward::getRewardId));
            if (!CollectionUtil.isEmpty(groupByReward)) {
                for (Map.Entry<Long, List<LotteryReward>> entry : groupByReward.entrySet()) {

                    Long rewardId = entry.getKey();
                    List<LotteryReward> value = entry.getValue();
                    BigDecimal totalRatio = value.stream().map(e -> Convert.toBigDecimal(e.getRatio())).reduce(BigDecimal.ZERO, BigDecimal::add);
                    String rewardName = value.get(0).getRewardName();
                    Integer rewardType = value.get(0).getRewardType();
                    Long rewardValue = value.get(0).getRewardValue();

                    LotteryRewardVO vo = new LotteryRewardVO();
                    vo.setRatio(Convert.toDouble(totalRatio));
                    vo.setRewardId(rewardId);
                    vo.setRewardName(rewardName);
                    vo.setRewardType(rewardType);
                    vo.setRewardValue(rewardValue);
                    vo.setLotteryPriceType(lotteryDO.getLotteryPriceType());
                    vo.setLotteryPriceTypeName(WalletTypeEnum.toEnum(vo.getLotteryPriceType()).getName());

                    LotteryRewardTypeEnum rewardTypeEnum = LotteryRewardTypeEnum.toEnum(rewardType);

                    switch (rewardTypeEnum) {
                        case GIFT:
                            GiftItemVO gift = giftService.getGift(rewardId);
                            if (gift != null) {
                                vo.setRewardIcon(gift.getGiftIcon());
                            }
                            break;
                        case PROPS:
                            PropsVO props = propsService.getProps(rewardId);
                            if (props != null) {
                                vo.setRewardIcon(props.getPropsIcon());
                            }
                            break;
                        default:
                            break;
                    }
                    list.add(vo);
                }
            }
        }

        return list;
    }

    @Override
    public boolean isLotteryExist(String lotteryId) {
        return activityLotteryDAO.countById(lotteryId) > 0;
    }

    /**
     * 获取排行榜
     *
     * @param lotteryId
     * @param lotteryRankType
     * @param num
     * @return
     */
    @Override
    public List<ActivityLotteryRankItem> getLotteryRank(String lotteryId, ActivityLotteryRankTypeEnum lotteryRankType, int num) {
        Calendar calendar = Calendar.getInstance();

        long beginTime = 0L;
        long endTime = 0L;

        switch (lotteryRankType) {
            case Daily:
                beginTime = DateUtil.beginOfDay(calendar).getTimeInMillis();
                endTime = DateUtil.endOfDay(calendar).getTimeInMillis();
                break;
            case Week:
                beginTime = DateUtil.beginOfWeek(calendar).getTimeInMillis();
                endTime = DateUtil.endOfWeek(calendar).getTimeInMillis();
                break;
            case Month:
                beginTime = DateUtil.beginOfMonth(calendar).getTimeInMillis();
                endTime = DateUtil.endOfMonth(calendar).getTimeInMillis();
                break;
            default:
                break;
        }

        return activityLotteryHisDAO.selectRankItems(lotteryId, beginTime, endTime, num);
    }

    /**
     * 获取抽奖活动奖励排行
     *
     * @param lotteryId
     * @param lotteryRankType
     * @param num
     * @return
     */
    @Override
    public List<ActivityLotteryRankItem> getLotteryRewardRank(String lotteryId, ActivityLotteryRankTypeEnum lotteryRankType, int num) {
        Calendar calendar = Calendar.getInstance();

        long beginTime = 0L;
        long endTime = 0L;

        switch (lotteryRankType) {
            case Daily:
                beginTime = DateUtil.beginOfDay(calendar).getTimeInMillis();
                endTime = DateUtil.endOfDay(calendar).getTimeInMillis();
                break;
            case Week:
                beginTime = DateUtil.beginOfWeek(calendar).getTimeInMillis();
                endTime = DateUtil.endOfWeek(calendar).getTimeInMillis();
                break;
            case Month:
                beginTime = DateUtil.beginOfMonth(calendar).getTimeInMillis();
                endTime = DateUtil.endOfMonth(calendar).getTimeInMillis();
                break;
            default:
                break;
        }

        return activityLotteryHisDAO.selectRewardRankItems(lotteryId, beginTime, endTime, num);
    }

    /**
     * 通过活动ID获取抽奖活动列表
     *
     * @param activityId
     * @return
     */
    @Override
    public List<ActivityLotteryDO> getLotteryByActivityId(Long activityId) {
        return activityLotteryDAO.getLotteryByActivityId(activityId);
    }
}
