package com.starbuds.server.service.activity.lottery;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.activity.LotteryNoticeTypeEnum;
import com.starbuds.server.common.constant.activity.LotteryRewardTypeEnum;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.finance.WalletTypeEnum;
import com.starbuds.server.common.constant.message.im.ContentType;
import com.starbuds.server.common.constant.message.im.ImActionType;
import com.starbuds.server.common.constant.message.im.ImTargetType;
import com.starbuds.server.common.constant.message.push.ModuleTypeEnum;
import com.starbuds.server.common.pojo.activity.LotteryReward;
import com.starbuds.server.common.pojo.data.NoticeConfigVO;
import com.starbuds.server.common.pojo.data.PropsVO;
import com.starbuds.server.common.pojo.db.ActivityDO;
import com.starbuds.server.common.pojo.db.ActivityLotteryDO;
import com.starbuds.server.common.pojo.db.GiftDO;
import com.starbuds.server.common.pojo.db.UserDO;
import com.starbuds.server.common.pojo.media.GiftItemVO;
import com.starbuds.server.common.pojo.message.im.biz.SystemMsg;
import com.starbuds.server.common.pojo.message.im.live.LiveLotteryCommentMsg;
import com.starbuds.server.common.pojo.message.im.rtc.RtcGiftNoticeMsg;
import com.starbuds.server.common.pojo.user.UserCardVO;
import com.starbuds.server.common.service.data.PropsService;
import com.starbuds.server.common.service.media.GiftService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.rtc.RtcRoomService;
import com.starbuds.server.common.service.user.RtcUserService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.service.activity.dao.ActivityDAO;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 抽奖通知
 *
 * @author Lay
 * @date 2020/2/22
 */
@Slf4j
@Component
public class LotteryNotice {

    @Reference(version = "1.0.0")
    UserService userService;

    @Reference(version = "1.0.0")
    GiftService giftService;

    @Reference(version = "1.0.0")
    PropsService propsService;

    @Reference(version = "1.0.0")
    ImService imService;

    @Resource
    ActivityDAO activityDAO;

    @Reference(version = "1.0.0")
    RtcUserService rtcUserService;

    @Reference(version = "1.0.0")
    RtcRoomService rtcRoomService;

    /**
     * 异步通知喜讯
     *
     * @param userId
     * @param activityLottery
     * @param winRewards
     * @param receiveUserId
     */
    @Async
    public void noticeWinAsync(Long userId, ActivityLotteryDO activityLottery, List<LotteryReward> winRewards, Long receiveUserId) {
        if (CollectionUtil.isEmpty(winRewards)) {
            return;
        }

        Set<Long> giftIds = new HashSet<>();
        Set<Long> propsIds = new HashSet<>();
        List<LotteryReward> noticeRewards = new ArrayList<>();
        for (LotteryReward lotteryReward : winRewards) {
            if (LotteryNoticeTypeEnum.NONE.getValue() != Optional.ofNullable(lotteryReward.getIsNotice()).orElse(LotteryNoticeTypeEnum.NONE.getValue())) {
                noticeRewards.add(lotteryReward);
                LotteryRewardTypeEnum lotteryRewardType = LotteryRewardTypeEnum.toEnum(lotteryReward.getRewardType());
                if (LotteryRewardTypeEnum.GIFT == lotteryRewardType) {
                    giftIds.add(lotteryReward.getRewardId());
                } else if (LotteryRewardTypeEnum.PROPS == lotteryRewardType) {
                    propsIds.add(lotteryReward.getRewardId());
                }
            }
        }

        if (CollectionUtil.isEmpty(noticeRewards)) {
            return;
        }

        Map<Long, String> giftNameMap = new HashMap<>();
        Map<Long, Long> giftPriceMap = new HashMap<>();
        Map<Long, Integer> giftPriceTypeMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(giftIds)) {
            List<GiftItemVO> giftItems = giftService.getGiftItemsByIds(new ArrayList<>(giftIds));
            giftNameMap = giftItems.stream().collect(Collectors.toMap(GiftItemVO::getGiftId, GiftItemVO::getGiftName));
            giftPriceMap = giftItems.stream().collect(Collectors.toMap(GiftItemVO::getGiftId, GiftItemVO::getGiftPrice));
            giftPriceTypeMap = giftItems.stream().collect(Collectors.toMap(GiftItemVO::getGiftId, GiftItemVO::getGiftPriceType));
        }

        Map<Long, String> propsNameMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(propsIds)) {
            List<PropsVO> propsList = propsService.getPropsByIds(new ArrayList<>(propsIds));
            propsNameMap = propsList.stream().collect(Collectors.toMap(PropsVO::getPropsId, PropsVO::getPropsName));
        }

        String lotteryRewards = activityLottery.getLotteryRewards();
        List<LotteryReward> rewards = JSON.parseArray(lotteryRewards, LotteryReward.class);

        UserCardVO card = userService.getCard(userId);
        ActivityDO activityDO = activityDAO.selectById(activityLottery.getActivityId());

        for (LotteryReward reward : noticeRewards) {
            LotteryRewardTypeEnum lotteryRewardType = LotteryRewardTypeEnum.toEnum(reward.getRewardType());
            String rewardName;
            Integer quantity = Optional.ofNullable(reward.getQuantity()).orElse(1);
            Long rewardValue = null;
            Integer priceType = null;
            switch (lotteryRewardType) {
                case GIFT:
                    Long giftPrice = giftPriceMap.get(reward.getRewardId());
                    if (giftPrice == null) {
                        continue;
                    }
                    rewardName = Optional.ofNullable(giftNameMap.get(reward.getRewardId())).orElse("神秘礼物");
                    rewardValue = giftPrice * quantity;
                    priceType = giftPriceTypeMap.get(reward.getRewardId());
                    break;
                case PROPS:
                    rewardName = Optional.ofNullable(propsNameMap.get(reward.getRewardId())).orElse("神秘道具");
                    if (CollectionUtil.isNotEmpty(rewards)) {
                        for (LotteryReward item : rewards) {
                            if (reward.getRewardId().equals(item.getRewardId())) {
                                rewardValue = item.getRewardValue();
                                break;
                            }
                        }
                    }
                    priceType = WalletTypeEnum.STAR_DIAMOND.getValue();
                    break;
                case STAR_COIN:
                case STAR_YUAN:
                case STAR_DIAMOND:
                case CHARM:
                case WEALTH:
                    rewardName = String.format("%d%s", reward.getRewardId() * quantity, lotteryRewardType.getDesc());
                    break;
                default:
                    rewardName = "神秘大奖";
                    break;
            }

            if (card == null || StringUtils.isEmpty(rewardName) || priceType == null || rewardValue == null) {
                log.info("缺少信息，{} {} {}, 不通知", rewardName, priceType, rewardValue);
                continue;
            }

            sendChatRtcRoomMsg(activityDO, activityLottery, card, rewardName, quantity, rewardValue, priceType, reward, userId);
//            if (reward.getIsNotice() == LotteryNoticeTypeEnum.CHAT_NOTICE.getValue()) {
//
//            } else if (reward.getIsNotice() == LotteryNoticeTypeEnum.ALL_NOTICE.getValue()) {
//                sendChatRtcRoomMsg(activityDO, activityLottery, card, rewardName, quantity, rewardValue, priceType, reward, userId);
//                sendTopRtcRoomMsg(activityDO, activityLottery, card, rewardName, quantity, rewardValue, priceType, reward, receiveUserId);
//            } else {
//                sendTopRtcRoomMsg(activityDO, activityLottery, card, rewardName, quantity, rewardValue, priceType, reward, receiveUserId);
//            }

            if (rewardName.indexOf("碎片") > -1) {
                // 这里需要发送系统消息
                SystemMsg systemMsg = new SystemMsg();
                systemMsg.setContentType(ContentType.TEXT);
                systemMsg.setTitle("中奖通知");
                String body = String.format("恭喜获得 %s*%d ", rewardName, quantity);
                systemMsg.setBody(body);
                systemMsg.setActionType(ImActionType.NO_ACTION.getValue());
                imService.sendCustomImMsg(null, userId, systemMsg, null);
            }
        }
    }

    private void sendChatRtcRoomMsg(ActivityDO activityDO, ActivityLotteryDO activityLottery,
                                    UserCardVO card, String rewardName, Integer quantity,
                                    Long rewardValue, Integer priceType, LotteryReward reward,
                                    Long userId) {
        LiveLotteryCommentMsg msg = new LiveLotteryCommentMsg();
        msg.setActivityName(activityDO.getActivityName());
        msg.setActivityIcon(activityDO.getActivityNoticeIcon());
        msg.setActivityUrl(activityDO.getActivityUrl());
        msg.setLotteryId(activityLottery.getLotteryId());
        msg.setUserName(card.getUserName());
        msg.setUserAvatar(card.getUserAvatar());
        msg.setRewardName(rewardName);
        msg.setQuantity(quantity);
        msg.setRewardValue(rewardValue);
        msg.setPriceType(priceType);
        msg.setContent(String.format("恭喜 %s 获得 %s*%d 价值%d%s", card.getUserName(), rewardName, quantity, rewardValue, WalletTypeEnum.toEnum(priceType).getName()));
        msg.setNoticeType(reward.getIsNotice());
        imService.sendRtcRoomMsg(ImTargetType.ALL, userId, null, null, msg);
    }

    private void sendTopRtcRoomMsg(ActivityDO activityDO, ActivityLotteryDO activityLottery,
                                   UserCardVO card, String rewardName, Integer quantity,
                                   Long rewardValue, Integer priceType, LotteryReward reward,
                                   Long userId) {
        GiftDO giftDO = giftService.getGift(reward.getRewardId(), null);
        if (giftDO != null) {
            Long roomId = rtcUserService.getUserStayRoomId(userId);
            RtcGiftNoticeMsg rtcGiftNoticeMsg = new RtcGiftNoticeMsg();
            rtcGiftNoticeMsg.setGiftId(reward.getRewardId());
            rtcGiftNoticeMsg.setGiftName(rewardName);
            rtcGiftNoticeMsg.setRoomId(roomId);
            rtcGiftNoticeMsg.setRoomName("");
            rtcGiftNoticeMsg.setQuantity(quantity);
            rtcGiftNoticeMsg.setSenderUserName(card.getUserName());
            rtcGiftNoticeMsg.setSenderUserAvatar(card.getUserAvatar());
            rtcGiftNoticeMsg.setReceiveUserId(userId);
            UserDO userDO = userService.getUserById(userId);
            rtcGiftNoticeMsg.setReceiveUserName(userDO.getUserName());
            rtcGiftNoticeMsg.setNoticeStyle(0);
            // 首次显示连击
            rtcGiftNoticeMsg.setCombo(1L);
            NoticeConfigVO configVO = rtcRoomService.getNoticeConfig(giftDO.getNoticeId(), quantity, rewardValue);
            if (configVO != null) {
                if (!card.getUserId().equals(userId)) {
                    rtcGiftNoticeMsg.setValueAnimationId(configVO.getFloatScreenAnimationId());
                }
                rtcGiftNoticeMsg.setReceiverAvatar(userDO.getUserAvatar());
                rtcGiftNoticeMsg.setContent(String.format("恭喜 %s 获得 %s*%d 价值%d%s", card.getUserName(), rewardName, quantity, rewardValue, WalletTypeEnum.toEnum(priceType).getName()));
                imService.sendRtcRoomMsg(ImTargetType.ALL, userId, null, roomId, rtcGiftNoticeMsg);
            }
        }
    }
}
