package com.starbuds.server.service.activity.lottery;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.crypto.SecureUtil;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.activity.LotteryRewardTypeEnum;
import com.starbuds.server.common.constant.cache.RedisActivityKeys;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.finance.BagItemTypeEnum;
import com.starbuds.server.common.constant.finance.LotteryPayTypeEnum;
import com.starbuds.server.common.constant.finance.ScoreTypeEnum;
import com.starbuds.server.common.constant.finance.WalletTypeEnum;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rocketmq.RocketTopic;
import com.starbuds.server.common.constant.statistics.StatPlatformTypeEnum;
import com.starbuds.server.common.pojo.activity.LotteryHisRewardVO;
import com.starbuds.server.common.pojo.activity.LotteryReward;
import com.starbuds.server.common.pojo.activity.LotteryUserRewards;
import com.starbuds.server.common.pojo.db.ActivityLotteryDO;
import com.starbuds.server.common.pojo.db.ActivityLotteryHisDO;
import com.starbuds.server.common.pojo.db.ActivityLotteryHisRewardsDO;
import com.starbuds.server.common.pojo.finance.BagItemAddDTO;
import com.starbuds.server.common.pojo.mq.*;
import com.starbuds.server.common.service.core.CacheService;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.finance.BagService;
import com.starbuds.server.common.service.finance.WalletService;
import com.starbuds.server.common.service.media.GiftService;
import com.starbuds.server.common.service.statistics.StatPlatformService;
import com.starbuds.server.common.service.user.ScoreService;
import com.starbuds.server.common.utils.LogUtil;
import com.starbuds.server.service.activity.dao.ActivityLotteryDAO;
import com.starbuds.server.service.activity.dao.ActivityLotteryHisDAO;
import com.starbuds.server.service.activity.dao.ActivityLotteryHisRewardsDAO;
import com.starbuds.server.service.activity.rocket.RocketProducer;
import com.starbuds.server.service.activity.util.ProbabilityUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.beans.BeanUtils;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.concurrent.TimeUnit;

import static com.starbuds.server.service.activity.util.SpringContextUtil.isDev;

/**
 * 抽奖服务
 *
 * @author Lay
 * @date 2020/2/18
 */
@Component
@Slf4j
public class LotteryProvider {

    @Reference(version = "1.0.0")
    IdentityService identityService;

    @Reference(version = "1.0.0")
    WalletService walletService;

    @Reference(version = "1.0.0")
    CacheService cacheService;

    @Reference(version = "1.0.0")
    BagService bagService;

    @Reference(version = "1.0.0")
    ScoreService scoreService;

    @Reference(version = "1.0.0")
    StatPlatformService statPlatformService;

    @Reference(version = "1.0.0")
    GiftService giftService;

    @Resource
    ActivityLotteryDAO activityLotteryDAO;

    @Resource
    ActivityLotteryHisDAO activityLotteryHisDAO;

    @Resource
    ActivityLotteryHisRewardsDAO activityLotteryHisRewardsDAO;

    @Resource
    LotteryNotice lotteryNotice;

    @Resource
    RocketProducer rocketProducer;

    /**
     * 抽奖
     *
     * @param lotteryId
     * @param userId
     * @param times         礼物数量
     * @param channelId
     * @param isFree
     * @param receiveUserId
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public LotteryResult lottery(String lotteryId, Long userId, Integer times, Long channelId, boolean isFree, Long receiveUserId) {
        // 读取抽奖配置
        ActivityLotteryDO activityLotteryDO = activityLotteryDAO.selectById(lotteryId);
        if (activityLotteryDO != null && times > 0) {
            Long hisId = identityService.getId();

            // 奖项数据处理
            List<LotteryReward> lotteryRewards = JSON.parseArray(activityLotteryDO.getLotteryRewards(), LotteryReward.class);
            List<LotteryReward> poolRewards = JSON.parseArray(activityLotteryDO.getLotteryPoolRewards(), LotteryReward.class);

            Long amount = activityLotteryDO.getLotteryPrice() * times;
            Long balance = 0L;
            if (!isFree) {
                if (activityLotteryDO.getLotteryPayType() != null && activityLotteryDO.getLotteryPayType() == LotteryPayTypeEnum.PROP.getValue()) {
                    // 这里进行扣除碎片
                    Integer quantity = bagService.takeItem(userId, activityLotteryDO.getLotteryPropId(), BagItemTypeEnum.PROPS.getValue(), activityLotteryDO.getLotteryPropNum(), "碎片抽奖", hisId);
                    if (quantity == null || quantity < 0) {
                        LotteryResult lotteryResult = new LotteryResult();
                        lotteryResult.setBalance(0L);
                        return lotteryResult;
                    }
                    balance = Long.valueOf(bagService.getItemQuantity(userId, BagItemTypeEnum.PROPS, activityLotteryDO.getLotteryPropId()));
                } else {
                    if (amount > 0) {
                        // 扣钱
                        balance = walletService.reduceCoin(userId, activityLotteryDO.getLotteryPriceType(), amount, String.format("%s%d次", activityLotteryDO.getLotteryName(), times), hisId);
                        if (balance < 0) {
                            LotteryResult lotteryResult = new LotteryResult();
                            lotteryResult.setBalance(balance);
                            return lotteryResult;
                        }
                    }
                }
            }

            // 奖池
            long poolAddAmount = 0L;
            if (activityLotteryDO.getLotteryPoolRatio() != null) {
                poolAddAmount = BigDecimal.valueOf(amount).multiply(new BigDecimal(String.valueOf(activityLotteryDO.getLotteryPoolRatio()))).setScale(0, RoundingMode.DOWN).longValue();
            }
            Long poolAmount = cacheService.hashIncreaseLong(RedisActivityKeys.ACTIVITY_LOTTERY_POOL, lotteryId, poolAddAmount);

            // 奖池处理
            LotteryReward poolReward = null;
            if (CollectionUtil.isNotEmpty(poolRewards)) {
                // 资金池奖项选举
                poolReward = poolRewardElection(lotteryId, poolRewards);
            }

            // 中奖奖项
            Map<String, LotteryReward> winRewardMap = new HashMap<>();

            // 资金池判断
            if (poolReward != null && poolAmount > poolReward.getRewardValue()) {
                // 抢锁中大奖
                String lockKey = RedisActivityKeys.getActivityLotteryPoolLock(lotteryId);
                if (cacheService.setIfAbsent(lockKey, String.valueOf(userId), 10, TimeUnit.SECONDS)) {
                    // 中奖
                    addWinReward(winRewardMap, poolReward);
                    // 减次
                    times--;
                    // 清奖池
                    cacheService.hashIncreaseLong(RedisActivityKeys.ACTIVITY_LOTTERY_POOL, lotteryId, -poolReward.getRewardValue());
                    // 解锁
                    cacheService.delKey(lockKey);
                }
            }

            // 奖项抽奖
            if (CollectionUtil.isNotEmpty(lotteryRewards)) {
                LotteryReward[] rewards = lotteryRewards.toArray(new LotteryReward[0]);
                if (isDev()) {
                    log.info(LogUtil.devLog("奖项抽奖rewards={}"), JSON.toJSONString(rewards, true));
                }
                // 概率处理
                double[] probArr = new double[rewards.length];
                double[] guaranteedProbArr = new double[rewards.length];
                for (int i = 0; i < rewards.length; i++) {
                    probArr[i] = Optional.ofNullable(rewards[i].getRatio()).orElse(0d);
                    if (BoolIntVal.True == Optional.ofNullable(rewards[i].getIsGuaranteed()).orElse(BoolIntVal.False)) {
                        guaranteedProbArr[i] = Optional.ofNullable(rewards[i].getGuaranteedRatio()).orElse(0d);
                    }
                }

                // 连抽判断
                int roundNum = 0;
                Integer guaranteedRound = activityLotteryDO.getLotteryGuaranteedRound();
                if (guaranteedRound != null && times > guaranteedRound) {
                    // 多少个连抽
                    roundNum = times / guaranteedRound;
                    for (int i = 0; i < roundNum; i++) {
                        int index = ProbabilityUtil.getRandomColumnIndex(guaranteedProbArr);
                        if (index > -1) {
                            addWinReward(winRewardMap, rewards[index]);
                        }
                        if (isDev() && winRewardMap.size() == 2) {
                            log.info(LogUtil.devLog("多少个连抽 index={} lotteryId={} rewards={}"), index, lotteryId, JSON.toJSONString(rewards, true));
                        }
                    }
                }

                // 循环抽奖
                final int lotteryTimes = times - roundNum;
                for (int i = 0; i < lotteryTimes; i++) {
                    int index = ProbabilityUtil.getRandomColumnIndex(probArr);
                    if (index > -1) {
                        addWinReward(winRewardMap, rewards[index]);
                    }
                }
            }

            // 合并同类项
            Long winRewardsValue = 0L;
            List<LotteryReward> winRewards = new ArrayList<>(winRewardMap.values());

            Long curTime = System.currentTimeMillis();
            List<ActivityLotteryHisRewardsDO> hisRewardsDOList = new ArrayList<>();
            if (isDev()) {
                log.info(LogUtil.devLog("winRewards 1 {}"), JSON.toJSONString(winRewards, true));
            }
            for (LotteryReward lotteryReward : winRewards) {
                ActivityLotteryHisRewardsDO hisRewardsDO = new ActivityLotteryHisRewardsDO();
                hisRewardsDO.setRecordId(getRewardRecordId(hisId, lotteryReward.getRewardId()));
                hisRewardsDO.setHisId(hisId);
                hisRewardsDO.setActivityId(activityLotteryDO.getActivityId());
                hisRewardsDO.setLotteryId(lotteryId);
                hisRewardsDO.setUserId(userId);
                hisRewardsDO.setRewardId(lotteryReward.getRewardId());
                hisRewardsDO.setRewardValue(lotteryReward.getRewardValue());
                hisRewardsDO.setQuantity(lotteryReward.getQuantity());
                hisRewardsDO.setRewardType(lotteryReward.getRewardType());
                hisRewardsDO.setLotteryTime(curTime);
                hisRewardsDOList.add(hisRewardsDO);
                winRewardsValue += lotteryReward.getRewardValue();
            }

            // 写入抽奖记录和中奖记录
            ActivityLotteryHisDO activityLotteryHisDO = new ActivityLotteryHisDO();
            activityLotteryHisDO.setHisId(hisId);
            activityLotteryHisDO.setActivityId(activityLotteryDO.getActivityId());
            activityLotteryHisDO.setLotteryId(lotteryId);
            activityLotteryHisDO.setUserId(userId);
            activityLotteryHisDO.setHisCreateTime(curTime);
            activityLotteryHisDO.setHisCostAmount(amount);
            activityLotteryHisDO.setHisTimes(times);
            activityLotteryHisDO.setHisPriceType(activityLotteryDO.getLotteryPriceType());
            activityLotteryHisDO.setHisRewardValue(winRewardsValue);
            activityLotteryHisDO.setChannelId(channelId);
            activityLotteryHisDAO.insert(activityLotteryHisDO);
            if (CollectionUtil.isNotEmpty(hisRewardsDOList)) {
                activityLotteryHisRewardsDAO.batchInsert(hisRewardsDOList);
            }

            // 异步通知中奖喜讯
            lotteryNotice.noticeWinAsync(userId, activityLotteryDO, winRewards, receiveUserId);

            // 异步统计
            if (WalletTypeEnum.STAR_DIAMOND.getValue() == activityLotteryDO.getLotteryPriceType()) {
                statPlatformService.addPlatformStat(channelId, StatPlatformTypeEnum.LUCKY_ACTIVITY_PAY_AMOUNT, BigDecimal.valueOf(amount));
                statPlatformService.addPlatformStat(channelId, StatPlatformTypeEnum.LUCKY_ACTIVITY_WIN_AMOUNT, BigDecimal.valueOf(winRewardsValue));
            }

            LotteryResult lotteryResult = new LotteryResult();
            lotteryResult.setHisId(hisId);
            lotteryResult.setWinRewards(winRewards);
            lotteryResult.setBalance(balance);
            lotteryResult.setLotteryName(activityLotteryDO.getLotteryName());

            if (isDev()) {
                log.info(LogUtil.devLog("中奖信息 lotteryId={} {}"), lotteryId, JSON.toJSONString(lotteryResult, true));
            }
            return lotteryResult;
        }
        return null;
    }

    @Async
    @Transactional(rollbackFor = Exception.class)
    public void settleAsync(String lotteryName, Long hisId, Long userId, List<LotteryHisRewardVO> rewardList, Long channelId, int giftSceneType, String channelName, Long targetUserId) {
        log.info("抽奖结算");
        // 中奖备注
        String winRemarks = String.format("%s中奖", lotteryName);

        // 分发
        List<BagItemAddDTO> bagItemAddParams = new ArrayList<>();
        Map<Long, Integer> targetGiftsMap = new HashMap<>();
        Long starDiamond = 0L;
        Long starYuan = 0L;
        Long starCoin = 0L;
        Long wealthValue = 0L;
        Long charmValue = 0L;
        for (LotteryHisRewardVO lotteryRewardVO : rewardList) {
            LotteryRewardTypeEnum lotteryRewardType = LotteryRewardTypeEnum.toEnum(lotteryRewardVO.getRewardType());
            if (lotteryRewardType == null) {
                continue;
            }

            switch (lotteryRewardType) {
                case GIFT:
                    if (targetUserId == null) {
                        // 发放礼物
                        bagItemAddParams.add(BagItemAddDTO.builder()
                                .userId(userId)
                                .itemId(lotteryRewardVO.getRewardId())
                                .itemType(BagItemTypeEnum.GIFT.getValue())
                                .quantity(lotteryRewardVO.getQuantity())
                                .remarks(winRemarks)
                                .orderId(hisId)
                                .build());
                    } else {
                        // 赠送礼物 并合并相同礼物项
                        log.info("加入待赠送礼物 并合并相同礼物项");
                        Integer quantity = targetGiftsMap.get(lotteryRewardVO.getRewardId());
                        if (quantity == null) {
                            quantity = lotteryRewardVO.getQuantity();
                        } else {
                            quantity += lotteryRewardVO.getQuantity();
                        }
                        targetGiftsMap.put(lotteryRewardVO.getRewardId(), quantity);
                    }
                    break;
                case PROPS:
                    // 发放道具
                    bagItemAddParams.add(BagItemAddDTO.builder()
                            .userId(userId)
                            .itemId(lotteryRewardVO.getRewardId())
                            .itemType(BagItemTypeEnum.PROPS.getValue())
                            .quantity(lotteryRewardVO.getQuantity())
                            .remarks(winRemarks)
                            .orderId(hisId)
                            .build());
                    break;
                case STAR_DIAMOND:
                    // 发放星钻
                    starDiamond += lotteryRewardVO.getRewardId() * lotteryRewardVO.getQuantity();
                    break;
                case STAR_YUAN:
                    // 发放星元
                    starYuan += lotteryRewardVO.getRewardId() * lotteryRewardVO.getQuantity();
                    break;
                case STAR_COIN:
                    // 发放星币
                    starCoin += lotteryRewardVO.getRewardId() * lotteryRewardVO.getQuantity();
                    break;
                case WEALTH:
                    // 发放财富值
                    wealthValue += lotteryRewardVO.getRewardId() * lotteryRewardVO.getQuantity();
                    break;
                case CHARM:
                    // 发放魅力值
                    charmValue += lotteryRewardVO.getRewardId() * lotteryRewardVO.getQuantity();
                    break;
                default:
                    break;
            }
        }

        // 执行发放
        if (CollectionUtil.isNotEmpty(bagItemAddParams)) {
            bagService.batchAddItem(bagItemAddParams);
        }
        if (starDiamond > 0) {
            walletService.addCoin(userId, WalletTypeEnum.STAR_DIAMOND.getValue(), starDiamond, winRemarks, hisId);
            // 平台游戏消耗星钻
            QueueLotteryMsg lotteryMsg = new QueueLotteryMsg();
            lotteryMsg.setChannelId(channelId);
            lotteryMsg.setStarDiamond(starDiamond);
            rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.LOTTERY_WIN, lotteryMsg.serialize());
        }
        if (starYuan > 0) {
            walletService.addCoin(userId, WalletTypeEnum.STAR_YUAN.getValue(), starYuan, winRemarks, hisId);
        }
        if (starCoin > 0) {
            walletService.addCoin(userId, WalletTypeEnum.STAR_COIN.getValue(), starCoin, winRemarks, hisId);
        }
        if (wealthValue > 0) {
            scoreService.addScore(userId, ScoreTypeEnum.WEALTH.getValue(), wealthValue, winRemarks, hisId, 0L);
        }
        if (charmValue > 0) {
            scoreService.addScore(userId, ScoreTypeEnum.CHARM.getValue(), charmValue, winRemarks, hisId, 0L);
        }

        // 直接赠送
        if (CollectionUtil.isNotEmpty(targetGiftsMap)) {
            log.info("赠送礼物");
            QueueLotteryGiftRewardsMsg lotteryGiftRewardsMsg = new QueueLotteryGiftRewardsMsg();
            lotteryGiftRewardsMsg.setUserId(userId);
            lotteryGiftRewardsMsg.setTargetUserId(targetUserId);
            lotteryGiftRewardsMsg.setGiftRewards(targetGiftsMap);
            lotteryGiftRewardsMsg.setChannelId(channelId);
            lotteryGiftRewardsMsg.setGiftSceneType(giftSceneType);
            lotteryGiftRewardsMsg.setChannelName(channelName);
            rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.LOTTERY_GIFT_REWARDS, lotteryGiftRewardsMsg.serialize());
        }
    }

    @Async
    @Transactional(rollbackFor = Exception.class)
    public void settleAsync(Long userId, List<LotteryUserRewards> userRewardsList, Long channelId, Long roomId, Map<Long, Integer> targetUserSeats) {

        Map<Long, List<LotteryHisRewardVO>> targetUserRewardMap = new HashMap<>();

        for (LotteryUserRewards userRewards : userRewardsList) {
            log.info("抽奖结算: {}", JSON.toJSONString(userRewards));

            if (CollectionUtils.isEmpty(userRewards.getRewardList())) {
                continue;
            }

            // 中奖备注
            String winRemarks = String.format("%s中奖", userRewards.getLotteryName());

            // 分发
            List<BagItemAddDTO> bagItemAddParams = new ArrayList<>();
            Long starDiamond = 0L;
            Long starYuan = 0L;
            Long starCoin = 0L;
            Long wealthValue = 0L;
            Long charmValue = 0L;
            Long targetUserId = userRewards.getTargetUserId();
            Long hisId = userRewards.getHisId();

            for (LotteryHisRewardVO lotteryRewardVO : userRewards.getRewardList()) {
                LotteryRewardTypeEnum lotteryRewardType = LotteryRewardTypeEnum.toEnum(lotteryRewardVO.getRewardType());
                if (lotteryRewardType == null) {
                    continue;
                }

                switch (lotteryRewardType) {
                    case GIFT:
                        if (targetUserId == null) {
                            // 发放礼物
                            bagItemAddParams.add(BagItemAddDTO.builder()
                                    .userId(userId)
                                    .itemId(lotteryRewardVO.getRewardId())
                                    .itemType(BagItemTypeEnum.GIFT.getValue())
                                    .quantity(lotteryRewardVO.getQuantity())
                                    .remarks(winRemarks)
                                    .orderId(hisId)
                                    .build());
                        } else {
                            // 派发礼物
                            log.info("加入待赠送礼物 并合并相同礼物项");
                            if (!targetUserRewardMap.containsKey(targetUserId)) {
                                targetUserRewardMap.put(targetUserId, new ArrayList<>());
                            }
                            targetUserRewardMap.get(targetUserId).add(lotteryRewardVO);
                        }
                        break;
                    case PROPS:
                        // 发放道具
                        bagItemAddParams.add(BagItemAddDTO.builder()
                                .userId(userId)
                                .itemId(lotteryRewardVO.getRewardId())
                                .itemType(BagItemTypeEnum.PROPS.getValue())
                                .quantity(lotteryRewardVO.getQuantity())
                                .remarks(winRemarks)
                                .orderId(hisId)
                                .build());
                        break;
                    case STAR_DIAMOND:
                        // 发放星钻
                        starDiamond += lotteryRewardVO.getRewardId() * lotteryRewardVO.getQuantity();
                        break;
                    case STAR_YUAN:
                        // 发放星元
                        starYuan += lotteryRewardVO.getRewardId() * lotteryRewardVO.getQuantity();
                        break;
                    case STAR_COIN:
                        // 发放星币
                        starCoin += lotteryRewardVO.getRewardId() * lotteryRewardVO.getQuantity();
                        break;
                    case WEALTH:
                        // 发放财富值
                        wealthValue += lotteryRewardVO.getRewardId() * lotteryRewardVO.getQuantity();
                        break;
                    case CHARM:
                        // 发放魅力值
                        charmValue += lotteryRewardVO.getRewardId() * lotteryRewardVO.getQuantity();
                        break;
                    default:
                        break;
                }
            }

            // 执行发放
            if (CollectionUtil.isNotEmpty(bagItemAddParams)) {
                bagService.batchAddItem(bagItemAddParams);
            }
            if (starDiamond > 0) {
                walletService.addCoin(userId, WalletTypeEnum.STAR_DIAMOND.getValue(), starDiamond, winRemarks, hisId);
                // 平台游戏消耗星钻
                QueueLotteryMsg lotteryMsg = new QueueLotteryMsg();
                lotteryMsg.setChannelId(channelId);
                lotteryMsg.setStarDiamond(starDiamond);
                rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.LOTTERY_WIN, lotteryMsg.serialize());
            }
            if (starYuan > 0) {
                walletService.addCoin(userId, WalletTypeEnum.STAR_YUAN.getValue(), starYuan, winRemarks, hisId);
            }
            if (starCoin > 0) {
                walletService.addCoin(userId, WalletTypeEnum.STAR_COIN.getValue(), starCoin, winRemarks, hisId);
            }
            if (wealthValue > 0) {
                scoreService.addScore(userId, ScoreTypeEnum.WEALTH.getValue(), wealthValue, winRemarks, hisId, 0L);
            }
            if (charmValue > 0) {
                scoreService.addScore(userId, ScoreTypeEnum.CHARM.getValue(), charmValue, winRemarks, hisId, 0L);
            }
        }

        // 直接赠送
        if (CollectionUtil.isNotEmpty(targetUserRewardMap)) {
            log.info("赠送礼物");
            QueueLotteryGiftRewardsRtcMsg rewardsMsg = new QueueLotteryGiftRewardsRtcMsg();
            rewardsMsg.setUserId(userId);
            rewardsMsg.setRoomId(roomId);
            rewardsMsg.setTargetUserRewardMap(targetUserRewardMap);
            rewardsMsg.setTargetUserSeats(targetUserSeats);
            rewardsMsg.setChannelId(channelId);
            rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.LOTTERY_GIFT_REWARDS_RTC, rewardsMsg.serialize());
        }
    }

    /**
     * 发送
     *
     * @param userId
     * @param userRewards
     * @param channelId
     * @param hisId
     * @param receiveUserId
     */
    @Async
    @Transactional(rollbackFor = Exception.class)
    public void settleAsyncForVoiceChat(Long userId, LotteryUserRewards userRewards, Long channelId, Long hisId, Long receiveUserId) {

        log.info("抽奖结算");
        // 中奖备注
        String winRemarks = String.format("%s中奖", userRewards.getLotteryName());

        // 分发
        List<BagItemAddDTO> bagItemAddParams = new ArrayList<>();
        Long starDiamond = 0L;
        Long starYuan = 0L;
        Long starCoin = 0L;
        Long wealthValue = 0L;
        Long charmValue = 0L;

        for (LotteryHisRewardVO lotteryRewardVO : userRewards.getRewardList()) {
            LotteryRewardTypeEnum lotteryRewardType = LotteryRewardTypeEnum.toEnum(lotteryRewardVO.getRewardType());
            if (lotteryRewardType == null) {
                continue;
            }

            switch (lotteryRewardType) {
                case GIFT:
                    // 发放礼物
                    bagItemAddParams.add(BagItemAddDTO.builder()
                            .userId(userId)
                            .itemId(lotteryRewardVO.getRewardId())
                            .itemType(BagItemTypeEnum.GIFT.getValue())
                            .quantity(lotteryRewardVO.getQuantity())
                            .remarks(winRemarks)
                            .orderId(hisId)
                            .build());
                    break;
                case PROPS:
                    // 发放道具
                    bagItemAddParams.add(BagItemAddDTO.builder()
                            .userId(userId)
                            .itemId(lotteryRewardVO.getRewardId())
                            .itemType(BagItemTypeEnum.PROPS.getValue())
                            .quantity(lotteryRewardVO.getQuantity())
                            .remarks(winRemarks)
                            .orderId(hisId)
                            .build());
                    break;
                case STAR_DIAMOND:
                    // 发放星钻
                    starDiamond += lotteryRewardVO.getRewardId() * lotteryRewardVO.getQuantity();
                    break;
                case STAR_YUAN:
                    // 发放星元
                    starYuan += lotteryRewardVO.getRewardId() * lotteryRewardVO.getQuantity();
                    break;
                case STAR_COIN:
                    // 发放星币
                    starCoin += lotteryRewardVO.getRewardId() * lotteryRewardVO.getQuantity();
                    break;
                case WEALTH:
                    // 发放财富值
                    wealthValue += lotteryRewardVO.getRewardId() * lotteryRewardVO.getQuantity();
                    break;
                case CHARM:
                    // 发放魅力值
                    charmValue += lotteryRewardVO.getRewardId() * lotteryRewardVO.getQuantity();
                    break;
                default:
                    break;
            }
        }

        // 执行发放 走mq处理，对方收到粉钻，所以此处无需发放
//        if (CollectionUtil.isNotEmpty(bagItemAddParams)) {
//            bagService.batchAddItem(bagItemAddParams);
//        }
        if (starDiamond > 0) {
            walletService.addCoin(userId, WalletTypeEnum.STAR_DIAMOND.getValue(), starDiamond, winRemarks, hisId);
            // 平台游戏消耗星钻
            QueueLotteryMsg lotteryMsg = new QueueLotteryMsg();
            lotteryMsg.setChannelId(channelId);
            lotteryMsg.setStarDiamond(starDiamond);
            rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.LOTTERY_WIN, lotteryMsg.serialize());
        }
        if (starYuan > 0) {
            walletService.addCoin(userId, WalletTypeEnum.STAR_YUAN.getValue(), starYuan, winRemarks, hisId);
        }
        if (starCoin > 0) {
            walletService.addCoin(userId, WalletTypeEnum.STAR_COIN.getValue(), starCoin, winRemarks, hisId);
        }
        if (wealthValue > 0) {
            scoreService.addScore(userId, ScoreTypeEnum.WEALTH.getValue(), wealthValue, winRemarks, hisId, 0L);
        }
        if (charmValue > 0) {
            scoreService.addScore(userId, ScoreTypeEnum.CHARM.getValue(), charmValue, winRemarks, hisId, 0L);
        }

        // 直接赠送
        log.info("赠送礼物");
        QueueLotteryGiftRewardsVoiceChatMsg rewardsMsg = new QueueLotteryGiftRewardsVoiceChatMsg();
        rewardsMsg.setUserId(userId);
        rewardsMsg.setHisId(hisId);
        rewardsMsg.setOrderId(hisId);
        rewardsMsg.setUserRewards(userRewards.getRewardList());
        rewardsMsg.setChannelId(channelId);
        rewardsMsg.setReceiveUserId(receiveUserId);
        rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.LOTTERY_GIFT_REWARDS_VOICE_CHAT, rewardsMsg.serialize());
    }

    /**
     * 根据 settleAsyncForVoiceChat 改写
     *
     * @param userId
     * @param userRewards
     * @param channelId
     * @param orderId
     * @param receiveUserId
     */
    @Async
    @Transactional(rollbackFor = Exception.class)
    public void settleAsyncForPrivateChat(Long userId, LotteryUserRewards userRewards, Long channelId, Long orderId, Long receiveUserId) {

        log.info("私聊抽奖结算");
        // 中奖备注
        String winRemarks = String.format("%s中奖", userRewards.getLotteryName());

        // 分发
        List<BagItemAddDTO> bagItemAddParams = new ArrayList<>();
        Long starDiamond = 0L;
        Long starYuan = 0L;
        Long starCoin = 0L;
        Long wealthValue = 0L;
        Long charmValue = 0L;

        for (LotteryHisRewardVO lotteryRewardVO : userRewards.getRewardList()) {
            LotteryRewardTypeEnum lotteryRewardType = LotteryRewardTypeEnum.toEnum(lotteryRewardVO.getRewardType());
            if (lotteryRewardType == null) {
                continue;
            }

            switch (lotteryRewardType) {
                case GIFT:
                    // 发放礼物
                    bagItemAddParams.add(BagItemAddDTO.builder()
                            .userId(userId)
                            .itemId(lotteryRewardVO.getRewardId())
                            .itemType(BagItemTypeEnum.GIFT.getValue())
                            .quantity(lotteryRewardVO.getQuantity())
                            .remarks(winRemarks)
                            .orderId(orderId)
                            .build());
                    break;
                case PROPS:
                    // 发放道具
                    bagItemAddParams.add(BagItemAddDTO.builder()
                            .userId(userId)
                            .itemId(lotteryRewardVO.getRewardId())
                            .itemType(BagItemTypeEnum.PROPS.getValue())
                            .quantity(lotteryRewardVO.getQuantity())
                            .remarks(winRemarks)
                            .orderId(orderId)
                            .build());
                    break;
                case STAR_DIAMOND:
                    // 发放星钻
                    starDiamond += lotteryRewardVO.getRewardId() * lotteryRewardVO.getQuantity();
                    break;
                case STAR_YUAN:
                    // 发放星元
                    starYuan += lotteryRewardVO.getRewardId() * lotteryRewardVO.getQuantity();
                    break;
                case STAR_COIN:
                    // 发放星币
                    starCoin += lotteryRewardVO.getRewardId() * lotteryRewardVO.getQuantity();
                    break;
                case WEALTH:
                    // 发放财富值
                    wealthValue += lotteryRewardVO.getRewardId() * lotteryRewardVO.getQuantity();
                    break;
                case CHARM:
                    // 发放魅力值
                    charmValue += lotteryRewardVO.getRewardId() * lotteryRewardVO.getQuantity();
                    break;
                default:
                    break;
            }
        }

        // 执行发放 走mq处理，对方收到粉钻，所以此处无需发放
//        if (CollectionUtil.isNotEmpty(bagItemAddParams)) {
//            bagService.batchAddItem(bagItemAddParams);
//        }
        if (starDiamond > 0) {
            walletService.addCoin(userId, WalletTypeEnum.STAR_DIAMOND.getValue(), starDiamond, winRemarks, orderId);
            // 平台游戏消耗星钻
            QueueLotteryMsg lotteryMsg = new QueueLotteryMsg();
            lotteryMsg.setChannelId(channelId);
            lotteryMsg.setStarDiamond(starDiamond);
            rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.LOTTERY_WIN, lotteryMsg.serialize());
        }
        if (starYuan > 0) {
            walletService.addCoin(userId, WalletTypeEnum.STAR_YUAN.getValue(), starYuan, winRemarks, orderId);
        }
        if (starCoin > 0) {
            walletService.addCoin(userId, WalletTypeEnum.STAR_COIN.getValue(), starCoin, winRemarks, orderId);
        }
        if (wealthValue > 0) {
            scoreService.addScore(userId, ScoreTypeEnum.WEALTH.getValue(), wealthValue, winRemarks, orderId, 0L);
        }
        if (charmValue > 0) {
            scoreService.addScore(userId, ScoreTypeEnum.CHARM.getValue(), charmValue, winRemarks, orderId, 0L);
        }

        // 直接赠送
        log.info("私聊赠送礼物");
        QueueLotteryGiftRewardsPrivateChatMsg rewardsMsg = new QueueLotteryGiftRewardsPrivateChatMsg();
        rewardsMsg.setUserId(userId);
        rewardsMsg.setOrderId(orderId);
        rewardsMsg.setUserRewards(userRewards.getRewardList());
        rewardsMsg.setChannelId(channelId);
        rewardsMsg.setReceiveUserId(receiveUserId);
        rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.LOTTERY_GIFT_REWARDS_PRIVATE_CHAT, rewardsMsg.serialize());
    }

    /**
     * 获得用户中奖信息
     *
     * @param receiveUserId 中奖用户id
     * @param lotteryResult
     * @return
     */
    public LotteryUserRewards getLotteryUserRewards(Long receiveUserId, LotteryResult lotteryResult) {
        LotteryUserRewards userRewards = new LotteryUserRewards();
        userRewards.setTargetUserId(receiveUserId);
        userRewards.setLotteryName(lotteryResult.getLotteryName());

        List<LotteryReward> winRewards = lotteryResult.getWinRewards();
        if (CollectionUtil.isNotEmpty(winRewards)) {
            List<LotteryHisRewardVO> list = new ArrayList<>();
            for (LotteryReward lotteryReward : winRewards) {
                LotteryHisRewardVO rewardVO = new LotteryHisRewardVO();
                rewardVO.setHisId(lotteryResult.getHisId());
                rewardVO.setRewardId(lotteryReward.getRewardId());
                rewardVO.setRewardType(lotteryReward.getRewardType());
                rewardVO.setQuantity(lotteryReward.getQuantity());
                list.add(rewardVO);
            }
            userRewards.setRewardList(list);
        }
        return userRewards;
    }

    private static String getRewardRecordId(Long hisId, Long rewardId) {
        return SecureUtil.md5(String.format("%d:%d", hisId, rewardId));
    }

    /**
     * 资金池奖项选举
     *
     * @param lotteryId
     * @param lotteryRewards
     * @return
     */
    private LotteryReward poolRewardElection(String lotteryId, List<LotteryReward> lotteryRewards) {
        LotteryReward[] rewards = lotteryRewards.toArray(new LotteryReward[0]);
        double[] probArr = new double[rewards.length];
        for (int i = 0; i < rewards.length; i++) {
            probArr[i] = rewards[i].getRatio();
        }
        int index = ProbabilityUtil.getRandomColumnIndex(probArr);

        return rewards[index];
    }

    private static String getRewardKey(Integer rewardType, Long rewardId) {
        return String.format("%d:%d", rewardType, rewardId);
    }

    private static void addWinReward(Map<String, LotteryReward> map, LotteryReward reward) {
        if (reward == null) {
            return;
        }
        String rewardKey = getRewardKey(reward.getRewardType(), reward.getRewardId());
        if (map.containsKey(rewardKey)) {
            LotteryReward oldReward = map.get(rewardKey);
            oldReward.setQuantity(oldReward.getQuantity() + reward.getQuantity());
            oldReward.setRewardValue(oldReward.getRewardValue() + reward.getRewardValue());
        } else {
            LotteryReward newReward = new LotteryReward();
            BeanUtils.copyProperties(reward, newReward);
            map.put(rewardKey, newReward);
        }
    }
}
