package com.starbuds.server.service.activity.lotterygame;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.starbuds.server.common.constant.activity.LotteryGamePlayTypeEnum;
import com.starbuds.server.common.constant.activity.LotteryGameRewardTypeEnum;
import com.starbuds.server.common.constant.activity.LotteryGameWinNoticeTypeEnum;
import com.starbuds.server.common.constant.cache.RedisActivityKeys;
import com.starbuds.server.common.constant.finance.AddItemToBagSourceEnum;
import com.starbuds.server.common.constant.finance.BagItemTypeEnum;
import com.starbuds.server.common.constant.message.im.ContentType;
import com.starbuds.server.common.constant.message.im.ImActionType;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rocketmq.RocketTopic;
import com.starbuds.server.common.pojo.activity.bo.LotteryGameRewardBO;
import com.starbuds.server.common.pojo.activity.bo.LotteryGameRewardSummary;
import com.starbuds.server.common.pojo.activity.bo.LotteryGameWinItemBO;
import com.starbuds.server.common.pojo.db.UserDO;
import com.starbuds.server.common.pojo.media.bo.PriceInfoBO;
import com.starbuds.server.common.pojo.message.im.activity.LotteryGameComingSoonImMsg;
import com.starbuds.server.common.pojo.message.im.biz.SystemMsg;
import com.starbuds.server.common.pojo.message.im.rtc.RtcAllRoomNoticeStyleMsg;
import com.starbuds.server.common.pojo.mq.LotteryGameStatisticMsg;
import com.starbuds.server.common.pojo.mq.lotterygame.QueueLotteryGameSendRewardMsg;
import com.starbuds.server.common.pojo.user.UserSimpleVO;
import com.starbuds.server.common.service.activity.LotteryGameService;
import com.starbuds.server.common.service.data.PropsService;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.media.GiftService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.message.biz.LotteryGameImMsgService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.common.utils.LogUtil;
import com.starbuds.server.service.activity.dao.LotteryGameDAO;
import com.starbuds.server.service.activity.dao.LotteryGameSnapshotDAO;
import com.starbuds.server.service.activity.dao.LotteryGameWinHisDAO;
import com.starbuds.server.service.activity.events.EventService;
import com.starbuds.server.service.activity.events.event.LotteryGameCleanCacheEvent;
import com.starbuds.server.service.activity.events.message.LotteryGameCleanCacheMsg;
import com.starbuds.server.service.activity.pojo.bo.LotteryGameWinNoticeMsg;
import com.starbuds.server.service.activity.provider.LotteryGameProvider;
import com.starbuds.server.service.activity.provider.LotteryGameSnapshotProvider;
import com.starbuds.server.service.activity.rocket.RocketProducer;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang3.BooleanUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.math.NumberUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

import static com.starbuds.server.common.constant.activity.LotteryGameTimerStatusEnum.IN_PROGRESS;
import static com.starbuds.server.common.constant.activity.LotteryGameTimerStatusEnum.OVER;
import static com.starbuds.server.service.activity.util.SpringContextUtil.isDev;

/**
 * 标记抽奖游戏结束
 *
 * @author aoe
 * @date 2021/11/26
 */
@Service
@Slf4j
public class MarkUpEndServiceImpl {

    @Reference(version = "1.0.0")
    private RedisService redisService;

    @Reference(version = "1.0.0")
    private GiftService giftService;

    @Reference(version = "1.0.0")
    private PropsService propsService;

    @Resource
    private LotteryGameProvider lotteryGameProvider;

    @Reference(version = "1.0.0")
    private LotteryGameImMsgService lotteryGameImMsgService;

    @Resource
    private LotteryGameSnapshotDAO lotteryGameSnapshotDAO;

    @Resource
    private LotteryGameSnapshotProvider lotteryGameSnapshotProvider;

    @Resource
    private RocketProducer rocketProducer;

    @Resource
    private LotteryGameDAO lotteryGameDAO;

    @Resource
    private LotteryGameWinHisDAO lotteryGameWinHisDAO;

    @Resource
    private LotteryGameService lotteryGameService;

    @Reference(version = "1.0.0")
    private ImService imService;

    @Reference(version = "1.0.0")
    private UserService userService;

    @Resource
    private EventService eventService;

    /**
     * 标记抽奖游戏结束
     *
     * @param snapId
     * @return true:成功
     */
    public boolean markUpEnd(long snapId) {
        String key = RedisActivityKeys.getLotteryGameStatus(snapId);
        Boolean del = redisService.delKey(key);
        if (BooleanUtils.isFalse(del)) {
            log.warn(String.format("标记抽奖游戏结束,删除 【抽奖游戏状态】 失败 key=%s snapId=%s", key, snapId));
            return false;
        }
        log.info("标记抽奖游戏结束删除Key【抽奖游戏状态】成功 {}", key);

        LotteryGamePlayTypeEnum playType = LotteryGamePlayTypeEnum.toEnum(lotteryGameSnapshotDAO.selectPlayType(snapId));
        if (playType == null) {
            log.warn("标记抽奖游戏结束失败 playType is null snapId=" + snapId);
            return false;
        }

        Long timerId = lotteryGameSnapshotDAO.selectTimerIdBySnapId(snapId);
        Long canTimer2Over = lotteryGameDAO.selectCanTimer2Over(timerId, IN_PROGRESS.value);
        if (canTimer2Over == null) {
            log.warn("标记抽奖游戏结束失败 没有符合要求的活动能结束 snapId={} timerId={}", snapId, timerId);
            return false;
        }
        int updateTimerToOver = updateTimerToOver(timerId);
        if (updateTimerToOver != 1) {
            log.warn("标记抽奖游戏结束失败 更新抽奖定时器配置状态为已结束失败 snapId=" + snapId);
            return false;
        }

        LotteryGameComingSoonImMsg imMsg = new LotteryGameComingSoonImMsg();
        imMsg.setSnapId(snapId);
        imMsg.setMsg(playType.desc + "已结束，准备开奖");
        imMsg.setPlayType(playType.value);
        boolean send = lotteryGameImMsgService.sendComingSoonImMsg(imMsg);
        if (!send) {
            log.error("标记抽奖游戏结束失败 发送即将开奖消息失败 snapId=" + snapId);
            return false;
        }

        List<Long> winUserIds = lotteryGameWinHisDAO.selectUserIds(snapId);
        sendRewards(snapId, playType, winUserIds);

        // 发送抽奖游戏IM消息-开奖结果
        boolean sendImMsg = lotteryGameImMsgService.sendLotteryResultImMsg(snapId, playType);
        if (!sendImMsg) {
            log.error("发送抽奖游戏IM消息-开奖结果失败 snapId=" + snapId, " playType=" + playType.desc);
        }
        log.info("发送抽奖游戏IM消息-开奖结果成功 snapId={} playType={}", snapId, playType.desc);

        // 更新统计数据
        updateStat(snapId, winUserIds.size());
        cleanCache(snapId, winUserIds);
        return true;
    }

    private void sendRewards(long snapId, LotteryGamePlayTypeEnum playType, List<Long> winUserIds) {
        if (CollectionUtils.isEmpty(winUserIds)) {
            log.info("抽奖游戏结束：没有中奖的用户");
            return;
        }

        final AddItemToBagSourceEnum source = LotteryGamePlayTypeEnum.DIVIDE_CAKE == playType ?
                AddItemToBagSourceEnum.DIVIDE_CAKE : AddItemToBagSourceEnum.RED_BAG;
        Map<String, LotteryGameRewardSummary> rewardSummaryMap = rewardSummaryMap(snapId);
        // key:物品价值 需要客户端支持发送消息
        Map<Long, QueueLotteryGameSendRewardMsg> rewardValueMap = new HashMap<>();
        // ⚠️ 多线程环境下允许丢失部分消息
        List<LotteryGameWinNoticeMsg> winNoticeMessages = new ArrayList<>();

        winUserIds.parallelStream()
                .forEach(userId -> {
                    List<QueueLotteryGameSendRewardMsg> messages = getSendRewardMessages(snapId, userId, rewardSummaryMap, source, playType.desc);
                    List<String> divideCakeWinItems = new ArrayList<>();
                    messages.stream()
                            .filter(Objects::nonNull)
                            .forEach(msg -> {
                                boolean sendMsg = rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.LOTTERY_GAME_SEND_REWARD, msg.serialize());
                                if (!sendMsg) {
                                    log.error("抽奖游戏发送发放奖品消息失败 " + JSON.toJSONString(msg));
                                    return;
                                }
                                if (AddItemToBagSourceEnum.DIVIDE_CAKE == msg.getSource()) {
                                    divideCakeWinItems.add(String.format("【%s】%s个", msg.getItemName(), msg.getQuantity()));
                                    rewardValueMap.put(msg.getItemValue(), msg);
                                }
                                // 添加需要发送中奖通知的消息
                                addWinNoticeMsg(winNoticeMessages, rewardSummaryMap, msg);
                                if (isDev()) {
                                    log.info("抽奖游戏发送发放奖品消息 {} {}", sendMsg, JSON.toJSONString(msg, true));
                                }
                            });
                    sendDivideCakeWinImMsg(userId, divideCakeWinItems);
                });
        sendDivideCakeTop3Winners(rewardValueMap, winUserIds.size());
        // 发送全房间带样式的通知
        sendRtcAllRoomStyleNotice(source.getName(), winNoticeMessages);
    }

    private void sendDivideCakeWinImMsg(Long userId, List<String> divideCakeWinItems) {
        if (CollectionUtils.isEmpty(divideCakeWinItems)) {
            return;
        }
        String winItems = StringUtils.join(divideCakeWinItems, ",");
        SystemMsg systemMsg = new SystemMsg();
        systemMsg.setContentType(ContentType.TEXT);
        systemMsg.setTitle(AddItemToBagSourceEnum.DIVIDE_CAKE.getName());
        systemMsg.setBody(winItems);
        systemMsg.setActionType(ImActionType.NO_ACTION.getValue());
        imService.sendCustomImMsg(null, userId, systemMsg, null);
    }

    private void sendDivideCakeTop3Winners(Map<Long, QueueLotteryGameSendRewardMsg> rewardValueMap, int winUserCount) {
        if (MapUtils.isEmpty(rewardValueMap)) {
            return;
        }
        List<Long> top3Values = rewardValueMap.keySet()
                .stream()
                .sorted(Comparator.reverseOrder())
                .limit(3)
                .collect(Collectors.toList());
        if (CollectionUtils.isEmpty(top3Values)) {
            return;
        }

        List<QueueLotteryGameSendRewardMsg> winners = top3Values.stream()
                .map(key -> rewardValueMap.get(key))
                .collect(Collectors.toList());
        List<Long> userIds = winners.stream()
                .map(winner -> winner.getUserId())
                .collect(Collectors.toList());
        Map<Long, UserDO> userMap = userService.getUserMapByIds(userIds);

        List<String> msgList = winners.stream()
                .map(winner -> {
                    String userName = "牛二";
                    UserDO user = userMap.get(winner.getUserId());
                    if (user != null) {
                        userName = user.getUserName();
                    }
                    return String.format("%s获得【%s】", userName, winner.getItemName());
                })
                .collect(Collectors.toList());

        String top3Msg = String.format("恭喜%s等%s名用户获奖！", StringUtils.join(msgList, "、"), winUserCount);
        imService.sendAllRoomNotice(top3Msg);
    }

    private List<QueueLotteryGameSendRewardMsg> getSendRewardMessages(long snapId, Long userId,
                                                                      Map<String, LotteryGameRewardSummary> rewardSummaryMap,
                                                                      AddItemToBagSourceEnum source, String msgDesc) {
        String key = RedisActivityKeys.getLotteryGameWinHisItems(snapId, userId);
        String value = redisService.get(key);
        if (StringUtils.isBlank(value)) {
            return Collections.emptyList();
        }

        List<QueueLotteryGameSendRewardMsg> messages = new ArrayList<>();
        JSONArray rewards = JSON.parseArray(value);
        for (int i = 0; i < rewards.size(); i++) {
            // [奖品id-物品类型-数量]
            String[] reward = StringUtils.split(rewards.getString(i), "-");
            if (reward.length != 3) {
                log.error(String.format("中奖信息有误 snapId=%s userId=%s value=%s", snapId, userId, value));
                continue;
            }

            long rewardId = NumberUtils.toLong(reward[0]);
            int rewardType = NumberUtils.toInt(reward[1]);
            int quantity = NumberUtils.toInt(reward[2]);
            String rewardSummaryKey = getRewardSummaryMapKey(rewardId, rewardType);
            LotteryGameRewardSummary rewardSummary = rewardSummaryMap.get(rewardSummaryKey);

            LotteryGameRewardTypeEnum rewardTypeEnum = LotteryGameRewardTypeEnum.toEnum(rewardSummary.getRewardType());
            if (LotteryGameRewardTypeEnum.NONE == rewardTypeEnum) {
                continue;
            }

            QueueLotteryGameSendRewardMsg msg = new QueueLotteryGameSendRewardMsg();
            msg.setSnapId(snapId);
            msg.setSource(source);
            msg.setItemId(rewardId);
            String rewardName = rewardSummary.getRewardName();
            msg.setItemName(rewardName);
            BagItemTypeEnum itemType = BagItemTypeEnum.PROPS;
            if (LotteryGameRewardTypeEnum.GIFT == rewardTypeEnum) {
                itemType = BagItemTypeEnum.GIFT;
            }
            msg.setItemType(itemType);
            msg.setQuantity(quantity);
            msg.setItemValue(rewardSummary.getRewardValue());
            msg.setExpireTime(-1L);
            msg.setUserId(userId);
            msg.setRemark(String.format("%s获得%s个%s", msgDesc, quantity, rewardName));
            messages.add(msg);
        }

        return messages;
    }

    private Map<String, LotteryGameRewardSummary> rewardSummaryMap(long snapId) {
        List<LotteryGameRewardSummary> rewardSummaries = getRewardSummaries(snapId);
        if (CollectionUtils.isEmpty(rewardSummaries)) {
            return Collections.emptyMap();
        }

        Map<String, LotteryGameRewardSummary> rewardSummaryMap = new HashMap<>();
        rewardSummaries.stream()
                .filter(Objects::nonNull)
                .forEach(summary -> {
                    String summaryKey = getRewardSummaryMapKey(summary.getRewardId(), summary.getRewardType());
                    rewardSummaryMap.put(summaryKey, summary);
                });
        return rewardSummaryMap;
    }

    private List<LotteryGameRewardSummary> getRewardSummaries(long snapId) {
        String key = RedisActivityKeys.getLotteryGameRewardSummaries(snapId);
        String value = redisService.get(key);
        if (StringUtils.isBlank(value)) {
            return Collections.emptyList();
        }
        return JSON.parseArray(value, LotteryGameRewardSummary.class);
    }

    private String getRewardSummaryMapKey(Long rewardId, Integer rewardType) {
        return String.format("%s-%s", rewardId, rewardType);
    }

    /**
     * 是否可以结束
     *
     * @param snapId
     * @param now
     * @param minEndTime
     * @param maxEndTime
     * @return true:可以结束
     */
    public boolean canEnd(long snapId, long now, long minEndTime, long maxEndTime) {
        if (now < minEndTime) {
            // 没有到达活动最早结束时间
            return false;
        }
        if (maxEndTime <= now) {
            log.info("抽奖活动可以结束-到达活动最晚结束时间");
            return true;
        }
        if (lotteryGameProvider.getRewardCount(snapId) < 1) {
            log.info("抽奖活动可以结束-奖项总数量=0");
            return true;
        }
        return false;
    }

    /**
     * 更新抽奖定时器配置状态为已结束
     *
     * @param timerId
     * @return
     */
    public int updateTimerToOver(long timerId) {
        return lotteryGameDAO.updateTimer2Over(timerId, OVER.value, IN_PROGRESS.value, System.currentTimeMillis());
    }

    /**
     * 保存抽奖活动开出的奖品列表到缓存
     *
     * @param snapId
     * @return true 成功
     */
    @Deprecated
    private void saveWinItems2CacheOld(long snapId) {
        List<LotteryGameWinItemBO> winItems = lotteryGameSnapshotProvider.getLotteryGameRewards(snapId);
        final Map<Long, PriceInfoBO> giftPriceMap = giftService.getGiftPriceMap(getItemIds(winItems, LotteryGameRewardTypeEnum.GIFT));
        final Map<Long, PriceInfoBO> propsPriceMap = propsService.getPropsPriceMap(getItemIds(winItems, LotteryGameRewardTypeEnum.PROPS));
        if (isDev()) {
            log.info(LogUtil.obj2JsonObj(giftPriceMap));
            log.info(LogUtil.obj2JsonObj(propsPriceMap));
        }
        winItems = winItems.stream()
                .filter(Objects::nonNull)
                .map(item -> addPriceInfo(item, giftPriceMap, propsPriceMap))
                .collect(Collectors.toList());
        redisService.set(RedisActivityKeys.getLotteryGameWinItems(snapId), JSON.toJSONString(winItems));
    }

    private List<LotteryGameWinItemBO> getWinItems(List<LotteryGameRewardBO> rewards) {
        if (CollectionUtils.isEmpty(rewards)) {
            Collections.emptyList();
        }

        List<LotteryGameWinItemBO> winItems = rewards.stream()
                .filter(Objects::nonNull)
                .map(reward -> {
                    LotteryGameWinItemBO winItem = new LotteryGameWinItemBO();
                    BeanUtils.copyProperties(reward, winItem);
                    return winItem;
                })
                .collect(Collectors.toList());

        final Map<Long, PriceInfoBO> giftPriceMap = giftService.getGiftPriceMap(getItemIds(winItems, LotteryGameRewardTypeEnum.GIFT));
        final Map<Long, PriceInfoBO> propsPriceMap = propsService.getPropsPriceMap(getItemIds(winItems, LotteryGameRewardTypeEnum.PROPS));
        if (isDev()) {
            log.info(LogUtil.obj2JsonObj(giftPriceMap));
            log.info(LogUtil.obj2JsonObj(propsPriceMap));
        }
        return winItems.stream()
                .filter(Objects::nonNull)
                .map(item -> addPriceInfo(item, giftPriceMap, propsPriceMap))
                .collect(Collectors.toList());
    }

    /**
     * 获得奖品id
     *
     * @param winItems
     * @param typeEnum
     * @return
     */
    @Deprecated
    private List<Long> getItemIds(List<LotteryGameWinItemBO> winItems, LotteryGameRewardTypeEnum typeEnum) {
        return winItems.stream()
                .filter(Objects::nonNull)
                .filter(item -> LotteryGameRewardTypeEnum.toEnum(item.getRewardType()) == typeEnum)
                .map(item -> item.getRewardId())
                .distinct()
                .collect(Collectors.toList());
    }

    /**
     * 添加价格信息
     *
     * @param item
     * @param giftPriceMap
     * @param propsPriceMap
     * @return
     */
    private LotteryGameWinItemBO addPriceInfo(LotteryGameWinItemBO item, Map<Long, PriceInfoBO> giftPriceMap,
                                              Map<Long, PriceInfoBO> propsPriceMap) {
        LotteryGameRewardTypeEnum type = LotteryGameRewardTypeEnum.toEnum(item.getRewardType());
        if (type == null || type == LotteryGameRewardTypeEnum.NONE) {
            item.setRewardPrice(0L);
            return item;
        }
        Long id = item.getRewardId();
        PriceInfoBO priceInfo = type == LotteryGameRewardTypeEnum.GIFT ? giftPriceMap.get(id) : propsPriceMap.get(id);
        item.setRewardPrice(priceInfo.getPrice());
        return item;
    }

    private void cleanCache(final long snapId, final List<Long> winUserIds) {
        List<LotteryGameRewardSummary> rewardSummaries = getRewardSummaries(snapId);
        eventService.publish(new LotteryGameCleanCacheEvent(
                new LotteryGameCleanCacheMsg(snapId, winUserIds, rewardSummaries)
        ));
    }

    /**
     * 更新统计数据
     *
     * @param snapId
     * @param winUserCount 中奖用户数量
     */
    private void updateStat(long snapId, int winUserCount) {
        if (winUserCount < 1) {
            log.info("没有人中奖，不发送抽奖游戏统计消息 snapId={}", snapId);
            return;
        }

        LotteryGameStatisticMsg msg = new LotteryGameStatisticMsg();
        msg.setSnapId(snapId);
        msg.setRewardNum(winUserCount);
        msg.setGiveRewardTotal(lotteryGameWinHisDAO.selectRewardTotalPrice(snapId));
        lotteryGameService.buildStatistic(msg.getSnapId(), msg.getRewardNum(), msg.getGiveRewardTotal());
    }

    /**
     * 添加需要发送中奖通知的消息
     *
     * @param winNoticeMessages
     * @param rewardSummaryMap
     * @param msg
     */
    private void addWinNoticeMsg(final List<LotteryGameWinNoticeMsg> winNoticeMessages,
                                 final Map<String, LotteryGameRewardSummary> rewardSummaryMap,
                                 final QueueLotteryGameSendRewardMsg msg) {
        String rewardSummaryKey = getRewardSummaryMapKey(msg.getItemId(), msg.getItemType().getValue());
        LotteryGameRewardSummary rewardSummary = rewardSummaryMap.get(rewardSummaryKey);
        if (rewardSummary == null) {
            return;
        }
        LotteryGameWinNoticeTypeEnum winNoticeType = LotteryGameWinNoticeTypeEnum.toEnum(rewardSummary.getWinNoticeType());
        if (winNoticeType == null) {
            return;
        }
        if (LotteryGameWinNoticeTypeEnum.NONE == winNoticeType) {
            return;
        }
        LotteryGameWinNoticeMsg winNoticeMsg = LotteryGameWinNoticeMsg.builder()
                .itemName(msg.getItemName())
                .quantity(msg.getQuantity())
                .itemValue(msg.getItemValue())
                .userId(msg.getUserId())
                .winNoticeType(winNoticeType.value)
                .build();
        winNoticeMessages.add(winNoticeMsg);
    }

    /**
     * 发送全房间带样式的通知
     *
     * @param chartTitle
     * @param winNoticeMessages
     */
    private void sendRtcAllRoomStyleNotice(String chartTitle, final List<LotteryGameWinNoticeMsg> winNoticeMessages) {
        List<Long> userIds = winNoticeMessages.stream()
                .map(LotteryGameWinNoticeMsg::getUserId)
                .filter(Objects::nonNull)
                .collect(Collectors.toList());
        Map<Long, UserSimpleVO> userMap = userService.getSimpleUserMap(userIds);

        winNoticeMessages.parallelStream()
                .map(msg -> getRtcAllRoomNoticeStyleMsg(chartTitle, msg, userMap))
                .forEach(winNoticeMsg -> {
                    boolean sendAllRoomMsg = imService.sendAllRoomMsg(winNoticeMsg);
                    if (!sendAllRoomMsg) {
                        log.error("发送全房间带样式的通知失败 " + winNoticeMsg);
                    }
                });
    }

    private RtcAllRoomNoticeStyleMsg getRtcAllRoomNoticeStyleMsg(String chartTitle, LotteryGameWinNoticeMsg winNoticeMsg,
                                                                 Map<Long, UserSimpleVO> userMap) {
        RtcAllRoomNoticeStyleMsg msg = new RtcAllRoomNoticeStyleMsg();
        msg.setChartTitle(chartTitle);
        Long userId = winNoticeMsg.getUserId();
        msg.setReceiverId(userId);
        UserSimpleVO user = userMap.get(userId);
        if (user != null) {
            msg.setReceiverName(user.getUserName());
            msg.setReceiverAvatar(user.getUserAvatar());
        }
        msg.setRewardName(winNoticeMsg.getItemName());
        Integer quantity = winNoticeMsg.getQuantity();
        msg.setRewardQuantity(quantity);
        msg.setRewardTotal(winNoticeMsg.getItemValue() * quantity);

        return msg;
    }
}
