package com.starbuds.server.service.activity.lotterygame;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.finance.AddItemToBagSourceEnum;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rocketmq.RocketTopic;
import com.starbuds.server.common.pojo.db.LotteryGameExecuteHisDO;
import com.starbuds.server.common.pojo.mq.QueueAddItemToBagMsg;
import com.starbuds.server.common.pojo.mq.lotterygame.QueueLotteryGameSendRewardMsg;
import com.starbuds.server.common.pojo.mq.statcommonbiz.QueueDivideCakeSendGiftTotalValueMsg;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.service.activity.dao.LotteryGameExecuteHisDAO;
import com.starbuds.server.service.activity.rocket.RocketProducer;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;

import static com.google.common.base.Preconditions.checkState;

/**
 * 发放奖品
 *
 * @author aoe
 * @date 2021/12/22
 */
@Service
@Slf4j
public class SendRewardServiceImpl {

    @Resource
    private LotteryGameExecuteHisDAO lotteryGameExecuteHisDAO;

    @Resource
    private RocketProducer rocketProducer;

    @Reference(version = "1.0.0")
    private ImService imService;

    /**
     * 发放奖品
     *
     * @param msg
     */
    @Transactional
    public void sendReward(QueueLotteryGameSendRewardMsg msg) {
        LotteryGameExecuteHisDO executeHis = getExecuteHis(msg);
        int insert = lotteryGameExecuteHisDAO.insert(executeHis);
        checkState(insert == 1);

        QueueAddItemToBagMsg addItemToBagMsg = getAddItemToBagMsg(executeHis.getHisId(), msg);
        boolean sendMsg = rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.ADD_ITEM_TO_BAG, addItemToBagMsg.serialize());
        String msgStr = JSON.toJSONString(msg);
        if (!sendMsg) {
            log.error("发放奖品失败 " + msgStr);
            return;
        }
        log.info("发放奖品成功 {}", msgStr);
        sendStatDivideCakeSendGiftTotalValueMsg(executeHis.getHisId(), msg);
    }

    private LotteryGameExecuteHisDO getExecuteHis(QueueLotteryGameSendRewardMsg msg) {
        LotteryGameExecuteHisDO po = new LotteryGameExecuteHisDO();
        po.setSnapId(msg.getSnapId());
        po.setUserId(msg.getUserId());
        po.setRewardId(msg.getItemId());
        po.setCreateTime(System.currentTimeMillis());
        return po;
    }

    private QueueAddItemToBagMsg getAddItemToBagMsg(Long orderId, QueueLotteryGameSendRewardMsg sendRewardMsg) {
        QueueAddItemToBagMsg msg = new QueueAddItemToBagMsg();
        BeanUtils.copyProperties(sendRewardMsg, msg);
        msg.setOrderId(orderId);
        if (AddItemToBagSourceEnum.DIVIDE_CAKE == sendRewardMsg.getSource()) {
            msg.setNoSendImMsg(true);
        }
        return msg;
    }

    /**
     * 发送瓜分蛋糕发放奖品统计消息
     *
     * @param hisId
     * @param msg
     */
    private void sendStatDivideCakeSendGiftTotalValueMsg(Long hisId, QueueLotteryGameSendRewardMsg msg) {
        if (AddItemToBagSourceEnum.DIVIDE_CAKE != msg.getSource()) {
            return;
        }

        QueueDivideCakeSendGiftTotalValueMsg sendGiftTotalValueMsg = new QueueDivideCakeSendGiftTotalValueMsg();
        sendGiftTotalValueMsg.setOrderId(hisId);
        Long total = msg.getQuantity() * msg.getItemValue();
        sendGiftTotalValueMsg.setTotalValue(total);
        boolean sendMsg = rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.DIVIDE_CAKE_SEND_GIFT_TOTAL_VALUE, sendGiftTotalValueMsg.serialize());
        String msgStr = JSON.toJSONString(msg);
        if (!sendMsg) {
            log.error("发送瓜分蛋糕发放奖品统计消息失败 " + msgStr);
            return;
        }
        log.info("发送瓜分蛋糕发放奖品统计消息成功 {}", msgStr);
    }
}
