package com.starbuds.server.service.activity.lotterygame.task;

import com.starbuds.server.common.pojo.db.LotteryGameSnapshotDO;
import com.starbuds.server.common.service.message.biz.LotteryGameImMsgService;
import com.starbuds.server.service.activity.dao.LotteryGameSnapshotDAO;
import com.starbuds.server.service.activity.lotterygame.MarkUpEndServiceImpl;
import com.starbuds.server.service.activity.provider.LotteryGameProvider;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.List;

/**
 * 标记抽奖游戏结束
 *
 * @author aoe
 * @date 2021/11/26
 */
@Slf4j
@Component
public class MarkUpActivityEndTask implements LotteryGameTimedTask {

    @Resource
    private LotteryGameSnapshotDAO lotteryGameSnapshotDAO;

    @Reference(version = "1.0.0")
    private LotteryGameImMsgService lotteryGameImMsgService;

    @Resource
    private LotteryGameProvider lotteryGameProvider;

    @Resource
    private MarkUpEndServiceImpl markUpEndService;

    @Override
    public boolean doTask() {
        final long now = System.currentTimeMillis();
        List<LotteryGameSnapshotDO> snapList = lotteryGameProvider.getInProgressingSnaps();
        if (CollectionUtils.isEmpty(snapList)) {
            return false;
        }
        snapList.stream()
                .forEach(snap -> {
                    long snapId = snap.getSnapId();
                    if (!markUpEndService.canEnd(snapId, now, snap.getMinEndTime(), snap.getMaxEndTime())) {
                        return;
                    }
                    markUpEndService.markUpEnd(snapId);
                });
        return true;
    }

}
