package com.starbuds.server.service.activity.lotterygame.task;

import com.starbuds.server.common.constant.cache.RedisActivityKeys;
import com.starbuds.server.common.pojo.db.LotteryGameSnapshotDO;
import com.starbuds.server.common.service.data.PropsService;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.media.GiftService;
import com.starbuds.server.service.activity.dao.LotteryGameSnapshotDAO;
import com.starbuds.server.service.activity.provider.LotteryGameSnapshotProvider;
import com.starbuds.server.service.activity.util.TimeUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.List;
import java.util.Objects;

/**
 * 标记抽奖游戏开始
 *
 * @author aoe
 * @date 2021/11/26
 */
@Slf4j
@Component
public class MarkUpActivityStartTask implements LotteryGameTimedTask {

    @Resource
    private LotteryGameSnapshotDAO lotteryGameSnapshotDAO;

    @Reference(version = "1.0.0")
    private RedisService redisService;

    @Resource
    private LotteryGameSnapshotProvider lotteryGameSnapshotProvider;

    @Reference(version = "1.0.0")
    private GiftService giftService;

    @Reference(version = "1.0.0")
    private PropsService propsService;

    @Override
    public boolean doTask() {
        final long now = System.currentTimeMillis();
        List<LotteryGameSnapshotDO> snapList = lotteryGameSnapshotDAO.selectListByStartTime(TimeUtil.currentStr(now));
        if (CollectionUtils.isEmpty(snapList)) {
            return false;
        }

        snapList.stream()
                .filter(Objects::nonNull)
                .filter(snap -> TimeUtil.equalsTargetTime(snap.getStartTime(), now))
                .forEach(snap -> {
                    long snapId = snap.getSnapId();
                    // 标记活动已开始
                    redisService.set(RedisActivityKeys.getLotteryGameStatus(snapId), "inProgressing");
                    log.info("标记抽奖游戏开始 snapId={}", snapId);
                });
        return true;
    }

}
