package com.starbuds.server.service.activity.provider;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.pojo.db.ActivityAnnualAwardGiveHisDO;
import com.starbuds.server.service.activity.dao.ActivityAnnualAwardGiveHisDAO;
import com.starbuds.server.common.pojo.activity.qo.ActivityAnnualAwardGiveHisPageListQO;

import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.pojo.api.PageList;
import lombok.extern.slf4j.Slf4j;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.service.core.IdentityService;

import static com.starbuds.server.common.pojo.util.PageListUtil.buildPageListQueryWrapper;

import org.springframework.stereotype.Component;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.transaction.annotation.Transactional;

/**
 * 用户礼物发放记录
 */
@Component
@Slf4j
public class ActivityAnnualAwardGiveHisProvider {

    /**
     * 主键
     */
    private String PRIMARY_KEY = "hisId";

    @Reference(version = "1.0.0")
    private IdentityService identityService;

    @Resource
    private ActivityAnnualAwardGiveHisDAO activityAnnualAwardGiveHisDAO;

    /**
     * 插入数据
     *
     * @param po
     * @return null:插入失败
     */
    @Transactional(rollbackFor = Exception.class)
    public ActivityAnnualAwardGiveHisDO create(ActivityAnnualAwardGiveHisDO po) {
        po.setHisId(identityService.getId());
        po.setCreateTime(System.currentTimeMillis());
        boolean flag = activityAnnualAwardGiveHisDAO.insert(po) == 1;

        String json = JSON.toJSONString(po);
        if (flag) {
            log.info("用户礼物发放记录创建成功 {}", json);
            return po;
        }
        log.warn("用户礼物发放记录创建失败 {}", json);
        return null;
    }

    /**
     * 插入批量数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createBatch(List<ActivityAnnualAwardGiveHisDO> list) {
        boolean flag = activityAnnualAwardGiveHisDAO.insertBatch(list) > 0;
        return flag;
    }

    /**
     * 插入或更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createOrUpdate(ActivityAnnualAwardGiveHisDO po) {
        boolean flag = activityAnnualAwardGiveHisDAO.insertOrUpdate(po) == 1;
        return flag;
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean updateById(ActivityAnnualAwardGiveHisDO po) {
        po.setCreateTime(null);

        UpdateWrapper uw = new UpdateWrapper();
        uw.setPo(po);
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, po.getHisId());
        uw.setQw(qw);
        return update(uw);
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean update(UpdateWrapper uw) {
        boolean flag = activityAnnualAwardGiveHisDAO.update(uw) > 0;
        return flag;
    }

    /**
     * 根据id查询
     */
    public ActivityAnnualAwardGiveHisDO getById(Long id) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return activityAnnualAwardGiveHisDAO.selectOne(qw);
    }

    /**
     * 判断是否存在
     *
     * @param id 主键
     * @return true:存在
     */
    public boolean exist(Long id) {
        if (id == null) {
            return false;
        }
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return activityAnnualAwardGiveHisDAO.selectCount(qw) > 0;
    }

    /**
     * 分页查询
     */
    public PageList<ActivityAnnualAwardGiveHisDO> getPageList(ActivityAnnualAwardGiveHisPageListQO qo) {
        QueryWrapper qw = buildPageListQueryWrapper(qo);
        PageList<ActivityAnnualAwardGiveHisDO> page = new PageList<>(qo.getPageIndex(), qo.getPageSize());
        qw.setPageCursor(page.getPageCursor());
        qw.setPageSize(page.getPageSize());
        page.setList(activityAnnualAwardGiveHisDAO.selectPageList(qw));
        page.setTotal(activityAnnualAwardGiveHisDAO.selectCount(qw));
        return page;
    }

    /**
     * 根据ids获得 DO Map
     *
     * @param ids id集合
     * @return Map<主键, DO对象>
     */
    public Map<Long, ActivityAnnualAwardGiveHisDO> getMapByIds(Collection ids) {
        return activityAnnualAwardGiveHisDAO.selectMapByIds(ids);
    }

    // 手动编码开始

}