package com.starbuds.server.service.activity.provider;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.pojo.activity.qo.ActivityGiftsRankReceivePageListQO;
import com.starbuds.server.common.pojo.db.ActivityGiftsRankReceiveDO;
import com.starbuds.server.service.activity.dao.ActivityGiftsRankReceiveDAO;
import com.starbuds.server.common.pojo.rtc.qo.RtcRoomAuctionPageListQO;

import com.starbuds.server.common.pojo.api.PageList;
import lombok.extern.slf4j.Slf4j;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.service.core.IdentityService;

import static com.starbuds.server.common.pojo.util.PageListUtil.buildPageListQueryWrapper;

import org.springframework.stereotype.Component;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.transaction.annotation.Transactional;

/**
 * 多礼物活动收礼榜单，按天分隔。收礼人当天只有一条记录
 */
@Component
@Slf4j
public class ActivityGiftsRankReceiveProvider {

    /**
     * 主键
     */
    private String PRIMARY_KEY = "activityGiftsRankReceiveId";

    @Reference(version = "1.0.0")
    private IdentityService identityService;

    @Resource
    private ActivityGiftsRankReceiveDAO activityGiftsRankReceiveDAO;

    /**
     * 插入数据
     *
     * @param po
     * @return null:插入失败
     */
    @Transactional(rollbackFor = Exception.class)
    public ActivityGiftsRankReceiveDO create(ActivityGiftsRankReceiveDO po) {
        po.setActivityGiftsRankReceiveId(identityService.getId());
        po.setCreateTime(System.currentTimeMillis());
        po.setUpdateTime(po.getCreateTime());
        boolean flag = activityGiftsRankReceiveDAO.insert(po) == 1;

        String json = JSON.toJSONString(po);
        if (flag) {
            log.info("多礼物活动收礼榜单，按天分隔。收礼人当天只有一条记录创建成功 {}", json);
            return po;
        }
        log.warn("多礼物活动收礼榜单，按天分隔。收礼人当天只有一条记录创建失败 {}", json);
        return null;
    }

    /**
     * 插入批量数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createBatch(List<ActivityGiftsRankReceiveDO> list) {
        boolean flag = activityGiftsRankReceiveDAO.insertBatch(list) > 0;
        return flag;
    }

    /**
     * 插入或更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createOrUpdate(ActivityGiftsRankReceiveDO po) {
        boolean flag = activityGiftsRankReceiveDAO.insertOrUpdate(po) == 1;
        return flag;
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean updateById(ActivityGiftsRankReceiveDO po) {
        po.setCreateTime(null);
        po.setUpdateTime(System.currentTimeMillis());

        UpdateWrapper uw = new UpdateWrapper();
        uw.setPo(po);
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, po.getActivityGiftsRankReceiveId());
        uw.setQw(qw);
        return update(uw);
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean update(UpdateWrapper uw) {
        boolean flag = activityGiftsRankReceiveDAO.update(uw) > 0;
        return flag;
    }

    /**
     * 根据id查询
     */
    public ActivityGiftsRankReceiveDO getById(Long id) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return activityGiftsRankReceiveDAO.selectOne(qw);
    }

    /**
     * 判断是否存在
     *
     * @param id 主键
     * @return true:存在
     */
    public boolean exist(Long id) {
        if (id == null) {
            return false;
        }
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return activityGiftsRankReceiveDAO.selectCount(qw) > 0;
    }

    /**
     * 分页查询
     */
    public PageList<ActivityGiftsRankReceiveDO> getPageList(ActivityGiftsRankReceivePageListQO qo) {
        QueryWrapper qw = buildPageListQueryWrapper(qo);
        PageList<ActivityGiftsRankReceiveDO> page = new PageList<>(qo.getPageIndex(), qo.getPageSize());
        qw.setPageCursor(page.getPageCursor());
        qw.setPageSize(page.getPageSize());
        page.setList(activityGiftsRankReceiveDAO.selectPageList(qw));
        page.setTotal(activityGiftsRankReceiveDAO.selectCount(qw));
        return page;
    }

    /**
     * 根据ids获得 DO Map
     *
     * @param ids id集合
     * @return Map<主键, DO对象>
     */
    public Map<Long, ActivityGiftsRankReceiveDO> getMapByIds(Collection ids) {
        return activityGiftsRankReceiveDAO.selectMapByIds(ids);
    }

    // 手动编码开始

}