package com.starbuds.server.service.activity.provider;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.pojo.activity.qo.ActivityGiftsRankSendPageListQO;
import com.starbuds.server.common.pojo.db.ActivityGiftsRankSendDO;
import com.starbuds.server.service.activity.dao.ActivityGiftsRankSendDAO;
import com.starbuds.server.common.pojo.rtc.qo.RtcRoomAuctionPageListQO;

import com.starbuds.server.common.pojo.api.PageList;
import lombok.extern.slf4j.Slf4j;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.service.core.IdentityService;

import static com.starbuds.server.common.pojo.util.PageListUtil.buildPageListQueryWrapper;

import org.springframework.stereotype.Component;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.transaction.annotation.Transactional;

/**
 * 多礼物活动赠送榜单，按天分隔。赠送人当天只有一条记录
 */
@Component
@Slf4j
public class ActivityGiftsRankSendProvider {

    /**
     * 主键
     */
    private String PRIMARY_KEY = "activityGiftsRankSendId";

    @Reference(version = "1.0.0")
    private IdentityService identityService;

    @Resource
    private ActivityGiftsRankSendDAO activityGiftsRankSendDAO;

    /**
     * 插入数据
     *
     * @param po
     * @return null:插入失败
     */
    @Transactional(rollbackFor = Exception.class)
    public ActivityGiftsRankSendDO create(ActivityGiftsRankSendDO po) {
        po.setActivityGiftsRankSendId(identityService.getId());
        po.setCreateTime(System.currentTimeMillis());
        po.setUpdateTime(po.getCreateTime());
        boolean flag = activityGiftsRankSendDAO.insert(po) == 1;

        String json = JSON.toJSONString(po);
        if (flag) {
            log.info("多礼物活动赠送榜单，按天分隔。赠送人当天只有一条记录创建成功 {}", json);
            return po;
        }
        log.warn("多礼物活动赠送榜单，按天分隔。赠送人当天只有一条记录创建失败 {}", json);
        return null;
    }

    /**
     * 插入批量数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createBatch(List<ActivityGiftsRankSendDO> list) {
        boolean flag = activityGiftsRankSendDAO.insertBatch(list) > 0;
        return flag;
    }

    /**
     * 插入或更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createOrUpdate(ActivityGiftsRankSendDO po) {
        boolean flag = activityGiftsRankSendDAO.insertOrUpdate(po) == 1;
        return flag;
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean updateById(ActivityGiftsRankSendDO po) {
        po.setCreateTime(null);
        po.setUpdateTime(System.currentTimeMillis());

        UpdateWrapper uw = new UpdateWrapper();
        uw.setPo(po);
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, po.getActivityGiftsRankSendId());
        uw.setQw(qw);
        return update(uw);
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean update(UpdateWrapper uw) {
        boolean flag = activityGiftsRankSendDAO.update(uw) > 0;
        return flag;
    }

    /**
     * 根据id查询
     */
    public ActivityGiftsRankSendDO getById(Long id) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return activityGiftsRankSendDAO.selectOne(qw);
    }

    /**
     * 判断是否存在
     *
     * @param id 主键
     * @return true:存在
     */
    public boolean exist(Long id) {
        if (id == null) {
            return false;
        }
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return activityGiftsRankSendDAO.selectCount(qw) > 0;
    }

    /**
     * 分页查询
     */
    public PageList<ActivityGiftsRankSendDO> getPageList(ActivityGiftsRankSendPageListQO qo) {
        QueryWrapper qw = buildPageListQueryWrapper(qo);
        PageList<ActivityGiftsRankSendDO> page = new PageList<>(qo.getPageIndex(), qo.getPageSize());
        qw.setPageCursor(page.getPageCursor());
        qw.setPageSize(page.getPageSize());
        page.setList(activityGiftsRankSendDAO.selectPageList(qw));
        page.setTotal(activityGiftsRankSendDAO.selectCount(qw));
        return page;
    }

    /**
     * 根据ids获得 DO Map
     *
     * @param ids id集合
     * @return Map<主键, DO对象>
     */
    public Map<Long, ActivityGiftsRankSendDO> getMapByIds(Collection ids) {
        return activityGiftsRankSendDAO.selectMapByIds(ids);
    }

    // 手动编码开始

}