package com.starbuds.server.service.activity.provider;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.pojo.activity.qo.ActivityGiftsSaveHisPageListQO;
import com.starbuds.server.common.pojo.db.ActivityGiftsSaveHisDO;
import com.starbuds.server.service.activity.dao.ActivityGiftsSaveHisDAO;
import com.starbuds.server.common.pojo.rtc.qo.RtcRoomAuctionPageListQO;

import com.starbuds.server.common.pojo.api.PageList;
import lombok.extern.slf4j.Slf4j;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.service.core.IdentityService;

import static com.starbuds.server.common.pojo.util.PageListUtil.buildPageListQueryWrapper;

import org.springframework.stereotype.Component;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.transaction.annotation.Transactional;

/**
 * 多礼物活动保存记录（保证幂等）
 */
@Component
@Slf4j
public class ActivityGiftsSaveHisProvider {

    /**
     * 主键
     */
    private String PRIMARY_KEY = "hisId";

    @Reference(version = "1.0.0")
    private IdentityService identityService;

    @Resource
    private ActivityGiftsSaveHisDAO activityGiftsSaveHisDAO;

    /**
     * 插入数据
     *
     * @param po
     * @return null:插入失败
     */
    @Transactional(rollbackFor = Exception.class)
    public ActivityGiftsSaveHisDO create(ActivityGiftsSaveHisDO po) {
        po.setHisId(identityService.getId());
        po.setCreateTime(System.currentTimeMillis());
        boolean flag = activityGiftsSaveHisDAO.insert(po) == 1;

        String json = JSON.toJSONString(po);
        if (flag) {
            log.info("多礼物活动保存记录（保证幂等）创建成功 {}", json);
            return po;
        }
        log.warn("多礼物活动保存记录（保证幂等）创建失败 {}", json);
        return null;
    }

    /**
     * 插入批量数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createBatch(List<ActivityGiftsSaveHisDO> list) {
        boolean flag = activityGiftsSaveHisDAO.insertBatch(list) > 0;
        return flag;
    }

    /**
     * 插入或更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createOrUpdate(ActivityGiftsSaveHisDO po) {
        boolean flag = activityGiftsSaveHisDAO.insertOrUpdate(po) == 1;
        return flag;
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean updateById(ActivityGiftsSaveHisDO po) {
        po.setCreateTime(null);

        UpdateWrapper uw = new UpdateWrapper();
        uw.setPo(po);
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, po.getHisId());
        uw.setQw(qw);
        return update(uw);
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean update(UpdateWrapper uw) {
        boolean flag = activityGiftsSaveHisDAO.update(uw) > 0;
        return flag;
    }

    /**
     * 根据id查询
     */
    public ActivityGiftsSaveHisDO getById(Long id) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return activityGiftsSaveHisDAO.selectOne(qw);
    }

    /**
     * 判断是否存在
     *
     * @param id 主键
     * @return true:存在
     */
    public boolean exist(Long id) {
        if (id == null) {
            return false;
        }
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return activityGiftsSaveHisDAO.selectCount(qw) > 0;
    }

    /**
     * 分页查询
     */
    public PageList<ActivityGiftsSaveHisDO> getPageList(ActivityGiftsSaveHisPageListQO qo) {
        QueryWrapper qw = buildPageListQueryWrapper(qo);
        PageList<ActivityGiftsSaveHisDO> page = new PageList<>(qo.getPageIndex(), qo.getPageSize());
        qw.setPageCursor(page.getPageCursor());
        qw.setPageSize(page.getPageSize());
        page.setList(activityGiftsSaveHisDAO.selectPageList(qw));
        page.setTotal(activityGiftsSaveHisDAO.selectCount(qw));
        return page;
    }

    /**
     * 根据ids获得 DO Map
     *
     * @param ids id集合
     * @return Map<主键, DO对象>
     */
    public Map<Long, ActivityGiftsSaveHisDO> getMapByIds(Collection ids) {
        return activityGiftsSaveHisDAO.selectMapByIds(ids);
    }

    // 手动编码开始

}