package com.starbuds.server.service.activity.provider;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.pojo.activity.qo.LotteryGameConfigItemPageListQO;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;
import com.starbuds.server.common.pojo.db.LotteryGameConfigItemDO;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.service.activity.dao.LotteryGameConfigItemDAO;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;

import static com.starbuds.server.common.pojo.util.PageListUtil.buildPageListQueryWrapper;

/**
 * 抽奖游戏配置详情
 */
@Component
@Slf4j
public class LotteryGameConfigItemProvider {

    /**
     * 主键
     */
    private String PRIMARY_KEY = "configItemId";

    @Reference(version = "1.0.0")
    private IdentityService identityService;

    @Resource
    private LotteryGameConfigItemDAO lotteryGameConfigItemDAO;

    /**
     * 插入数据
     *
     * @param po
     * @return null:插入失败
     */
    @Transactional(rollbackFor = Exception.class)
    public LotteryGameConfigItemDO create(LotteryGameConfigItemDO po) {
        po.setConfigItemId(identityService.getId());
        po.setCreateTime(System.currentTimeMillis());
        po.setUpdateTime(po.getCreateTime());
        po.setIsDeleted(BoolIntVal.False);
        boolean flag = lotteryGameConfigItemDAO.insert(po) == 1;

        String json = JSON.toJSONString(po);
        if (flag) {
            log.info("抽奖游戏配置详情创建成功 {}", json);
            return po;
        }
        log.warn("抽奖游戏配置详情创建失败 {}", json);
        return null;
    }

    /**
     * 插入批量数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createBatch(List<LotteryGameConfigItemDO> list) {
        boolean flag = lotteryGameConfigItemDAO.insertBatch(list) > 0;
        return flag;
    }

    /**
     * 插入或更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createOrUpdate(LotteryGameConfigItemDO po) {
        boolean flag = lotteryGameConfigItemDAO.insertOrUpdate(po) == 1;
        return flag;
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean updateById(LotteryGameConfigItemDO po) {
        po.setCreateTime(null);
        po.setUpdateTime(System.currentTimeMillis());

        UpdateWrapper uw = new UpdateWrapper();
        uw.setPo(po);
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, po.getConfigItemId());
        uw.setQw(qw);
        return update(uw);
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean update(UpdateWrapper uw) {
        boolean flag = lotteryGameConfigItemDAO.update(uw) > 0;
        return flag;
    }

    /**
     * 根据id查询
     */
    public LotteryGameConfigItemDO getById(Long id) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return lotteryGameConfigItemDAO.selectOne(qw);
    }

    /**
     * 判断是否存在
     *
     * @param id 主键
     * @return true:存在
     */
    public boolean exist(Long id) {
        if (id == null) {
            return false;
        }
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return lotteryGameConfigItemDAO.selectCount(qw) > 0;
    }

    /**
     * 分页查询
     */
    public PageList<LotteryGameConfigItemDO> getPageList(LotteryGameConfigItemPageListQO qo) {
        QueryWrapper qw = buildPageListQueryWrapper(qo);
        PageList<LotteryGameConfigItemDO> page = new PageList<>(qo.getPageIndex(), qo.getPageSize());
        qw.setPageCursor(page.getPageCursor());
        qw.setPageSize(page.getPageSize());
        page.setList(lotteryGameConfigItemDAO.selectPageList(qw));
        page.setTotal(lotteryGameConfigItemDAO.selectCount(qw));
        return page;
    }

    // 手动编码开始

}