package com.starbuds.server.service.activity.provider;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;
import com.starbuds.server.common.pojo.db.LotteryGameConfigDO;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.service.activity.dao.LotteryGameConfigDAO;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;

/**
 * 抽奖游戏配置
 */
@Component
@Slf4j
public class LotteryGameConfigProvider {

    /**
     * 主键
     */
    private String PRIMARY_KEY = "configId";

    @Reference(version = "1.0.0")
    private IdentityService identityService;

    @Resource
    private LotteryGameConfigDAO lotteryGameConfigDAO;

    /**
     * 插入数据
     *
     * @param po
     * @return null:插入失败
     */
    @Transactional(rollbackFor = Exception.class)
    public LotteryGameConfigDO create(LotteryGameConfigDO po) {
        po.setConfigId(identityService.getId());
        po.setCreateTime(System.currentTimeMillis());
        po.setUpdateTime(po.getCreateTime());
        po.setIsDeleted(BoolIntVal.False);
        boolean flag = lotteryGameConfigDAO.insert(po) == 1;

        String json = JSON.toJSONString(po);
        if (flag) {
            log.info("抽奖游戏配置创建成功 {}", json);
            return po;
        }
        log.warn("抽奖游戏配置创建失败 {}", json);
        return null;
    }

    /**
     * 插入批量数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createBatch(List<LotteryGameConfigDO> list) {
        boolean flag = lotteryGameConfigDAO.insertBatch(list) > 0;
        return flag;
    }

    /**
     * 插入或更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createOrUpdate(LotteryGameConfigDO po) {
        boolean flag = lotteryGameConfigDAO.insertOrUpdate(po) == 1;
        return flag;
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean updateById(LotteryGameConfigDO po) {
        po.setCreateTime(null);
        po.setUpdateTime(System.currentTimeMillis());

        UpdateWrapper uw = new UpdateWrapper();
        uw.setPo(po);
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, po.getConfigId());
        uw.setQw(qw);
        return update(uw);
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean update(UpdateWrapper uw) {
        boolean flag = lotteryGameConfigDAO.update(uw) > 0;
        return flag;
    }

    /**
     * 根据id查询
     */
    public LotteryGameConfigDO getById(Long id) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return lotteryGameConfigDAO.selectOne(qw);
    }

    /**
     * 判断是否存在
     *
     * @param id 主键
     * @return true:存在
     */
    public boolean exist(Long id) {
        if (id == null) {
            return false;
        }
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return lotteryGameConfigDAO.selectCount(qw) > 0;
    }

}