package com.starbuds.server.service.activity.provider;

import com.starbuds.server.common.constant.cache.RedisActivityKeys;
import com.starbuds.server.common.pojo.db.LotteryGameSnapshotDO;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.service.activity.dao.LotteryGameSnapshotDAO;
import com.starbuds.server.service.activity.dao.LotteryGameTimerDAO;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.math.NumberUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Collections;
import java.util.List;

import static com.starbuds.server.service.activity.util.SpringContextUtil.isDev;

/**
 * 抽奖游戏
 *
 * @author aoe
 * @date 2021/11/26
 */
@Component
@Slf4j
public class LotteryGameProvider {

    @Reference(version = "1.0.0")
    private RedisService redisService;

    @Resource
    private LotteryGameTimerDAO lotteryGameTimerDAO;

    @Resource
    private LotteryGameSnapshotDAO lotteryGameSnapshotDAO;

    /**
     * 获得奖项总数量
     *
     * @param snapId
     * @return
     */
    public int getRewardCount(long snapId) {
        String key = RedisActivityKeys.getLotteryGameRewardCount(snapId);
        if (isDev()) {
            String value = redisService.get(key);
            int stock = NumberUtils.toInt(redisService.get(key));
            log.info("获得奖项总数量 snapId={} value={} stock={}", snapId, value, stock);
        }
        return NumberUtils.toInt(redisService.get(key));
    }

    /**
     * 抽奖游戏进行中的快照集合
     * @return
     */
    /*public List<LotteryGameSnapshotDO> getInProgressingSnaps() {
        // 抽奖游戏进行中的快照Map snapId:minEndTime,maxEndTime
        String spanIdsKey = RedisActivityKeys.getLotteryGameInProgressingSnapIds();
        Map<Object, Object> snapMap = cacheService.hashGetAll(spanIdsKey);
        if (snapMap == null || snapMap.isEmpty()) {
            return Collections.emptyList();
        }
        List<LotteryGameSnapshotDO> snapList = new ArrayList<>();
        snapMap.forEach((k, v) -> {
            LotteryGameSnapshotDO snap = new LotteryGameSnapshotDO();
            snap.setSnapId(NumberUtils.toLong("" + k));
            String[] minMax = StringUtils.split((String) v, ",");
            snap.setMinEndTime(NumberUtils.toLong(minMax[0]));
            snap.setMaxEndTime(NumberUtils.toLong(minMax[1]));
            snapList.add(snap);
        });
        return snapList;
    }*/

    /**
     * 抽奖游戏进行中的快照集合
     *
     * @return
     */
    public List<LotteryGameSnapshotDO> getInProgressingSnaps() {
        List<Long> timerIds = lotteryGameTimerDAO.selectInProgressingTimerIds();
        if (CollectionUtils.isEmpty(timerIds)) {
            return Collections.emptyList();
        }
        return lotteryGameSnapshotDAO.selectInProgressingList(timerIds);
    }
}
