package com.starbuds.server.service.activity.rocket.handler;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.aliyun.openservices.ons.api.PropertyKeyConst;
import com.starbuds.server.common.constant.activity.ActivityRankTypeEnum;
import com.starbuds.server.common.constant.activity.ActivityTypeEnum;
import com.starbuds.server.common.constant.activity.RankTypeEnum;
import com.starbuds.server.common.constant.cache.RedisCacheKeys;
import com.starbuds.server.common.constant.cache.RedisRiskKeys;
import com.starbuds.server.common.constant.common.AppModuleType;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.data.DataMsgHitBussinessEnum;
import com.starbuds.server.common.constant.data.PropsTypeEnum;
import com.starbuds.server.common.constant.finance.AddItemToBagSourceEnum;
import com.starbuds.server.common.constant.finance.BagItemTypeEnum;
import com.starbuds.server.common.constant.finance.WalletTypeEnum;
import com.starbuds.server.common.constant.media.GiftTypeEnum;
import com.starbuds.server.common.constant.message.im.ImActionType;
import com.starbuds.server.common.constant.message.im.ImTargetType;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rocketmq.RocketTopic;
import com.starbuds.server.common.pojo.activity.ActivityVO;
import com.starbuds.server.common.pojo.activity.LotteryHisRewardVO;
import com.starbuds.server.common.pojo.activity.LotteryReward;
import com.starbuds.server.common.pojo.activity.bo.AnnualGalaPeakFamilyRankBO;
import com.starbuds.server.common.pojo.data.MedalVO;
import com.starbuds.server.common.pojo.data.PropsVO;
import com.starbuds.server.common.pojo.db.*;
import com.starbuds.server.common.pojo.finance.BagItem;
import com.starbuds.server.common.pojo.finance.gift.GiftReceiver;
import com.starbuds.server.common.pojo.message.im.activity.HeadlineLotteryParticipateMsg;
import com.starbuds.server.common.pojo.message.im.biz.SystemMsg;
import com.starbuds.server.common.pojo.mq.*;
import com.starbuds.server.common.pojo.rtc.RtcRoomListItem;
import com.starbuds.server.common.pojo.user.UserBasicInfo;
import com.starbuds.server.common.pojo.user.UserCardVO;
import com.starbuds.server.common.service.activity.*;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.data.DataMsgHisService;
import com.starbuds.server.common.service.data.PropsService;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.data.ReleaseConfigService;
import com.starbuds.server.common.service.finance.BagService;
import com.starbuds.server.common.service.finance.WalletService;
import com.starbuds.server.common.service.media.GiftService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.user.AnchorService;
import com.starbuds.server.service.activity.dao.*;
import com.starbuds.server.service.activity.lottery.LotteryProvider;
import com.starbuds.server.service.activity.lottery.LotteryResult;
import com.starbuds.server.service.activity.provider.GiftRankProvider;
import com.starbuds.server.service.activity.rocket.RocketConfig;
import com.starbuds.server.service.activity.rocket.RocketProducer;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.junit.jupiter.api.Assertions;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;

import static com.starbuds.server.service.activity.util.SpringContextUtil.isDev;

/**
 * 备注
 *
 * @author Lay
 * @date 2020/1/3
 */
@Component
@Slf4j
public class NormalMsgHandler {

    @Value("${spring.application.name}")
    String applicationName;

    @Reference(version = "1.0.0")
    GiftService giftService;

    @Reference(version = "1.0.0")
    AnchorService anchorService;

    @Reference(version = "1.0.0")
    RedisService redisService;

    @Reference(version = "1.0.0")
    ImService imService;

    @Reference(version = "1.0.0")
    ActivityGiftsRankService activityGiftsRankService;

    @Resource
    UserDAO userDAO;

    @Resource
    ActivityDAO activityDAO;

    @Resource
    ActivityWeekDAO activityWeekDAO;

    @Resource
    ActivityLotteryDAO activityLotteryDAO;

    @Resource
    GiftRankProvider giftRankProvider;

    @Resource
    LotteryProvider lotteryProvider;

    @Resource
    ActivityGiftRankAnchorDAO activityGiftRankAnchorDAO;

    @Resource
    ActivityGiftRankUserDAO activityGiftRankUserDAO;

    @Resource
    ActivityGiftRankAssistsDAO activityGiftRankAssistsDAO;

    @Resource
    ActivityGiftRankRoomDAO activityGiftRankRoomDAO;

    @Resource
    ActivityLotteryHisRewardsDAO activityLotteryHisRewardsDAO;

    @Reference(version = "1.0.0")
    ActivityBarrageService activityBarrageService;

    @Resource
    RocketConfig rocketConfig;

    @Reference(version = "1.0.0")
    DataMsgHisService dataMsgHisService;

    @Resource
    LotteryGameService lotteryGameService;

    @Reference(version = "1.0.0")
    BagService bagService;

    @Reference(version = "1.0.0")
    ReleaseConfigService releaseConfigService;

    @Reference(version = "1.0.0")
    IdentityService identityService;

    @Reference(version = "1.0.0")
    ActivityAnniversaryService activityAnniversaryService;

    @Reference(version = "1.0.0")
    WalletService walletService;

    @Reference(version = "1.0.0")
    PropsService propsService;

    @Resource
    ActivityRechargeForkHisDAO activityRechargeForkHisDAO;

    @Resource
    RocketProducer rocketProducer;

    /**
     * 礼物支付消息
     *
     * @param msg
     */
    @Transactional(rollbackFor = Exception.class)
    public void handleGiftPaid(QueueGiftPaidMsg msg) {
        // 公会 PK 活动
        ActivityVO activityVO = activityDAO.selectByType(ActivityTypeEnum.PK.getValue());
        if (activityVO != null) {
            // 查询礼物
            GiftDO giftDO = giftService.getGift(msg.getGiftId(), null);

            // 接收人
            UserCardVO receiver = userDAO.selectCardById(msg.getReceiverUserId());

            // 公会id
            Long agentId = anchorService.getAgentId(msg.getReceiverUserId());

            // 赠送人
            UserCardVO sender = userDAO.selectCardById(msg.getSenderUserId());

            long timestamp = msg.getTimestamp();

            // 星钻礼物
            if (WalletTypeEnum.STAR_DIAMOND == WalletTypeEnum.toEnum(msg.getGiftPriceType())) {

                // 活动期间
                if (activityVO.getActivityBeginTime() <= timestamp && activityVO.getActivityEndTime() >= timestamp) {
                    log.info("活动期间");
                    List<String> keys = new ArrayList<>();
                    List<String> members = new ArrayList<>();
                    keys.add(RedisCacheKeys.getActivityRankKey(activityVO.getActivityId(), RankTypeEnum.ANCHOR_LIST.getValue()));
                    keys.add(RedisCacheKeys.getActivityRankKey(activityVO.getActivityId(), RankTypeEnum.WEALTH_LIST.getValue()));
                    if (agentId != null) {
                        keys.add(RedisCacheKeys.getActivityRankKey(activityVO.getActivityId(), RankTypeEnum.AGENT_LIST.getValue()));
                    }
                    // 金主榜
                    keys.add(RedisCacheKeys.getAnchorActivityRankKey(activityVO.getActivityId(), RankTypeEnum.INVESTOR_LIST.getValue(), receiver.getUserId()));
                    log.info("排行榜key:{}", keys);
                    // 实际积分
                    BigDecimal actualScore = BigDecimal.ZERO;
                    WalletTypeEnum walletType = WalletTypeEnum.toEnum(giftDO.getGiftPriceType());

                    switch (walletType) {
                        // 星钻
                        case STAR_DIAMOND:
                            Long totalValue = giftDO.getGiftPrice() * msg.getQuantity();
                            // 礼物类型
                            Integer giftType = giftDO.getGiftType();

                            // 幸运礼物
                            if (giftType == GiftTypeEnum.LUCKY.getValue()) {
                                actualScore = new BigDecimal(totalValue).divide(new BigDecimal(100), 2, BigDecimal.ROUND_HALF_UP);
                            } else if (giftType == GiftTypeEnum.COMMON.getValue()) {
                                // 普通礼物
                                actualScore = new BigDecimal(totalValue).setScale(2, BigDecimal.ROUND_HALF_UP);
                            }

                            members.add(Convert.toStr(receiver.getUserId()));
                            members.add(Convert.toStr(sender.getUserId()));
                            if (agentId != null) {
                                members.add(Convert.toStr(agentId));
                            }
                            members.add(Convert.toStr(sender.getUserId()));

                            log.info("实际积分:{}", actualScore);
                            for (int i = 0; i < keys.size(); i++) {
                                redisService.zIncrementScore(keys.get(i), members.get(i), Convert.toDouble(actualScore));
                            }
                            break;
                        default:
                            break;
                    }
                }
            }
        }

        // 判断单礼物活动
        Long curTime = System.currentTimeMillis();
        List<ActivityDO> giftNumActivities = activityDAO.selectAllByType(ActivityTypeEnum.GIFT_NUM.getValue(), curTime);
        if (CollectionUtils.isNotEmpty(giftNumActivities)) {
            for (ActivityDO activity : giftNumActivities) {
                if (activity.getModuleType() != null && (activity.getModuleType() == AppModuleType.BOTH || activity.getModuleType() == AppModuleType.LIVE)) {
                    // 判断礼物命中
                    if (msg.getGiftId().equals(activity.getGiftId())) {
                        Long activityId = activity.getActivityId();
                        Long score = Convert.toLong(msg.getQuantity());
                        // 给主播增加收获个数
                        ActivityGiftRankAnchorDO activityGiftRankAnchor = new ActivityGiftRankAnchorDO();
                        activityGiftRankAnchor.setActivityId(activityId);
                        activityGiftRankAnchor.setUserId(msg.getReceiverUserId());
                        activityGiftRankAnchor.setRankScore(score);
                        activityGiftRankAnchor.setUpdateTime(curTime);
                        activityGiftRankAnchorDAO.insertOrIncrease(activityGiftRankAnchor);

                        // 给用户增加赠送个数
                        ActivityGiftRankUserDO activityGiftRankUser = new ActivityGiftRankUserDO();
                        activityGiftRankUser.setActivityId(activityId);
                        activityGiftRankUser.setUserId(msg.getSenderUserId());
                        activityGiftRankUser.setRankScore(score);
                        activityGiftRankUser.setUpdateTime(curTime);
                        activityGiftRankUserDAO.insertOrIncrease(activityGiftRankUser);

                        // 增加助攻
                        ActivityGiftRankAssistsDO activityGiftRankAssists = new ActivityGiftRankAssistsDO();
                        activityGiftRankAssists.setActivityId(activityId);
                        activityGiftRankAssists.setAnchorUserId(msg.getReceiverUserId());
                        activityGiftRankAssists.setUserId(msg.getSenderUserId());
                        activityGiftRankAssists.setRankScore(score);
                        activityGiftRankAssists.setUpdateTime(curTime);
                        activityGiftRankAssistsDAO.insertOrIncrease(activityGiftRankAssists);
                    }
                }
            }
        }

        // 判断多礼物活动
        List<ActivityDO> giftsActivities = activityDAO.selectAllByType(ActivityTypeEnum.GIFT_MANY.getValue(), curTime);
        if (CollectionUtils.isNotEmpty(giftsActivities)) {
            for (ActivityDO activity : giftNumActivities) {
                if (activity.getModuleType() != null && (activity.getModuleType() == AppModuleType.BOTH || activity.getModuleType() == AppModuleType.LIVE)) {

                    List<Long> giftIds = Arrays.stream(activity.getGiftIds().split(",")).map(Long::valueOf).collect(Collectors.toList());

                    // 属于多礼物活动
                    if (giftIds.stream().filter(Objects::nonNull).anyMatch(i -> i.equals(msg.getGiftId()))) {
                        activityGiftsRankService.dealData(msg.getOrderId(), activity.getActivityId(), msg.getSenderUserId(), msg.getReceiverUserId(), msg.getQuantity() * msg.getGiftPrice(), msg.getGiftId());
                    }
                }

            }
        }
    }


    /**
     * 处理语聊房支付消息
     *
     * @param msg
     */
    public void handleRtcGiftPaid(QueueRtcGiftPaidMsg msg, String rocketTag) {
        //检查是否重复消费
        String appName = applicationName;
        String rockGropId = rocketConfig.getProps().get(PropertyKeyConst.GROUP_ID).toString();
        Long orderId = msg.getOrderId();
        String key = rocketTag + "." + appName + "." + rockGropId + "." + orderId + "." + msg.getGiftId();
        long curTime = System.currentTimeMillis();
        if (dataMsgHisService.checkMsgIsExe(key, DataMsgHitBussinessEnum.RTCGIFTPAID.getValue())) {
            //消息已经被消费
            return;
        }

        if (isDev()) {
            log.info("语聊房支付消息 {}", JSON.toJSONString(msg, true));
        }
        // 活动礼物处理
        handleActivityGift(msg.getGiftType(), msg.getGiftId(), msg.getSenderUserId(), msg.getTimestamp(), msg.getChannelId());

        // 单礼物活动  和 周星活动 和 多礼物活动
        List<Integer> activityTypes = Arrays.asList(ActivityTypeEnum.GIFT_NUM.getValue(), ActivityTypeEnum.WEEK.getValue(), ActivityTypeEnum.GIFT_MANY.getValue());

        // 获取活动
        List<ActivityDO> weekActivities = activityDAO.selectAllByTypes(activityTypes, curTime);
        if (CollectionUtils.isNotEmpty(weekActivities)) {
            Long date = Convert.toLong(DateUtil.format(new Date(msg.getTimestamp()), "yyyyMMdd"));
            Map<Long, Integer> targetUserSeat = msg.getTargetUserSeat();
            for (ActivityDO activity : weekActivities) {
                if (activity.getModuleType() != null && (activity.getModuleType() == AppModuleType.BOTH || activity.getModuleType() == AppModuleType.RTC)) {
                    // 活动类型
                    ActivityTypeEnum activityTypeEnum = ActivityTypeEnum.toEnum(activity.getActivityType());
                    switch (activityTypeEnum) {
                        case GIFT_NUM:
                            // 判断礼物命中 单礼物活动
                            if (msg.getGiftId().equals(activity.getGiftId())) {
                                Long activityId = activity.getActivityId();
                                Long quantity = Convert.toLong(msg.getQuantity());
                                // 给房间增加赠送个数
                                ActivityGiftRankRoomDO rankRoomDO = new ActivityGiftRankRoomDO();
                                rankRoomDO.setActivityId(activityId);
                                rankRoomDO.setRoomId(msg.getRoomId());
                                rankRoomDO.setRankScore(targetUserSeat.size() * quantity);
                                rankRoomDO.setUpdateTime(curTime);
                                activityGiftRankRoomDAO.insertOrUpdate(rankRoomDO);

                                // 给用户增加赠送个数
                                ActivityGiftRankUserDO activityGiftRankUser = new ActivityGiftRankUserDO();
                                activityGiftRankUser.setActivityId(activityId);
                                activityGiftRankUser.setUserId(msg.getSenderUserId());
                                activityGiftRankUser.setRankScore(targetUserSeat.size() * quantity);
                                activityGiftRankUser.setUpdateTime(curTime);
                                activityGiftRankUserDAO.insertOrIncrease(activityGiftRankUser);
                            }
                            break;
                        case WEEK:
                            // 获取周星活动
                            List<ActivityWeekDO> activityWeeks = activityWeekDAO.selectByActivityIdAndDate(activity.getActivityId(), date);
                            if (CollectionUtils.isNotEmpty(activityWeeks)) {
                                for (ActivityWeekDO weekDO : activityWeeks) {
                                    if (StringUtils.isNotEmpty(weekDO.getGifts()) && weekDO.getGifts().contains(Convert.toStr(msg.getGiftId()))) {
                                        giftRankProvider.addRichStarRankScore(msg.getGiftId(), msg.getSenderUserId(), Convert.toLong(msg.getQuantity()) * targetUserSeat.size());
                                        if (CollectionUtil.isNotEmpty(targetUserSeat)) {
                                            for (Long userId : targetUserSeat.keySet()) {
                                                giftRankProvider.addWeekStarRankScore(msg.getGiftId(), userId, Convert.toLong(msg.getQuantity()));
                                            }
                                        }
                                    }
                                }
                            }
                            break;
                        case GIFT_MANY:
                            // 判断多礼物活动
                            List<Long> giftIds = Arrays.stream(activity.getGiftIds().split(",")).map(Long::valueOf).collect(Collectors.toList());

                            // 属于多礼物活动
                            if (giftIds.stream().filter(Objects::nonNull).anyMatch(i -> i.equals(msg.getGiftId()))) {
                                if (CollectionUtil.isNotEmpty(targetUserSeat)) {
                                    for (Long userId : targetUserSeat.keySet()) {
                                        activityGiftsRankService.dealData(msg.getOrderId(), activity.getActivityId(), msg.getSenderUserId(), userId, msg.getQuantity() * msg.getGiftPrice(), msg.getGiftId());
                                    }
                                }
                            }
                            break;
                        default:
                            break;
                    }
                }
            }
        }
        //更新消息执行状态
        dataMsgHisService.updateByKey(1, curTime, key);

    }

    /**
     * 处理语聊房麦下送礼物支付消息
     *
     * @param msg
     */
    public void handleRtcGiftUnderMaiPaid(QueueRtcGiftUnderMaiPaidMsg msg) {
        QueueRtcGiftPaidMsg rtcGiftPaidMsg = new QueueRtcGiftPaidMsg();
        BeanUtils.copyProperties(msg, rtcGiftPaidMsg);

        rtcGiftPaidMsg.setTargetUserSeat(msg.getReceiverUsers().stream().map(GiftReceiver::getUserId).collect(Collectors.toMap(Function.identity(), v -> -1)));
        rtcGiftPaidMsg.setAppVer(msg.getAppVer());
        rtcGiftPaidMsg.setAllSeat(BoolIntVal.False);
        handleRtcGiftPaid(rtcGiftPaidMsg, RocketTag.RTC_UNDER_MAI_GIFT_PAID);
    }

    /**
     * 私聊送礼物支付消息
     *
     * @param msg
     */
    public void handlePrivateChatGiftPaid(QueueBaseGiftPaidMsg msg) {
        //检查是否重复消费
        String appName = applicationName;
        String rockGropId = rocketConfig.getProps().get(PropertyKeyConst.GROUP_ID).toString();
        Long orderId = msg.getOrderId();
        String key = RocketTag.PRIVATE_CHAT_GIFT_PAID + "." + appName + "." + rockGropId + "." + orderId + "." + msg.getGiftId();
        long curTime = System.currentTimeMillis();
        if (dataMsgHisService.checkMsgIsExe(key, DataMsgHitBussinessEnum.RTCGIFTPAID.getValue())) {
            //消息已经被消费
            return;
        }

        // 活动礼物处理
        handleActivityGift(msg.getGiftType(), msg.getGiftId(), msg.getSenderUserId(), msg.getTimestamp(), msg.getChannelId());

        // 单礼物活动  和 周星活动 和 多礼物活动
        List<Integer> activityTypes = Arrays.asList(ActivityTypeEnum.GIFT_NUM.getValue(), ActivityTypeEnum.WEEK.getValue(), ActivityTypeEnum.GIFT_MANY.getValue());

        // 获取活动
        List<ActivityDO> weekActivities = activityDAO.selectAllByTypes(activityTypes, curTime);

        if (CollectionUtils.isNotEmpty(weekActivities)) {
            for (ActivityDO activity : weekActivities) {
                if (activity.getModuleType() != null && (activity.getModuleType() == AppModuleType.BOTH || activity.getModuleType() == AppModuleType.RTC)) {
                    // 活动类型
                    ActivityTypeEnum activityTypeEnum = ActivityTypeEnum.toEnum(activity.getActivityType());
                    switch (activityTypeEnum) {
                        case GIFT_NUM:
                            // 判断礼物命中 单礼物活动
                            if (msg.getGiftId().equals(activity.getGiftId())) {
                                Long activityId = activity.getActivityId();
                                Long quantity = Convert.toLong(msg.getQuantity());

                                // 给用户增加赠送个数
                                ActivityGiftRankUserDO activityGiftRankUser = new ActivityGiftRankUserDO();
                                activityGiftRankUser.setActivityId(activityId);
                                activityGiftRankUser.setUserId(msg.getSenderUserId());
                                activityGiftRankUser.setRankScore(quantity);
                                activityGiftRankUser.setUpdateTime(curTime);
                                activityGiftRankUserDAO.insertOrIncrease(activityGiftRankUser);
                            }
                            break;
                        case WEEK:
                            // 获取周星活动
                            List<ActivityWeekDO> activityWeeks = activityWeekDAO.selectByActivityIdAndDate(activity.getActivityId(), Convert.toLong(DateUtil.format(new Date(msg.getTimestamp()), "yyyyMMdd")));
                            if (CollectionUtils.isNotEmpty(activityWeeks)) {
                                for (ActivityWeekDO weekDO : activityWeeks) {
                                    if (StringUtils.isNotEmpty(weekDO.getGifts()) && weekDO.getGifts().contains(Convert.toStr(msg.getGiftId()))) {
                                        giftRankProvider.addRichStarRankScore(msg.getGiftId(), msg.getSenderUserId(), Convert.toLong(msg.getQuantity()));
                                        giftRankProvider.addWeekStarRankScore(msg.getGiftId(), msg.getReceiveUserId(), Convert.toLong(msg.getQuantity()));
                                    }
                                }
                            }
                            break;
                        case GIFT_MANY:
                            // 判断多礼物活动
                            List<Long> giftIds = Arrays.stream(activity.getGiftIds().split(",")).map(Long::valueOf).collect(Collectors.toList());

                            // 属于多礼物活动
                            if (giftIds.stream().filter(Objects::nonNull).anyMatch(i -> i.equals(msg.getGiftId()))) {
                                activityGiftsRankService.dealData(msg.getOrderId(), activity.getActivityId(), msg.getSenderUserId(), msg.getReceiveUserId(), msg.getQuantity() * msg.getGiftPrice(), msg.getGiftId());
                            }
                            break;
                        default:
                            break;
                    }
                }
            }
        }

        //更新消息执行状态
        dataMsgHisService.updateByKey(1, curTime, key);
    }

    /**
     * 语音速配礼物支付消息
     *
     * @param msg
     */
    public void handleVoiceGiftPaid(QueueVoiceGiftPaidMsg msg) {

        //检查是否重复消费
        String appName = applicationName;
        String rockGropId = rocketConfig.getProps().get(PropertyKeyConst.GROUP_ID).toString();
        Long orderId = msg.getOrderId();
        String key = RocketTag.VOICE_GIFT_PAID + "." + appName + "." + rockGropId + "." + orderId + "." + msg.getGiftId();
        long curTime = System.currentTimeMillis();
        if (dataMsgHisService.checkMsgIsExe(key, DataMsgHitBussinessEnum.RTCGIFTPAID.getValue())) {
            //消息已经被消费
            return;
        }

        // 活动礼物处理
        handleActivityGift(msg.getGiftType(), msg.getGiftId(), msg.getSenderUserId(), msg.getTimestamp(), msg.getChannelId());

        // 单礼物活动  和 周星活动 和 多礼物活动
        List<Integer> activityTypes = Arrays.asList(ActivityTypeEnum.GIFT_NUM.getValue(), ActivityTypeEnum.WEEK.getValue(), ActivityTypeEnum.GIFT_MANY.getValue());

        // 获取活动
        List<ActivityDO> weekActivities = activityDAO.selectAllByTypes(activityTypes, curTime);

        if (CollectionUtils.isNotEmpty(weekActivities)) {
            Long date = Convert.toLong(DateUtil.format(new Date(msg.getTimestamp()), "yyyyMMdd"));
            for (ActivityDO activity : weekActivities) {
                if (activity.getModuleType() != null && (activity.getModuleType() == AppModuleType.BOTH || activity.getModuleType() == AppModuleType.RTC)) {
                    // 活动类型
                    ActivityTypeEnum activityTypeEnum = ActivityTypeEnum.toEnum(activity.getActivityType());
                    switch (activityTypeEnum) {
                        case GIFT_NUM:
                            // 判断礼物命中 单礼物活动
                            if (msg.getGiftId().equals(activity.getGiftId())) {
                                Long activityId = activity.getActivityId();
                                Long quantity = Convert.toLong(msg.getQuantity());

                                // 给用户增加赠送个数
                                ActivityGiftRankUserDO activityGiftRankUser = new ActivityGiftRankUserDO();
                                activityGiftRankUser.setActivityId(activityId);
                                activityGiftRankUser.setUserId(msg.getSenderUserId());
                                activityGiftRankUser.setRankScore(quantity);
                                activityGiftRankUser.setUpdateTime(curTime);
                                activityGiftRankUserDAO.insertOrIncrease(activityGiftRankUser);
                            }
                            break;
                        case WEEK:
                            // 获取周星活动
                            List<ActivityWeekDO> activityWeeks = activityWeekDAO.selectByActivityIdAndDate(activity.getActivityId(), date);
                            if (CollectionUtils.isNotEmpty(activityWeeks)) {
                                for (ActivityWeekDO weekDO : activityWeeks) {
                                    if (StringUtils.isNotEmpty(weekDO.getGifts()) && weekDO.getGifts().contains(Convert.toStr(msg.getGiftId()))) {
                                        giftRankProvider.addRichStarRankScore(msg.getGiftId(), msg.getSenderUserId(), Convert.toLong(msg.getQuantity()));
                                        giftRankProvider.addWeekStarRankScore(msg.getGiftId(), msg.getReceiveUserId(), Convert.toLong(msg.getQuantity()));
                                    }
                                }
                            }
                            break;
                        case GIFT_MANY:
                            // 判断多礼物活动
                            List<Long> giftIds = Arrays.stream(activity.getGiftIds().split(",")).map(Long::valueOf).collect(Collectors.toList());

                            // 属于多礼物活动
                            if (giftIds.stream().filter(Objects::nonNull).anyMatch(i -> i.equals(msg.getGiftId()))) {
                                activityGiftsRankService.dealData(msg.getOrderId(), activity.getActivityId(), msg.getSenderUserId(), msg.getReceiveUserId(), msg.getQuantity() * msg.getGiftPrice(), msg.getGiftId());
                            }
                            break;
                        default:
                            break;
                    }
                }
            }
        }
        //更新消息执行状态
        dataMsgHisService.updateByKey(1, curTime, key);
    }

    /**
     * 处理活动礼物
     *
     * @param giftId
     */
    private void handleActivityGift(Integer giftType, Long giftId, Long userId, Long timestamp, Long channelId) {
        try {
            GiftDO gift = giftService.getGift(giftId, null);
            if (gift == null || StringUtils.isEmpty(gift.getFreeLotteryId())) {
                return;
            }

            String lotteryId = gift.getFreeLotteryId();
            ActivityLotteryDO activityLottery = activityLotteryDAO.selectById(lotteryId);
            if (activityLottery == null) {
                return;
            }

            ActivityDO activity = activityDAO.selectById(activityLottery.getActivityId());
            if (activity == null) {
                return;
            }
            // 活动时间范围
            if (activity.getActivityBeginTime() != null && timestamp < activity.getActivityBeginTime()) {
                // 活动未开始
                return;
            }
            if (activity.getActivityEndTime() != null && timestamp > activity.getActivityEndTime()) {
                // 活动已结束
                return;
            }

            LotteryResult lotteryResult = lotteryProvider.lottery(lotteryId, userId, 1, channelId, true, userId);
            if (lotteryResult != null) {
                List<LotteryReward> winRewards = lotteryResult.getWinRewards();
                if (CollectionUtil.isNotEmpty(winRewards)) {
                    List<LotteryHisRewardVO> list = new ArrayList<>();

                    for (LotteryReward lotteryReward : winRewards) {
                        LotteryHisRewardVO rewardVO = new LotteryHisRewardVO();
                        rewardVO.setRewardId(lotteryReward.getRewardId());
                        rewardVO.setRewardType(lotteryReward.getRewardType());
                        rewardVO.setQuantity(lotteryReward.getQuantity());
                        list.add(rewardVO);
                    }

                    // 异步结算
                    lotteryProvider.settleAsync(lotteryResult.getLotteryName(), lotteryResult.getHisId(), userId, list, channelId, 0, null, null);
                }
            }
        } catch (Exception e) {
            log.error("活动礼物处理失败", e);
        }
    }

    /**
     * 头条抽奖活动参与成功
     *
     * @param msg
     */
    public void handleHeadlineLotteryActivityParticipate(QueueHeadlineLotteryActivityParticipateMsg msg) {
        // 发送消息
        HeadlineLotteryParticipateMsg imMsg = new HeadlineLotteryParticipateMsg();
        imMsg.setContent(msg.getContent());
        imService.sendRtcRoomMsg(ImTargetType.GROUP, msg.getUserId(), null, 0L, imMsg);
    }

    /**
     * 处理全局广播消息
     *
     * @param msg
     */
    public void handleActivityGiftsPaid(QueueActivityGiftsPaidMsg msg) {
        activityBarrageService.sendActivityRankBroadcast(msg.getSenderUserId(), msg.getReceiverUserId(), msg.getRelatedTotalValue(), msg.getActivityId());
    }

    /**
     * 更新统计数据
     *
     * @param msg
     */
    public void handleLotteryGameStatistic(LotteryGameStatisticMsg msg) {
        lotteryGameService.buildStatistic(msg.getSnapId(), msg.getRewardNum(), msg.getGiveRewardTotal());
    }


    @Transactional(rollbackFor = Exception.class)
    public void handleActivityLogin(ActivityLoginUserMsg msg) {
        log.info("handleActivityLogin:进入" + msg.getUserId());
        //从缓存获取配置
        String config = releaseConfigService.getCachReleaseConfig("loginActivity2022");
        if (StringUtils.isNotEmpty(config)) {
            JSONObject obj = JSON.parseObject(config);
            long loginStart = obj.getLong("loginStart");
            long timeStart = obj.getLong("startTime");
            long timeEnd = obj.getLong("endTime");
            String activityLotteryId = obj.getString("activityLotteryId");

            long timeNow = System.currentTimeMillis();
            //登录周年庆送能够抽奖的礼物
            //两个月前到现在只登录过一次20211025开始只登录一次  1633667100000
            long loginNum = userDAO.selectCountBycreatTime(msg.getUserId(), loginStart);
//            long beforLoginNum= userDAO.selectCountByLesCreatTime(msg.getUserId(),loginStart);
            UserBasicInfo userBasicInfo = userDAO.getUser(msg.getUserId());

            //活动条件：注册时间在两个月之前 并且在两个月内只有一次登录记录
            log.info("handleActivityLogin:活动条件判断" + "userID:" + msg.getUserId() + ",ctime:" + userBasicInfo.getUserCreateTime() + ",lstart:" + loginStart + ",logNum:" + loginNum);
            if (!(userBasicInfo.getUserCreateTime() <= loginStart && loginNum == 1)) {
                return;
            }

            //登录时间在活动时间范围就赠送可以抽奖的礼物
            //开始时间2022-01-01:00  1640966400000
            //结束时间2022-01-21:24  1642780800000
            log.info("handleActivityLogin:活动送礼判断" + "activityLotteryId:" + activityLotteryId + ",timeNow:" + timeNow + ",timeStart:" + timeStart + ",timeEnd:" + timeEnd);
            if (StringUtils.isNotEmpty(activityLotteryId) && (timeNow >= timeStart && timeNow <= timeEnd)) {
                log.info("handleActivityLogin:送礼" + msg.getUserId());
                //送礼
                ActivityLotteryDO activityDO = activityLotteryDAO.selectById(activityLotteryId);
                int hasPropsNum = userDAO.getUserBagItemHas(msg.getUserId(), activityDO.getLotteryPropId(), BagItemTypeEnum.PROPS.getValue());
                //赠送保护每个用户礼物只能获取一次
                log.info("handleActivityLogin:送礼" + msg.getUserId() + "hasPropsNum:" + hasPropsNum);
                if (hasPropsNum <= 0) {
                    bagService.addItem(msg.getUserId(), activityDO.getLotteryPropId(), BagItemTypeEnum.PROPS.getValue(), 1, timeEnd, "周年庆老粉回归赠送", identityService.getId());
                }
            } else {
                log.info("handleActivityLogin:不满足" + "activityLotteryId:" + activityLotteryId + ",timeNow:" + timeNow + ",timeStart:" + timeStart + ",timeEnd:" + timeEnd);
                //不满足活动要求
                return;
            }

            //创建统计记录
            int date = com.starbuds.server.common.utils.DateUtil.getIntDate();
            if (userDAO.updateUserStatActivityLogin(date, 1, null, null, timeNow) <= 0) {
                userDAO.insertUserStatActivityLogin(identityService.getId(), 1, date, timeNow, timeNow);
            }

        } else {
            log.error("handleActivityLogin:活动未配置");
        }

    }

    @Transactional(rollbackFor = Exception.class)
    public void handleUserRechargeFinish(UserRechargeFinishMsg msg) {

        Long userId = msg.getUserId();
        Long orderPayId = msg.getOrderPayId();
        BigDecimal amount = msg.getAmount();


        if (amount == null) {
            log.info("handleUserRechargeFinish:进入错误" + "userId:" + userId + ",orderPayId:" + orderPayId + ",amount" + amount);
            return;
        }
        log.info("handleUserRechargeFinish:进入" + "userId:" + userId + ",orderPayId:" + orderPayId + ",amount" + amount.toPlainString());

        String config = releaseConfigService.getReleaseConfig("rechargeFinishSendItem");
        if (StringUtils.isEmpty(config)) {
            log.error("handleUserRechargeFinish:活动未配置赠送礼物");
            return;
        }


        JSONObject obj = JSON.parseObject(config);
        long propId = obj.getLong("propId");
        int toAmount = obj.getInteger("amount");
        long timeStart = obj.getLong("startTime");
        long timeEnd = obj.getLong("endTime");
        long timeNow = System.currentTimeMillis();
        String userWhiteList = obj.getString("userWhiteList");

        //判断是否是白名单，白名单不判断活动时间
        if (!getUserRechargeFinishWhiteList(userId, userWhiteList)) {
            //活动是否开始
            if (timeStart > timeNow || timeNow > timeEnd) {
                log.info("handleUserRechargeFinish:活动未开始或者结束:timeStart" + timeStart + "timeEnd:" + timeEnd + "timeNow" + timeNow);
                return;
            }
        }


        //获取记录
        ActivityRechargeForkHisDO mod = activityRechargeForkHisDAO.getActivityRechargeForkHis(userId);

        //现在充值的总金额
        BigDecimal total;
        //已经获取的餐叉数
        int fork;
        if (mod == null || mod.getHisId() <= 0) {
            total = amount;
            fork = 0;
        } else {
            total = mod.getAmount().add(amount);
            fork = mod.getPropsNum();
        }

        //现在应该获取的总餐叉数
        int num = total.divide(BigDecimal.valueOf(toAmount), RoundingMode.DOWN).intValue();
        //还应该获取的餐叉数量
        int propsNum = num - fork;

        log.info("handleUserRechargeFinish:送" + "num:" + num + ",fork:" + fork + ",propsNum:" + propsNum + ",toAmount" + toAmount + ",total" + total.toPlainString());
        //送餐叉
        if (propsNum > 0) {
            //添加记录
            if (mod == null || mod.getHisId() <= 0) {
                activityRechargeForkHisDAO.insert(identityService.getId(), userId, total, propsNum, timeNow, timeNow);
            } else {
                activityRechargeForkHisDAO.updateAmountAndPropsNum(userId, propsNum, total, System.currentTimeMillis());
            }

//            bagService.addItem(msg.getUserId(), propId, BagItemTypeEnum.PROPS.getValue(), 1, -1L, "充值累积1000赠送", orderPayId);

            QueueAddItemToBagMsg senmsg = getQueueAddItemToBagMsg(propId, BagItemTypeEnum.PROPS, propsNum, -1L, userId);
            rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.ADD_ITEM_TO_BAG, senmsg.serialize());
        } else {
            //不满足充值金额的时候累加金额
            if (mod == null || mod.getHisId() <= 0) {
                activityRechargeForkHisDAO.insert(identityService.getId(), userId, total, propsNum, timeNow, timeNow);
            } else {
                activityRechargeForkHisDAO.updateAmount(userId, amount, System.currentTimeMillis());
            }
        }

    }

    /**
     * 送礼快照消息
     *
     * @param msg
     */
    @Transactional(rollbackFor = Exception.class)
    public void handleSendGiftSnapshot(QueueSendGiftSnapShotMsg msg) {

        log.info("送礼快照消费: {}", JSON.toJSONString(msg));
        //检查是否重复消费
        String appName = applicationName;
        String rockGropId = rocketConfig.getProps().get(PropertyKeyConst.GROUP_ID).toString();
        Long orderId = msg.getOrderId();
        String key = RocketTag.SEND_GIFT_SNAPSHOT + "." + appName + "." + rockGropId + "." + orderId + "." + msg.getGiftId() + "." + msg.getActivityLotteryHisId();
        long curTime = System.currentTimeMillis();
        if (dataMsgHisService.checkMsgIsExe(key, DataMsgHitBussinessEnum.SENDGIFTSNAP.getValue())) {
            //消息已经被消费
            return;
        }
        log.info("送礼快照消费2: {}", JSON.toJSONString(msg));

        // 周年庆积分添加
        if (!msg.getIsFromBag().equals(BoolIntVal.True)) {

            // 抽奖礼物需要知道抽出来的礼物价值
            GiftTypeEnum giftType = GiftTypeEnum.toEnum(msg.getGiftType());

            if (GiftTypeEnum.LOTTERY.equals(giftType)) {
                //  抽奖礼物从tb_send_gift_snapshot查出activityLotteryHisId，
                //  结合tb_activity_lottery_his_rewards查出抽奖结果，
                //  结合tb_user_wallet_his查出收礼人
                if (StringUtils.isNotBlank(msg.getActivityLotteryHisId())) {

                    List<Long> activityLotteryHisIds = Arrays.stream(msg.getActivityLotteryHisId().split(",")).map(Long::valueOf).collect(Collectors.toList());
                    List<ActivityLotteryHisRewardsDO> rewardList = activityLotteryHisRewardsDAO.selectHisRewardsMapByHisIds(activityLotteryHisIds);
                    List<UserWalletHisDO> list = walletService.getUserWalletHisByOrderIds(activityLotteryHisIds, null, WalletTypeEnum.STAR_DUST.getValue());

                    rewardList.forEach(val -> {
                        Long receiveUserId = list.stream().filter(Objects::nonNull).filter(e -> val.getHisId().equals(e.getOrderId())).findFirst().orElseGet(UserWalletHisDO::new).getUserId();
                        activityAnniversaryService.addScore(msg.getOrderId(), val.getRewardId(), msg.getSenderUserId(), receiveUserId, msg.getRoomId(), msg.getGiftPrice() * val.getQuantity(), val.getRewardValue(), String.format("%s送给%s的%d个%s开出%s礼物", msg.getSenderUserName(), receiveUserId, val.getQuantity(), msg.getGiftName(), val.getRewardId()));
                    });
                }
            } else {
                log.info("送礼快照消费3: {}", JSON.toJSONString(msg));

                Set<Long> receiveUserIds = Arrays.stream(msg.getReceiverUserIds().split(",")).map(Long::valueOf).collect(Collectors.toSet());
                log.info("送礼快照消费4: {}", JSON.toJSONString(msg));
                receiveUserIds.forEach(val -> {
                    log.info("送礼快照消费5: {}, val: {}", JSON.toJSONString(msg), val);
                    activityAnniversaryService.addScore(msg.getOrderId(), msg.getGiftId(), msg.getSenderUserId(), val, msg.getRoomId(), msg.getGiftPrice() * msg.getQuantity(), msg.getGiftPrice() * msg.getQuantity(), String.format("%s送给%s的%d个%s礼物", msg.getSenderUserName(), val, msg.getQuantity(), msg.getGiftName()));
                });
            }
        }

        //更新消息执行状态
        dataMsgHisService.updateByKey(1, curTime, key);
    }

    /**
     * 用户退出家族消息处理
     *
     * @param msg
     */
    @Transactional(rollbackFor = Exception.class)
    public void handleFamilyUserExit(QueueUserExitFamilyMsg msg) {
        log.info("收到用户退出家族消息: {}", JSON.toJSONString(msg));

        // 查询当前巅峰家族活动
        List<Integer> activityTypes = new ArrayList<>();
        activityTypes.add(ActivityTypeEnum.PEAKFAMILY.getValue());
        Long now = System.currentTimeMillis();
        List<ActivityDO> activityList = activityDAO.getActivityListByTypes(activityTypes, now);
        Long activityId = activityList.stream().filter(Objects::nonNull).filter(o -> o.getActivityType().equals(ActivityTypeEnum.PEAKFAMILY.getValue())).findFirst().orElseGet(ActivityDO::new).getActivityId();
        if (activityId == null) {
            return;
        }

        // 清除巅峰家族的积分
        log.info("用户退出家族，清除个人积分: activityId: {}, familyId: {}, userId: {}", activityId, msg.getFamilyId(), msg.getUserId());
        activityAnniversaryService.clearPeakFamilyUserScore(activityId, msg.getFamilyId(), msg.getUserId());

        // 扣减用户已经发放的奖励
        reduceUserRewardFormPeakFamily(msg.getUserId(), msg.getFamilyId(), activityId);
    }

    /**
     * 扣减用户已经发放的奖励
     *
     * @param userId
     * @param familyId
     * @param activityId
     */
    public void reduceUserRewardFormPeakFamily(Long userId, Long familyId, Long activityId) {
        // 注意，此处用户第二次退出家族时，不会再处理
        String appName = applicationName;
        String rockGropId = rocketConfig.getProps().get(PropertyKeyConst.GROUP_ID).toString();
        String key = RocketTag.FAMILY_USER_EXIT + "." + appName + "." + rockGropId + "." + familyId + "." + userId;
        long curTime = System.currentTimeMillis();
        if (dataMsgHisService.checkMsgIsExe(key, DataMsgHitBussinessEnum.PEAK_FAMILY_USER_EXIT.getValue())) {
            //消息已经被消费
            log.info("用户已经退出过一次家族，不再走扣除奖励逻辑 key: {}", key);
            return;
        }

        // 查询用户巅峰家族奖励
        Map<Long, ActivityAnnualAwardConfigDO> userPeakFamilyReward = getUserPeakFamilyReward(activityId, userId, familyId);

        log.info("开始查询用户发放的奖励 userPeakFamilyReward: {}", JSON.toJSONString(userPeakFamilyReward));

        if (userPeakFamilyReward == null) {
            return;
        }

        ActivityAnnualAwardConfigDO userReward = userPeakFamilyReward.get(userId);
        List<ActivityAnnualAwardItemDO> itemDoList = userReward.getItemDoList();
        if (CollectionUtil.isEmpty(itemDoList)) {
            return;
        }

        log.info("用户退出家族扣除奖励中: userPeakFamilyReward: {}", JSON.toJSONString(userPeakFamilyReward));
        // 获取勋章卡使用后的勋章id
        final Long[] medalIds = {null};
        final String[] medalNames = {null};
        final Integer[] quantities = {null};
        itemDoList.forEach(val -> {
            if (val.getAwardType() != 1) {
                return;
            }

            PropsVO props = propsService.getProps(Long.valueOf(val.getAwardValue()));
            if (!props.getPropsType().equals(PropsTypeEnum.MEDAL_CARD.getValue())) {
                return;
            }

            if (props.getPropsExtString() != null) {
                medalIds[0] = Long.valueOf(props.getPropsExtString());
                medalNames[0] = props.getPropsName();
                quantities[0] = val.getNum();
            }

        });

        Long medalId = medalIds[0];
        String medalName = medalNames[0];
        Integer quantity = quantities[0];

        log.info("用户退出家族扣除奖励中: medalId: {}, medalName: {}, quantity: {}", medalId, medalName, quantity);

        // 查询背包中是否还有此道具
        List<BagItem<PropsVO>> propsList = bagService.getPropsList(userId, PropsTypeEnum.MEDAL_CARD.getValue());

        final Boolean[] isContainArr = {false};
        final Long[] propsIds = {null};
        final String[] propsNames = {null};
        itemDoList.forEach(val -> {
            // 不是道具就不用继续了
            if (val.getAwardType().equals(1)) {
                return;
            }

            Optional<BagItem<PropsVO>> any = propsList.stream().filter(Objects::nonNull).filter(o -> o.getItemId().equals(Long.valueOf(val.getAwardValue()))).findAny();

            isContainArr[0] = any.isPresent();
            propsIds[0] = any.isPresent() ? any.get().getItem().getPropsId() : null;
            propsNames[0] = any.isPresent() ? any.get().getItem().getPropsName() : null;
        });
        Long propsId = propsIds[0];
        String propsName = propsNames[0];
        boolean isContain = isContainArr[0];

        log.info("用户退出家族扣除奖励,勋章卡是否已被使用: {}, propsId: {}, propsName: {}", isContain, propsId, propsName);


        QueueReduceItemToBagMsg bagMsg = new QueueReduceItemToBagMsg();
        bagMsg.setUserId(userId);
        bagMsg.setOrderId(identityService.getId());
        bagMsg.setQuantity(quantity);
        bagMsg.setRemark("退出家族，扣除巅峰家族奖励");
        bagMsg.setSource(AddItemToBagSourceEnum.PEAKFAMILY);
        if (isContain) {
            bagMsg.setItemType(BagItemTypeEnum.PROPS);
            bagMsg.setItemId(propsId);
            bagMsg.setItemName(propsName);
        } else {
            bagMsg.setItemType(BagItemTypeEnum.MEDAL);
            bagMsg.setItemId(medalId);
            bagMsg.setItemName(medalName);
        }

        log.info("用户退出家族，扣除巅峰家族奖励: bagMsg: {}", JSON.toJSONString(bagMsg));
        rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.REDUCE_ITEM_TO_BAG, bagMsg.serialize());

        //更新消息执行状态
        dataMsgHisService.updateByKey(1, curTime, key);
    }

    /**
     * 获取用户巅峰家族奖励
     *
     * @param activityId
     * @param userId
     * @param familyId
     * @return
     */
    public Map<Long, ActivityAnnualAwardConfigDO> getUserPeakFamilyReward(Long activityId, Long userId, Long familyId) {

        List<AnnualGalaPeakFamilyRankBO> rankList = activityAnniversaryService.getPeakFamilyRankList(activityId);
        if (rankList == null) {
            log.info("巅峰家族发放奖励：rankList为空");
            return null;
        }

        List<AnnualGalaPeakFamilyRankBO> rankSortedList = rankList.stream().sorted(Comparator.comparing(AnnualGalaPeakFamilyRankBO::getScore).reversed()).collect(Collectors.toList());
        List<AnnualGalaPeakFamilyRankBO> rank5List = rankSortedList.subList(0, Math.min(rankList.size(), 5));

        List<ActivityAnnualAwardGiveHisDO> userRecordList = activityAnniversaryService.getUserRecordList(userId, activityId);

        for (int i = 1; i < rank5List.size() + 1; i++) {

            AnnualGalaPeakFamilyRankBO val = rank5List.get(i - 1);

            if (!val.getFamilyId().equals(familyId)) {
                continue;
            }

            // 获取当前排名的奖励信息
            Map<Integer, ActivityAnnualAwardConfigDO> awardConfigDOMap = activityAnniversaryService.getRewardConfig(activityId, ActivityRankTypeEnum.TOTAL.getValue());

            if (CollectionUtil.isNotEmpty(userRecordList)) {

                Map<Long, ActivityAnnualAwardConfigDO> res = new HashMap<>();
                res.put(userId, awardConfigDOMap.get(i));
                return res;
            }
        }

        return null;
    }


    private QueueAddItemToBagMsg getQueueAddItemToBagMsg(Long itemId, BagItemTypeEnum itemType,
                                                         Integer quantity, Long expireTime, Long userId) {
        QueueAddItemToBagMsg msg = new QueueAddItemToBagMsg();
        msg.setOrderId(System.currentTimeMillis());
        msg.setSource(AddItemToBagSourceEnum.DAILY_RECHARGE_TOTAL_TO_1000);
        msg.setItemId(itemId);
        msg.setItemType(itemType);
        msg.setQuantity(quantity);
        msg.setExpireTime(expireTime);
        msg.setUserId(userId);
        msg.setItemName("餐叉");
        msg.setRemark("累计充值1000奖励餐叉*1");
        return msg;
    }

    //白名单
    private boolean getUserRechargeFinishWhiteList(long userId, String list) {

        if (StringUtils.isEmpty(list)) {
            return false;
        }

        String[] array = list.split(",");
        String user = String.valueOf(userId);
        for (String u : array) {
            if (user.equals(u)) {
                return true;
            }
        }
        return false;
    }


}
