package com.starbuds.server.service.activity.rocket.listener;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.aliyun.openservices.ons.api.Action;
import com.aliyun.openservices.ons.api.ConsumeContext;
import com.aliyun.openservices.ons.api.Message;
import com.aliyun.openservices.ons.api.MessageListener;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.pojo.mq.*;
import com.starbuds.server.common.pojo.mq.lotterygame.QueueLotteryGameSendRewardMsg;
import com.starbuds.server.service.activity.lotterygame.SendRewardServiceImpl;
import com.starbuds.server.service.activity.rocket.handler.NormalMsgHandler;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

/**
 * 普通队列消息监听
 *
 * @author Lay
 * @date 2020/1/3
 */
@Component
@Slf4j
public class NormalMsgListener implements MessageListener {

    @Resource
    NormalMsgHandler normalMsgHandler;

    @Resource
    private SendRewardServiceImpl sendRewardService;

    @Override
    public Action consume(Message message, ConsumeContext consumeContext) {
        log.info("收到普通队列消息 {} {}", message.getTopic(), message.getTag());
        try {
            switch (message.getTag()) {
                case RocketTag.GIFT_PAID:
                    // 礼物支付消息
                    normalMsgHandler.handleGiftPaid(JSON.parseObject(message.getBody(), QueueGiftPaidMsg.class));
                    break;
                case RocketTag.RTC_GIFT_PAID:
                    // 语聊房支付消息
                    normalMsgHandler.handleRtcGiftPaid(JSON.parseObject(message.getBody(), QueueRtcGiftPaidMsg.class), RocketTag.RTC_GIFT_PAID);
                    break;
                case RocketTag.RTC_UNDER_MAI_GIFT_PAID:
                    // 语聊房麦下送礼物支付消息
                    normalMsgHandler.handleRtcGiftUnderMaiPaid(JSON.parseObject(message.getBody(), QueueRtcGiftUnderMaiPaidMsg.class));
                    break;
                case RocketTag.PRIVATE_CHAT_GIFT_PAID:
                    // 私聊送礼物支付消息
                    normalMsgHandler.handlePrivateChatGiftPaid(JSON.parseObject(message.getBody(), QueueBaseGiftPaidMsg.class));
                    break;
                case RocketTag.VOICE_GIFT_PAID:
                    // 语音速配支付消息
                    normalMsgHandler.handleVoiceGiftPaid(JSON.parseObject(message.getBody(), QueueVoiceGiftPaidMsg.class));
                    break;
                case RocketTag.HEADLINE_LOTTERY_ACTIVITY_PARTICIPATE:
                    // 头条抽奖活动参与成功
                    normalMsgHandler.handleHeadlineLotteryActivityParticipate(JSON.parseObject(message.getBody(), QueueHeadlineLotteryActivityParticipateMsg.class));
                    break;
                case RocketTag.ACTIVITY_GIFTS_PAID:
                    normalMsgHandler.handleActivityGiftsPaid(JSON.parseObject(message.getBody(), QueueActivityGiftsPaidMsg.class));
                    break;
                case RocketTag.LOTTERY_GAME_STATISTIC:
//                    normalMsgHandler.handleLotteryGameStatistic(JSON.parseObject(message.getBody(), LotteryGameStatisticMsg.class));
                    break;
                case RocketTag.ACTIVITY_LOGING:
                    normalMsgHandler.handleActivityLogin(JSON.parseObject(message.getBody(), ActivityLoginUserMsg.class));
                    break;
                case RocketTag.USER_RECHARGE_FINISH:
                    normalMsgHandler.handleUserRechargeFinish(JSON.parseObject(message.getBody(), UserRechargeFinishMsg.class));
                    break;
                case RocketTag.SEND_GIFT_SNAPSHOT:
                    // 送礼快照消息
                    normalMsgHandler.handleSendGiftSnapshot(JSON.parseObject(message.getBody(), QueueSendGiftSnapShotMsg.class));
                    break;
                case RocketTag.LOTTERY_GAME_SEND_REWARD:
                    // 抽奖游戏发放奖品
                    sendRewardService.sendReward(JSON.parseObject(message.getBody(), QueueLotteryGameSendRewardMsg.class));
                    break;
                case RocketTag.FAMILY_USER_EXIT:
                    // 用户退出家族消息
                    normalMsgHandler.handleFamilyUserExit(JSON.parseObject(message.getBody(), QueueUserExitFamilyMsg.class));
                    break;
                default:
                    break;
            }
        } catch (Exception e) {
            log.error(String.format("消息处理失败-%s-%s", message.getTag(), JSON.parseObject(message.getBody(), JSONObject.class)), e);
            return Action.ReconsumeLater;
        }
        return Action.CommitMessage;
    }
}
