package com.starbuds.server.service.activity.util;

import cn.hutool.core.util.RandomUtil;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.HashMap;
import java.util.Map;
import java.util.TreeSet;

/**
 * 概率工具
 *
 * @author Lay
 * @date 2020/1/7
 */
public class ProbabilityUtil {
    /**
     * 计算保底概率
     *
     * @param firstRoundProb  首轮概率
     * @param missedRounds    连续未中奖轮数
     * @param guaranteedRound 保底轮次
     * @return
     */
    public static double getGuaranteedProbability(double firstRoundProb, int missedRounds, int guaranteedRound) {
        if (missedRounds == 1) {
            return firstRoundProb;
        }

        // 如果大于保底轮次 则直接返回最大概率
        if (missedRounds >= guaranteedRound) {
            return 1;
        }

        final BigDecimal K = new BigDecimal(String.valueOf(firstRoundProb));
        final BigDecimal N = BigDecimal.valueOf(missedRounds);
        final BigDecimal P = BigDecimal.valueOf(guaranteedRound);

        // 轮次概率
        BigDecimal roundProb = BigDecimal.valueOf(2).multiply(N.subtract(BigDecimal.ONE)).multiply(BigDecimal.ONE.divide(P, 2, RoundingMode.HALF_UP).subtract(K)).divide(P.subtract(BigDecimal.ONE), 2, RoundingMode.HALF_UP);
        // 计算概率
        return K.add(roundProb).divide(roundProb.add(BigDecimal.valueOf(2).divide(P, 2, RoundingMode.HALF_UP)).multiply(P.subtract(N).add(BigDecimal.ONE).divide(BigDecimal.valueOf(2), 2, RoundingMode.HALF_UP)), 2, RoundingMode.HALF_UP).doubleValue();
    }

    /**
     * 按照概率数组抽取下标
     *
     * @param probArr
     * @param enlarge
     * @return
     */
    public static int getRandomColumnIndex(double[] probArr, int enlarge) {
        // 数轴区间
        TreeSet<Integer> axisList = new TreeSet<>();
        Map<Integer, Integer> mapIndex = new HashMap<>();

        // 利用数轴原理设置概率值分布
        int probTotal = 0;
        for (int i = 0; i < probArr.length; i++) {
            if (probArr[i] == 0) {
                continue;
            }
            int axisNum = (new BigDecimal(String.valueOf(probArr[i]))).multiply(BigDecimal.valueOf(enlarge)).intValue();
            probTotal += axisNum;
            axisList.add(probTotal);
            mapIndex.put(probTotal, i);
        }

        if (probTotal > enlarge) {
            enlarge = probTotal;
        } else if (enlarge - probTotal > 0) {
            // 补齐空缺
            axisList.add(enlarge);
            mapIndex.put(enlarge, -1);
        }

        // 产生随机数
        int randomNum = RandomUtil.randomInt(1, enlarge + 1);

        // 遍历检查区间
        for (Integer axisNum : axisList) {
            if (randomNum <= axisNum) {
                return mapIndex.get(axisNum);
            }
        }
        return -1;
    }

    /**
     * 按照概率数组抽取下标
     *
     * @param probArr
     * @return
     */
    public static int getRandomColumnIndex(double[] probArr) {
        // 根据小数决定放大概率
        int maxBit = 0;
        for (double val : probArr) {
            if (val > 1) {
                throw new RuntimeException("概率不能大于1");
            }
            int bit = String.valueOf(val).length() - 2;
            if (bit > maxBit) {
                maxBit = bit;
            }
        }

        // 放大倍数
        int enlarge = (int) Math.pow(10d, maxBit);
        return getRandomColumnIndex(probArr, enlarge);
    }
}
