package com.starbuds.server.service.activity.impl;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.activity.LotteryGameRewardTypeEnum;
import com.starbuds.server.common.constant.cache.RedisActivityKeys;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.pojo.activity.bo.LotteryGameWinItemBO;
import com.starbuds.server.common.pojo.db.LotteryGameUserQualificationHisDO;
import com.starbuds.server.common.service.activity.LotteryGameService;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.service.activity.dao.LotteryGameUserQualificationHis4TestDAO;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.springframework.test.annotation.Rollback;
import org.springframework.transaction.annotation.Transactional;
import test.BaseTest;

import javax.annotation.Resource;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author aoe
 * @date 2021/11/26
 */
@Slf4j
public class LotteryGameServiceImplTest extends BaseTest {

    @Reference(version = "1.0.0")
    private LotteryGameService lotteryGameService;

    @Reference(version = "1.0.0")
    private RedisService redisService;

    @Resource
    private LotteryGameUserQualificationHis4TestDAO lotteryGameUserQualificationHis4TestDAO;

    private final long SNAP_ID = 1L;

    @Test
    void lottery() {
        String key = RedisActivityKeys.getLotteryGameWinItems(SNAP_ID);
        cleanWinItemsFromRedis(key);
        Assertions.assertTrue(saveWinItems2Redis(key));

        List<String> list = redisService.lRange(key, 0, -1);
        Assertions.assertNotNull(list);
        Assertions.assertEquals(60, list.size());

        List<LotteryGameWinItemBO> winItems = list.stream()
                .map(json -> lotteryGameService.lottery(SNAP_ID))
                .collect(Collectors.toList());
        // 数量来源：private LotteryGameConfigItemDO insertLotteryGameConfigItem(LotteryGameConfigDO config)
        Assertions.assertEquals(10, winItems.stream().filter(Objects::nonNull)
                .filter(item -> LotteryGameRewardTypeEnum.toEnum(item.getRewardType()) == LotteryGameRewardTypeEnum.GIFT)
                .count());
        Assertions.assertEquals(20, winItems.stream().filter(Objects::nonNull)
                .filter(item -> LotteryGameRewardTypeEnum.toEnum(item.getRewardType()) == LotteryGameRewardTypeEnum.PROPS)
                .count());
        Assertions.assertEquals(30, winItems.stream().filter(Objects::isNull).count());

        cleanWinItemsFromRedis(key);
    }

    @Test
    @Rollback
    @Transactional
    void existBySnapIdUserId() {
        long snapId = System.currentTimeMillis();
        long userId = 1L;
        LotteryGameUserQualificationHisDO po = new LotteryGameUserQualificationHisDO();
        po.setHisId(System.currentTimeMillis());
        po.setSnapId(snapId);
        po.setUserId(userId);
        po.setIsEffective(BoolIntVal.True);
        po.setCreateTime(System.currentTimeMillis());
        lotteryGameUserQualificationHis4TestDAO.insert(po);

        Assertions.assertEquals(true, lotteryGameService.existBySnapIdUserId(snapId, userId));
        Assertions.assertEquals(false, lotteryGameService.existBySnapIdUserId((snapId + 1), userId));
        Assertions.assertEquals(false, lotteryGameService.existBySnapIdUserId(snapId, (userId + 1)));
    }


    /**
     * 将中奖礼物放入Redis
     *
     * @return
     */
    private boolean saveWinItems2Redis(String key) {
        String jsonArrStr = "[{\"rewardId\":392612766810112,\"rewardNum\":2,\"rewardPrice\":5200,\"rewardType\":2},{\"rewardId\":392612766810112,\"rewardNum\":2,\"rewardPrice\":5200,\"rewardType\":2},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":392612766810112,\"rewardNum\":2,\"rewardPrice\":5200,\"rewardType\":2},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":449708946358272,\"rewardNum\":1,\"rewardPrice\":1000,\"rewardType\":1},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":392612766810112,\"rewardNum\":2,\"rewardPrice\":5200,\"rewardType\":2},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":392612766810112,\"rewardNum\":2,\"rewardPrice\":5200,\"rewardType\":2},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":449708946358272,\"rewardNum\":1,\"rewardPrice\":1000,\"rewardType\":1},{\"rewardId\":392612766810112,\"rewardNum\":2,\"rewardPrice\":5200,\"rewardType\":2},{\"rewardId\":449708946358272,\"rewardNum\":1,\"rewardPrice\":1000,\"rewardType\":1},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":392612766810112,\"rewardNum\":2,\"rewardPrice\":5200,\"rewardType\":2},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":392612766810112,\"rewardNum\":2,\"rewardPrice\":5200,\"rewardType\":2},{\"rewardId\":449708946358272,\"rewardNum\":1,\"rewardPrice\":1000,\"rewardType\":1},{\"rewardId\":392612766810112,\"rewardNum\":2,\"rewardPrice\":5200,\"rewardType\":2},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":392612766810112,\"rewardNum\":2,\"rewardPrice\":5200,\"rewardType\":2},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":392612766810112,\"rewardNum\":2,\"rewardPrice\":5200,\"rewardType\":2},{\"rewardId\":449708946358272,\"rewardNum\":1,\"rewardPrice\":1000,\"rewardType\":1},{\"rewardId\":392612766810112,\"rewardNum\":2,\"rewardPrice\":5200,\"rewardType\":2},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":392612766810112,\"rewardNum\":2,\"rewardPrice\":5200,\"rewardType\":2},{\"rewardId\":449708946358272,\"rewardNum\":1,\"rewardPrice\":1000,\"rewardType\":1},{\"rewardId\":392612766810112,\"rewardNum\":2,\"rewardPrice\":5200,\"rewardType\":2},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":392612766810112,\"rewardNum\":2,\"rewardPrice\":5200,\"rewardType\":2},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":449708946358272,\"rewardNum\":1,\"rewardPrice\":1000,\"rewardType\":1},{\"rewardId\":392612766810112,\"rewardNum\":2,\"rewardPrice\":5200,\"rewardType\":2},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":392612766810112,\"rewardNum\":2,\"rewardPrice\":5200,\"rewardType\":2},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":449708946358272,\"rewardNum\":1,\"rewardPrice\":1000,\"rewardType\":1},{\"rewardId\":449708946358272,\"rewardNum\":1,\"rewardPrice\":1000,\"rewardType\":1},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":392612766810112,\"rewardNum\":2,\"rewardPrice\":5200,\"rewardType\":2},{\"rewardId\":392612766810112,\"rewardNum\":2,\"rewardPrice\":5200,\"rewardType\":2},{\"rewardId\":449708946358272,\"rewardNum\":1,\"rewardPrice\":1000,\"rewardType\":1},{\"rewardId\":3,\"rewardNum\":3,\"rewardPrice\":0,\"rewardType\":0},{\"rewardId\":392612766810112,\"rewardNum\":2,\"rewardPrice\":5200,\"rewardType\":2}]";

        List<String> jsonItems = JSON.parseArray(jsonArrStr, String.class);
        return redisService.lLeftPushAll(key, jsonItems) > 0;
    }

    private boolean cleanWinItemsFromRedis(String key) {
        // 清理数据
        return redisService.delKey(key);
    }
}
