package com.starbuds.server.service.activity.impl;

import com.alibaba.fastjson.JSON;
import com.google.common.collect.ImmutableList;
import com.starbuds.server.common.constant.activity.LotteryGameCostTypeEnum;
import com.starbuds.server.common.constant.activity.LotteryGamePlayTypeEnum;
import com.starbuds.server.common.constant.activity.LotteryGameRewardTypeEnum;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.pojo.activity.bo.LotteryGameRewardBO;
import com.starbuds.server.common.pojo.activity.vo.LotteryGameSummary;
import com.starbuds.server.common.pojo.db.LotteryGameConfigDO;
import com.starbuds.server.common.pojo.db.LotteryGameConfigItemDO;
import com.starbuds.server.common.pojo.db.LotteryGameSnapshotDO;
import com.starbuds.server.common.pojo.db.LotteryGameTimerDO;
import com.starbuds.server.common.service.activity.LotteryGameSnapshotService;
import com.starbuds.server.common.utils.LogUtil;
import com.starbuds.server.service.activity.provider.LotteryGameConfigItemProvider;
import com.starbuds.server.service.activity.provider.LotteryGameConfigProvider;
import com.starbuds.server.service.activity.provider.LotteryGameSnapshotProvider;
import com.starbuds.server.service.activity.provider.LotteryGameTimerProvider;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.springframework.test.annotation.Rollback;
import org.springframework.transaction.annotation.Transactional;
import test.BaseTest;

import javax.annotation.Resource;
import java.math.BigDecimal;

@Slf4j
public class LotteryGameSnapshotServiceImplTest extends BaseTest {

    @Resource
    private LotteryGameConfigProvider lotteryGameConfigProvider;

    @Resource
    private LotteryGameConfigItemProvider lotteryGameConfigItemProvider;

    @Resource
    private LotteryGameTimerProvider lotteryGameTimerProvider;

    @Reference(version = "1.0.0")
    private LotteryGameSnapshotService lotteryGameSnapshotService;

    @Resource
    private LotteryGameSnapshotProvider lotteryGameSnapshotProvider;

    @Test
    @Rollback
    @Transactional
    public void createLotteryGameSnapshot() {
        LotteryGameConfigDO config = insertLotteryGameConfig();
        Assertions.assertNotNull(config);

        LotteryGameConfigItemDO item = insertLotteryGameConfigItem(config);
        Assertions.assertNotNull(item);

        LotteryGameTimerDO timer = insertLotteryGameTimer(config, item);
        Assertions.assertNotNull(item);

        LotteryGameSnapshotDO snap = lotteryGameSnapshotProvider.createLotteryGameSnapshot(timer.getTimerId());
        Assertions.assertNotNull(snap);

//        Long snapId = snap.getSnapId();
//        checkSummary(lotteryGameSnapshotService.getSummary(snapId));
//        checkSummary(lotteryGameSnapshotService.getSummaryByCache(snapId));
    }

    private final String itemName = "红包雨测试规则不要删";
    private final Integer costType = LotteryGameCostTypeEnum.NONE.getValue();
    private final String iconUrl = "http://media.qn.panygo.com/gift/icon/c504e9af4c0244bcb427aad46bc1ae88.png";
    private final BigDecimal clickRate = new BigDecimal("0.6");

    private LotteryGameConfigDO insertLotteryGameConfig() {
        LotteryGameConfigDO po = new LotteryGameConfigDO();
        po.setConfigName("红包雨测试不要删");
        po.setPlayType(LotteryGamePlayTypeEnum.RED_BAG.value);
        po.setNotice("红包雨将于#分钟#后到达，各就各位！");
        long now = System.currentTimeMillis();
        po.setCreateTime(now);
        po.setUpdateTime(now);
        po.setAdminId(26L);
        po.setAdminName("小白");
        po.setIsDeleted(BoolIntVal.False);
        return lotteryGameConfigProvider.create(po);
    }

    private LotteryGameConfigItemDO insertLotteryGameConfigItem(LotteryGameConfigDO config) {
        LotteryGameConfigItemDO po = new LotteryGameConfigItemDO();
        po.setConfigId(config.getConfigId());
        po.setItemName(itemName);
        po.setCostType(costType);
        po.setIconUrl(iconUrl);
        po.setCountDown(60L);
        po.setMinDurationSeconds(10L);
        po.setMaxDurationSeconds(60L);

        ImmutableList<LotteryGameRewardBO> rewards = ImmutableList.of(
                getReward(LotteryGameRewardTypeEnum.GIFT, 449708946358272L, 1, 10, 10000L),
                getReward(LotteryGameRewardTypeEnum.PROPS, 392612766810112L, 2, 20, 104000L),
                getReward(LotteryGameRewardTypeEnum.NONE, 3L, 3, 30, 0L)
        );
        po.setRewardPool(JSON.toJSONString(rewards));

        po.setClickRate(clickRate);
        po.setStatus(1);
        return lotteryGameConfigItemProvider.create(po);
    }

    private LotteryGameTimerDO insertLotteryGameTimer(LotteryGameConfigDO config, LotteryGameConfigItemDO item) {
        LotteryGameTimerDO po = new LotteryGameTimerDO();
        po.setLotteryNo("" + System.currentTimeMillis());
        po.setPlayType(config.getPlayType());
        po.setLotteryGameConfigItemId(item.getConfigItemId());
        long now = System.currentTimeMillis();
        po.setStartTime(now + (10 * 1000));
        po.setEndTime(now + (30 * 1000));
        po.setAdminId(26L);
        po.setAdminName("小白");
        return lotteryGameTimerProvider.create(po);
    }

    private LotteryGameRewardBO getReward(LotteryGameRewardTypeEnum type,
                                          Long rewardId, Integer rewardNum, Integer rewardStock, Long totalValue) {
        LotteryGameRewardBO reward = new LotteryGameRewardBO();
        reward.setRewardType(type.getValue());
        reward.setRewardId(rewardId);
        reward.setRewardNum(rewardNum);
        reward.setRewardStock(rewardStock);
        reward.setRewardTotalValue(totalValue);
        return reward;
    }

    private void checkSummary(LotteryGameSummary summary) {
        log.info(LogUtil.obj2JsonObj(summary));
        Assertions.assertNotNull(summary);

        Assertions.assertEquals(itemName, summary.getItemName());
        Assertions.assertEquals(costType, summary.getCostType());
        Assertions.assertNull(summary.getCostItem());
        Assertions.assertEquals(0, summary.getCostValue());
        Assertions.assertTrue(clickRate.compareTo(summary.getClickRate()) == 0);
        Assertions.assertNotNull(summary.getStartTime());
        Assertions.assertNotNull(summary.getRewards());
        Assertions.assertTrue(summary.getRewards().size() == 2);
    }
}