package com.starbuds.server.service.activity.lotterygame;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.activity.LotteryGamePlayTypeEnum;
import com.starbuds.server.common.constant.activity.LotteryGameRewardTypeEnum;
import com.starbuds.server.common.constant.cache.RedisActivityKeys;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rocketmq.RocketTopic;
import com.starbuds.server.common.pojo.activity.bo.LotteryGameWinItemBO;
import com.starbuds.server.common.pojo.mq.QueueLotteryGameEndMsg;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.service.activity.rocket.RocketProducer;
import org.apache.dubbo.config.annotation.Reference;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.springframework.test.util.ReflectionTestUtils;
import test.BaseTest;

import javax.annotation.Resource;
import java.util.List;

class MarkUpEndServiceImplTest extends BaseTest {

    @Resource
    private MarkUpEndServiceImpl markUpEndService;

    @Reference(version = "1.0.0")
    private RedisService redisService;

    @Resource
    private RocketProducer rocketProducer;

    private final long SNAP_ID = 1L;

    //    @Test
    void saveWinItems2Cache() {
        String key = RedisActivityKeys.getLotteryGameWinItems(SNAP_ID);
        // 清理数据
        redisService.delKey(key);

        boolean save = saveWinItems2Cache(SNAP_ID);
        Assertions.assertTrue(save);

        List<String> list = redisService.lRange(key, 0, -1);
        Assertions.assertNotNull(list);
        Assertions.assertEquals(60, list.size());

        list.stream()
                .map(json -> JSON.parseObject(json, LotteryGameWinItemBO.class))
                .forEach(item -> {
                    Assertions.assertNotNull(item.getRewardNum());
                    Assertions.assertNotNull(LotteryGameRewardTypeEnum.toEnum(item.getRewardType()));
                    Assertions.assertNotNull(item.getRewardId());

                    LotteryGameRewardTypeEnum type = LotteryGameRewardTypeEnum.toEnum(item.getRewardType());
                    switch (type) {
                        case GIFT:
                        case PROPS:
                            Assertions.assertTrue(item.getRewardPrice() > 0);
                            break;
                        case NONE:
                            Assertions.assertTrue(item.getRewardPrice() == 0);
                            break;
                    }
                });

        // 清理数据
        Assertions.assertTrue(redisService.delKey(key));
    }

    @Test
    void sendMsg() {
        QueueLotteryGameEndMsg lotteryGameEndMsg = new QueueLotteryGameEndMsg();
        lotteryGameEndMsg.setSnapId(1L);
        lotteryGameEndMsg.setPlayType(LotteryGamePlayTypeEnum.RED_BAG);
        boolean sendLotteryGameEndMsg = rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.LOTTERY_GAME_LOTTERY_END, lotteryGameEndMsg.serialize());
        Assertions.assertTrue(sendLotteryGameEndMsg);
    }

    private boolean saveWinItems2Cache(long snapId) {
        return ReflectionTestUtils.invokeMethod(markUpEndService, "saveWinItems2Cache", snapId);
    }
}